package CodeGen;

/**
 * This type was created in VisualAge.
 *
 * Version 0.1 - creates the agent class shells, datatype shells, and 
 *		conversation classes for and agentTool Agent Diagram and its
 *		associated Conversation Diagrams.  It does not fill in any of
 *		the Agent class information outside of the default "write"
 *		method.
 * Version 0.2 - adds multiple actions in states (not transitions) and
 *      fixes a few cosmetic bugs.
 * Version 0.3 - basic bug fixes discovered by 623 class May 2000. 
 * Version 0.4 - handles tuple values on actions - June 2000.
 *             - states requiring nonblocking reads to read OR send a message 
 *
 * Not yet implemented:
 *  1. actions on transitions
 *     -- to include special "return" action
  *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import system.*;
import java.util.*;
import java.io.*;
public class AgentMom
{
	String version = " - agentMom transform ver 0.4";
		String outDirName = "C:/Windows/Desktop/Test"; 
		Vector combinedTypes;
		Vector initialVariables;
		Vector localVariables;
		Vector methods;
		boolean debug = false;
		MessageGUI gui;
/**
 * AgentMom constructor comment.
 */
public AgentMom() {
	super();
}
/**
 * This method was created in VisualAge.
 * @param out java.io.PrintWriter
 * @param stInit afit.system.ATstatetable
 */
private void addCombinedParams(PrintWriter out, String[] tokens) {
	// isVoid denotes whether or not the first parameter is a variable to which
	// the result of an assignment is made (e.g., x = funct(param1, param2, ...))
	// or a void function call (e.g., funct(param1, param2, ...))

	String className = "";
	String objectName = "";
	int i = 0;
	int firstParam = 1;
	boolean returnValue = false;
	int eqLoc = -1;

	/* determine if there is a return value by finding
	 * the location of the = sign, reset the location of
	 * the first parameter to the token beyond the = sign */
	for (int k = 0; k < tokens.length; k++)
		if (tokens[k].equals("=")) {
			returnValue = true;
			eqLoc = k;
			firstParam = k + 1;
		}

		/* loop through transitions */
	if (tokens.length > firstParam + 1) {
		/* get className and strip off "_Type" at end */
		className = getParameterClassName(tokens);
		if (!className.equals("")) {
			objectName = className.substring(0, className.length() - 5);

			if ((initialVariables.indexOf(objectName) < 0) && (localVariables.indexOf(objectName) < 0)) {
				combinedTypes.addElement(objectName);
				localVariables.addElement(objectName);
				out.println("\t" + className + " " + objectName + " = new " + className + "();");
				createParameterClass(tokens);
			}
		}
	}
}
/**
 * This method was created in VisualAge.
 * @param out java.io.PrintWriter
 * @param stInit int
 */
private void addStateConstants(PrintWriter out, ATstatetable stInit)
{
	int i = 1;
	String stateName = "";
	Vector v = new Vector(3);

	/* print definitions */
	out.println("\n\t/* state constant definitions */");
	out.println("\tfinal int StartState = 0;");

	// loop through states
	Enumeration stateEnum = stInit.returnStates();
	while (stateEnum.hasMoreElements())
	{
		try
		{
			stateName = ((ATstate) stateEnum.nextElement()).getName();
			if (v.indexOf(stateName) < 0)
			{
				v.addElement(stateName);
				out.println("\tfinal int " + stateName + " = " + i++ +";");
			}
		} catch (ClassCastException e)
		{
			// skip start and end states ???
		}
	}

	// loop through transitions
	Enumeration transEnum = stInit.returnTrans();
	while (transEnum.hasMoreElements())
	{
		try
		{
			stateName = ((ATtransition) transEnum.nextElement()).getCurrentState().getName() + "_out";
			if (v.indexOf(stateName) < 0)
			{
				v.addElement(stateName);
				out.println("	final int " + stateName + " = " + i++ +";");
			}
		} catch (ClassCastException e)
		{
			// skip start and end states ???
		}
	} //
}
/**
 * This method was created in VisualAge.
 * @param out java.io.PrintWriter
 * @param stInit afit.system.ATstatetable
 */
private void addVariableDeclarations(PrintWriter out, String[] tokens) {
	int firstParam = getFirstParamLocation(tokens);

	/* print variable declarations if they have not been declared before */

	for (int i = firstParam; i < tokens.length; i++) {
		String var = ((tokens[i].replace('<', ' ')).replace('>', ' ')).trim();
		if ((initialVariables.indexOf(var) < 0) && (localVariables.indexOf(var) < 0))
			if (!var.equals("=")) {
				out.println("\t" + "Object " + var + " = null;");
				localVariables.addElement(var);
			}
			/* if we found an = sign, skip adding the next token since it is an actin name */
			else
				i++;
	}
}
/**
 * This method was created in VisualAge.
 */
public void combineReceiveParameters(PrintWriter out, String[] receiveTokens, int depth)
{
	int i, j;
	String current;
	String paramName = "";

	/* compute parameter name */
	if (receiveTokens.length == 2)
		paramName = receiveTokens[1];
	else
		if (receiveTokens.length > 2)
		{
			for (i = 1; i < receiveTokens.length - 1; i++)
				paramName = paramName + receiveTokens[i] + "_";
			paramName = paramName + receiveTokens[receiveTokens.length - 1];
		}

		/* define statement */
	if (receiveTokens.length > 1)
	{
		for (i = 1; i <= depth; i++)
			out.print("\t");
		if (receiveTokens.length == 2)
		{
			out.println(paramName + " = m.content;");
		}
		if (receiveTokens.length > 2)
		{
			out.println(paramName + " = (" + paramName + "_Type) " + "m.content;");
			for (i = 1; i < receiveTokens.length; i++)
			{
				for (j = 1; j <= depth; j++)
					out.print("\t");
				current = receiveTokens[i];
				out.println(current + " = " + paramName + "." + current + ";");
			}
		}
	}
}
/**
 * This method was created in VisualAge.
 */
public void combineSendParameters(PrintWriter out, String[] actionTokens, int depth) {
	int i, j;
	String current;

	// if there are multiple parameters
	if (actionTokens.length > 2) {
		for (i = 1; i < actionTokens.length; i++) {
			for (j = 1; j <= depth; j++)
				out.print("\t");
			current = actionTokens[i];
			for (j = 1; j < actionTokens.length - 1; j++)
				out.print(actionTokens[j] + "_");
			out.print(actionTokens[actionTokens.length - 1] + "." + current);
			out.println(" = " + current + ";");
		}
		for (i = 1; i <= depth; i++)
			out.print("\t");
		out.print("m.content = ");
		for (i = 1; i < actionTokens.length - 1; i++)
			out.print(actionTokens[i] + "_");
		out.println(actionTokens[actionTokens.length - 1] + ";");
	}

	// if there is only one parameter
	if (actionTokens.length == 2) {
		for (i = 1; i <= depth; i++)
			out.print("\t");
		out.print("m.content = ");
		for (i = 1; i < actionTokens.length - 1; i++)
			out.print(actionTokens[i] + "_");
		out.println(actionTokens[actionTokens.length - 1] + ";");
	}
}
/**
 * This method was created in VisualAge.
 * @param tokens java.lang.String[]
 */
private String createParameterClass(String[] tokens)
{
	int i;
	int firstParam = getFirstParamLocation(tokens);
	String className = getParameterClassName(tokens);

	/* open the file */
	try
	{
		File outFile = new File(outDirName, className + ".java");
		FileOutputStream outFileStream = new FileOutputStream(outFile);
		PrintWriter out = new PrintWriter(outFileStream);

		/* print file header info */
		out.println("/*");
		out.println("*");
		out.println("* Data Type class: " + className + ".");
		out.println("*");
		out.println("* automatically generated by agentTool" + version);
		out.println("*");
		out.println("*/");
		out.println("import afit.mom.*;");

		// print class info
		out.println("public class " + className);
		out.println("{");
		for (i = firstParam; i < tokens.length; i++)
			out.println("	Object " + tokens[i] + ";");
		out.println("}");
		out.close();
	} catch (java.io.IOException e)
	{
		gui.write(" * * Error trying to open file " + outDirName + className + ".java ");
	}
	return className;
}
/**
 * This method was created in VisualAge.
 * @param a afit.system.ATagent
 */
private void genAgentCode(ATagent a, ATsystem sys) {
	String className = a.getName();
	try {
		gui.write("generating code for agent " + className);
		File outFile = new File(outDirName, className + ".java");
		FileOutputStream outFileStream = new FileOutputStream(outFile);
		PrintWriter out = new PrintWriter(outFileStream);

		/* class definition */
		// header
		out.println("/*");
		out.println("*");
		out.println("* Class for agent " + className);
		out.println("*");
		out.println("* automatically generated by agentTool" + version);
		out.println("*");
		out.println("*/");

		// import statements
		out.println("import java.net.*;");
		out.println("import java.io.*;");
		out.println("import java.awt.*;");
		out.println("import afit.mom.*;");

		// actual definiton
		out.println("public class " + className + " extends Agent");
		out.println("{");

		/* default constructor */
		out.println("\n/*");
		out.println("*");
		out.println("* Constructor for agent.");
		out.println("* ");
		out.println("* automatically generated by agentTool" + version);
		out.println("* ");
		out.println("*/");
		out.println("public " + className + " (String n, int p) ");
		out.println("{ ");
		out.println("\tsuper(n, p);");
		out.println("}");

		/* Run method */
		out.println("\n/*");
		out.println("*");
		out.println("* Run method for agent.  This must be completed by user.");
		out.println("* ");
		out.println("* automatically generated by agentTool" + version);
		out.println("* ");
		out.println("*/");
		out.println("\npublic void run() ");
		out.println("{");
		out.println("}");

		/* Write method */
		out.println("\n/*");
		out.println("*");
		out.println("* Write method - used to write conversation error messages to output.");
		out.println("* ");
		out.println("* automatically generated by agentTool" + version);
		out.println("* ");
		out.println("*/");
		out.println("\npublic void write(String s) ");
		out.println("{");
		out.println("\tSystem.out.println(s);");
		out.println("}");

		/* ReceiveMessage method - required for agentMom */
		out.println("\n/*");
		out.println("*");
		out.println("* Required method for agentMom framework - must be completed by user.");
		out.println("* ");
		out.println("* automatically generated by agentTool" + version);
		out.println("* ");
		out.println("*/ ");
		out.println("\npublic void receiveMessage(Socket server, ObjectInputStream input, ObjectOutputStream output) ");
		out.println("{");
		out.println("}");
		if (debug)
			gui.write("Looking for methods referenced in conversations.");

		/* find methods referenced in conversation that must be implmented in the agent */

		/* keep track so we don't duplicate method names */
		methods = new Vector(5);
		methods.addElement("run");
		methods.addElement("receiveMessage");
		methods.addElement("write");

		/* loop through each conversation */
		Enumeration convsEnum = sys.returnConvs();
		while (convsEnum.hasMoreElements()) {
			ATconversation c = (ATconversation) convsEnum.nextElement();
			for (int initResp = 1; initResp <= 2; initResp++) {
				ATstatetable stInit = c.getStateTable(initResp);

				/* if this conversation is for this agent */
				if (className.equals(stInit.getParticipant().getName())) {
					// loop through states ... and check actions
					Enumeration enum = stInit.returnStates();
					while (enum.hasMoreElements()) {
						try {
							ATstate s = (ATstate) enum.nextElement();
							Vector v = (Vector) s.getActions();
							Enumeration actions = v.elements();
							while (actions.hasMoreElements()) {
								String action = (String) actions.nextElement();
								if (!action.equals("")) {
									if (debug)
										gui.write("Generating method for " + action);
									String[] tokens = parseAction2(action);
									if (debug)
										for (int j = 0; j < tokens.length; j++)
											gui.write(" *** Token[" + j + "] = " + tokens[j]);

										// do we have to assign a value to the return value
									boolean returnValue = false;
									int eqLoc = 0;
									for (int k = 0; k < tokens.length; k++)
										if (tokens[k].equals("=")) {
											returnValue = true;
											eqLoc = k;
										}

										// a return value exists
									if (returnValue) {
										// the default return type is Object
										String typeName = "Object";

										// are we returning multiple values (i.e., <x, y> = doit(...)).
										if (eqLoc > 1) {
											String outTokens[] = new String[eqLoc + 1];
											// start at 1 since the first token is skipped in createParameterClass
											for (int l = 1; l < eqLoc + 1; l++) {
												outTokens[l] = ((tokens[l - 1].replace('<', ' ')).replace('>', ' ')).trim();
												if (debug)
													gui.write("outTokens[" + l + "] = |" + outTokens[l] + "|");
											}
											typeName = createParameterClass(outTokens);
										}

										// check for reserved words in action name
										if (tokens[eqLoc + 1].equals("run") || tokens[eqLoc + 1].equals("receiveMessage") || tokens[eqLoc + 1].equals("write")) {
											gui.writeError("Action " + tokens[eqLoc + 1] + " is reserved name for agentMom agent class!");
										}

										/* check to make sure we haven't already added a method by that name */
										else
											if (methods.indexOf(tokens[eqLoc + 1]) >= 0) {
												gui.write("* Warning * Action " + tokens[eqLoc + 1] + " has already been defined!");
											}

											/* add the new method */
											else {
												methods.addElement(tokens[eqLoc+1]);
												out.println("\n/*");
												out.println("*");
												out.println("* Method " + tokens[eqLoc + 1] + " referenced in conversation - must be completed by user.");
												out.println("* ");
												out.println("* automatically generated by agentTool" + version);
												out.println("* ");
												out.println("*/ ");
												if (tokens.length == eqLoc + 2)
													out.println("\npublic " + typeName + " " + tokens[eqLoc + 1] + " ()");
												else {
													out.print("\npublic " + typeName + " " + tokens[eqLoc + 1] + " (");
													for (int j = eqLoc + 2; j < tokens.length; j++)
														if (j == eqLoc + 2) {
															out.print("Object " + tokens[j]);
															if (tokens.length == eqLoc + 3)
																out.println(")");
														} else
															if (j == tokens.length - 1)
																out.println(", Object " + tokens[j] + ") ");
															else
																out.print(", Object " + tokens[j]);
												}
												out.println("{");
												out.println("\treturn null;");
												out.println("}");
											}
									}

									// void method
									if (!returnValue)
										if (methods.indexOf(tokens[0]) >= 0) {
											gui.write("* Warning * Action " + tokens[0] + " has already been defined!");
										}

										/* build method */
										else {
											methods.addElement(tokens[0]);
											out.println("\n/*");
											out.println("*");
											out.println("* Method " + tokens[0] + " referenced in conversation - must be completed by user.");
											out.println("* ");
											out.println("* automatically generated by agentTool" + version);
											out.println("* ");
											out.println("*/ ");
											if (tokens.length == 1)
												out.println("\npublic void " + tokens[0] + " ()");
											else {
												out.print("\npublic void " + tokens[0] + " (");
												for (int j = 1; j < tokens.length; j++)
													if (j == 1) {
														out.print("Object " + tokens[j]);
														if (tokens.length == 2)
															out.println(")");
													} else
														if (j == tokens.length - 1)
															out.println(", Object " + tokens[j] + ") ");
														else
															out.print(", Object " + tokens[j]);
											}
											out.println("{");
											out.println("}");
										}
									if (debug)
										gui.write("Done generating method for " + action);
								}
							}
						} catch (ClassCastException e) {
							// skip start and end states
						}
					}
				}
				if (debug)
					gui.write("Done with conversation " + c.getName() + " - VALID FOR THIS AGENT!");
			}
			if (debug)
				gui.write("Done with conversation " + c.getName());
		}
		if (debug)
			gui.write("Done looking at all conversations ");

		/* end of agent definition */
		out.println("\n}");
		out.close();
		if (debug)
			gui.write("Done with agent ... ");
	} catch (java.io.IOException e) {
		gui.write("** Error trying to open file " + outDirName + className + ".java ");
	}
}
/**
 * This method was created in VisualAge.
 */
public void genConversations(ATsystem sys) {
	ATagent a;
	ATconversation c;
	String outFileName = "";
	int i = 0;
	String[] param = null;
	ATtransition startTrans = null;
	ATtransition t;
	String stateName = "";
	String convClassName = "";
	gui = new MessageGUI("agentTool Code Generation ");
	gui.write("Starting code generation " + version);
	outDirName = gui.getSaveDirectory(outDirName);
	gui.write("Code generation directory is " + outDirName);

	/* start */
	Enumeration agentsEnum = sys.returnAgents();
	while (agentsEnum.hasMoreElements())
		genAgentCode((ATagent) agentsEnum.nextElement(), sys);

	/* enumerate each conversation */
	Enumeration convsEnum = sys.returnConvs();
	while (convsEnum.hasMoreElements()) {
		c = (ATconversation) convsEnum.nextElement();

		/* Generate code for Initiator & Respondent classes */
		for (int initResp = 1; initResp <= 2; initResp++) {
			/* initialize globals for each conversation side */
			combinedTypes = new Vector(3);
			initialVariables = new Vector(1);
			ATstatetable stInit = c.getStateTable(initResp);


			// create file for this half of the conversation
			try {
				/* define class specific names */
				String agentName = stInit.getParticipant().getName();
				String convName = c.getName();
				if (initResp == 1)
					convClassName = convName + "_" + agentName + "_I";
				else
					convClassName = convName + "_" + agentName + "_R";
				gui.write("generating code for conversation " + convClassName);
				File outFile = new File(outDirName, convClassName + ".java");
				FileOutputStream outFileStream = new FileOutputStream(outFile);
				PrintWriter out = new PrintWriter(outFileStream); //

				/* print file header info */
				out.println("/*");
				out.println("*");
				out.println("* Conversation class for " + convClassName + ".");
				out.println("*");
				out.println("* automatically generated by agentTool" + version);
				out.println("*");
				out.println("*/");
				out.println("import java.net.*;");
				out.println("import java.io.*;");
				out.println("import afit.mom.*;");
				out.println("public class " + convClassName + " extends Conversation");
				out.println("{");
				out.println("	" + agentName + " parent; // override parent definition in Agent");

				// find data types sent on 1st message
				Enumeration transEnum = stInit.returnTrans();
				while (transEnum.hasMoreElements()) {
					try {
						t = (ATtransition) transEnum.nextElement();
						if (t.getCurrentState() instanceof ATstartState)
							startTrans = t;
					} catch (ClassCastException e) {
					}
				} //

				/* print out objects needed to hold initial message contents */
				param = parseInitialObjects(startTrans.getTransMessage());
				for (i = 0; i < param.length; i++) {
					out.println("	Object " + param[i] + ";");
					initialVariables.addElement(param[i]);
				}

				/* generate constuctor */
				/* initiator */
				if (initResp == 1) {
					out.println("\n/*");
					out.println("*");
					out.println("* Constructor for  " + convClassName + ".");
					out.println("*");
					out.println("* automatically generated by agentTool" + version);
					out.println("*");
					out.println("*/");
					out.print("public " + convClassName + "(" + agentName + " a, String hostName, int portNum");
					for (i = 0; i < param.length; i++)
						out.print(", Object " + param[i] + "_in");
					out.println(")");

					// constructor body
					out.println("{");
					out.println("	super(a, hostName, portNum);");
					out.println("	parent = a;");

					// declare input variables
					out.println("\n	// save input variables to class attributes");
					for (i = 0; i < param.length; i++)
						out.println("	" + param[i] + " = " + param[i] + "_in ;");

					// open conncection, etc.
					out.println("\n	// open a new socket connection");
					out.println("	try {");
					out.println("	  connection = new Socket(hostName, portNum);");
					out.println("	  output = new ObjectOutputStream(connection.getOutputStream());");
					out.println("	  output.flush();");
					out.println("	  input = new ObjectInputStream(connection.getInputStream());");
					out.println("	} catch (IOException e)");
					out.println("	{");
					out.println("		parent.write(\"** IO exception in " + convClassName + ".\");");
					out.println("	} ");
					out.println("}");
				}

				/* responder */
				else {
					out.println("\n/*");
					out.println("*");
					out.println("* Constructor for  " + convClassName + ".");
					out.println("*");
					out.println("* automatically generated by agentTool" + version);
					out.println("*");
					out.println("*/");
					out.print("public " + convClassName + "(Socket s, ObjectInputStream i, ObjectOutputStream o, " + agentName + " a, Message m1)");

					// constructor body
					out.println("{");
					out.println("	super(s, i, o, a, m1);");
					out.println("	parent = a;");
					out.println("}");
				}


				/* generate RUN method */
				localVariables = new Vector(3);
				printRunOpening(out, stInit, initResp);

				// RUN - conversation body 
				printConvBody(out, stInit, initResp);

				// RUN - closing code template
				printRunClosing(out, convClassName);
				out.println("} /* end class */");
				out.close(); //

			} catch (java.io.IOException e) {
				gui.write("*** ERROR - cannot save to file:" + outDirName + outFileName);
				e.printStackTrace(System.out);
				return;
			}
		}
	}
	gui.write("Complete code generation.");
}
/**
 * This method was created in VisualAge.
 */
private void genConvState(PrintWriter out, ATstate state, ATstatetable stInit, int initResp) {
	ATtransition t;
	boolean receive;

	// print out simple state case
	out.println("\t\t\t\tcase " + state.getName() + " :");

	/* define actions in the state  */
	Vector v = (Vector) state.getActions();
	Enumeration actions = v.elements();
	while (actions.hasMoreElements()) {
		String[] vars = {};
		String var = "";
		String a = (String) actions.nextElement();
		if (!a.equals("")) {
			int eqLoc = a.indexOf('=');
			if (eqLoc >= 0) {
				out.print("\t\t\t\t\t");
				vars = parseTuple(a.substring(0, eqLoc));
				var = vars[0];
				for (int j = 1; j < vars.length; j++)
					var = var + "_" + vars[j];
				out.print(var);
				out.println(" = parent." + a.substring(eqLoc + 1, a.length()).trim() + ";");
			} else
				out.println("\t\t\t\t\tparent." + a + ";");
		}
		
		/* now decompose tuple into its parts */
		if (vars.length > 1)
			for (int j = 0; j < vars.length; j++)
				out.println("\t\t\t\t\t" + vars[j] + " = " + var + "." + vars[j] + ";");
	}



	/* goto next state */
	out.println("\t\t\t\t\tstate = " + state.getName() + "_out;");
	out.println("\t\t\t\t\tbreak;");


	/* begin case */
	out.println("\t\t\t\tcase " + state.getName() + "_out :");

	// step through transitions
	Enumeration te = stInit.returnTrans();
	boolean transmit = false;
	boolean received = false;
	while (te.hasMoreElements()) {
		t = (ATtransition) te.nextElement();
		if ((t.getCurrentState().getName()).equals(state.getName()))
		{
			if (!t.getTransMessage().equals("")) transmit = true;
			if (!t.getRecMessage().equals("")) received = true;
			
		}
	}
	boolean complexState = transmit && received;
	
	// step through transitions
	Enumeration transEnum = stInit.returnTrans();
	receive = false;
	while (transEnum.hasMoreElements()) {
		t = (ATtransition) transEnum.nextElement();
		if ((t.getCurrentState().getName()).equals(state.getName()))
			receive = genConvTrans(out, t, receive, initResp, complexState);
	}
	if (complexState)
	out.println("\t\t\t\t\tstate = "+state.getName()+";");
	out.println("\t\t\t\t\tbreak; ");
}
/**
 * This method was created in VisualAge.
 */
private boolean genConvTrans(PrintWriter out, ATtransition t, boolean receive, int initResp, boolean complexState) {
	String guard = t.getGuardCondition();
	guard = OCLtoJava(guard);
	if (debug) {
		//		gui.write("For transition " + t.getRecMessage() + "[" + guard + "]" + "/" + t.getActions() + "^" + t.getTransMessage());
		gui.write("    t.getTransMessage().equals(\"\") = " + !t.getTransMessage().equals(""));
		gui.write("  guard.equals(\"\") = " + guard.equals(""));
		gui.write("      t.getRecMessage().equals(\"\") = " + t.getRecMessage().equals(""));
	}

	// turn this off temporarily	
	//	if (t.getActions() != null)
	//		if (!t.getActions().equals("")) {
	//			gui.writeError("transition actions not supported by this transformation!");
	//			gui.write("\tfound transition " + t.getRecMessage() + " [" + guard + "] / " + t.getActions() + " ^ " + t.getTransMessage());
	//		}

	/* xmit - no guard - no receive */
	if ((!t.getTransMessage().equals("")) && (guard.equals("")) && (t.getRecMessage().equals(""))) {
		String[] actionTokens = parseAction(t.getTransMessage());
		if (actionTokens.length >= 0) {
			out.println("\t\t\t\t\tm = new Message();");
			out.println("\t\t\t\t\tm.performative = \"" + actionTokens[0] + "\";");
			combineSendParameters(out, actionTokens, 5);
			out.println("\t\t\t\t\tsendMessage(m, output);");
			if (t.getNextState() instanceof ATendState)
				out.println("\t\t\t\t\t\tnotDone = false;");
			else
				out.println("\t\t\t\t\tstate = " + t.getNextState().getName() + ";");
		}
	}
	/* xmit - guard - no receive */
	if ((!t.getTransMessage().equals("")) && (!guard.equals("")) && (t.getRecMessage().equals(""))) {
		String[] actionTokens = parseAction(t.getTransMessage());
		if (actionTokens.length >= 0) {
			out.println("\t\t\t\t\tif (" + guard + ")");
			out.println("\t\t\t\t\t{");
			out.println("\t\t\t\t\t\tm = new Message();");
			out.println("\t\t\t\t\t\tm.performative = \"" + actionTokens[0] + "\";");
			combineSendParameters(out, actionTokens, 6);
			out.println("\t\t\t\t\t\tsendMessage(m, output);");
			if (t.getNextState() instanceof ATendState)
				out.println("\t\t\t\t\t\tnotDone = false;");
			else
				out.println("\t\t\t\t\t\tstate = " + t.getNextState().getName() + ";");
			out.println("\t\t\t\t\t}");
		}
	}
	/* no xmit - no guard - no receive */
	if ((t.getTransMessage().equals("")) && (guard.equals("")) && (t.getRecMessage().equals(""))) {
		if (t.getNextState() instanceof ATendState)
			out.println("\t\t\t\t\tnotDone = false;");
		else
			out.println("\t\t\t\t\tstate = " + t.getNextState().getName() + ";");
	}
	/* no xmit - guard - no receive */
	if ((t.getTransMessage().equals("")) && (!guard.equals("")) && (t.getRecMessage().equals(""))) {
		out.println("\t\t\t\t\tif (" + guard + ")");
		if (t.getNextState() instanceof ATendState)
			out.println("\t\t\t\t\tnotDone = false;");
		else
			out.println("\t\t\t\t\tstate = " + t.getNextState().getName() + ";");
	}

	/* xmit - guard - receive */
	if ((!t.getTransMessage().equals("")) && (!guard.equals("")) && (!t.getRecMessage().equals(""))) {
		String[] actionTokens = parseAction(t.getTransMessage());
		if (actionTokens.length >= 0) {
			if (!receive && (initResp == 0 || !(t.getCurrentState() instanceof ATstartState)))
				if (complexState)
					out.println("\t\t\t\t\tm = nonblockedReadMessage(input);");
				else
					out.println("\t\t\t\t\tm = readMessage(input);");
			String[] receiveTokens = parseAction(t.getRecMessage());
			if (actionTokens.length >= 0) {
				out.println("\t\t\t\t\tif (m.performative.equals(\"" + receiveTokens[0] + "\") && (" + guard + "))");
				out.println("\t\t\t\t\t{");
				combineReceiveParameters(out, receiveTokens, 6);
				out.println("\t\t\t\t\t\tm = new Message();");
				out.println("\t\t\t\t\t\tm.performative = \"" + actionTokens[0] + "\";");
				combineSendParameters(out, actionTokens, 6);
				out.println("\t\t\t\t\t\tsendMessage(m, output);");
				if (t.getNextState() instanceof ATendState)
					out.println("\t\t\t\t\t\tnotDone = false;");
				else
					out.println("\t\t\t\t\t\tstate = " + t.getNextState().getName() + ";");
				out.println("\t\t\t\t\t}");
			}
		}
	}
	/* no xmit - no guard - receive */
	if ((t.getTransMessage().equals("")) && (guard.equals("")) && (!t.getRecMessage().equals(""))) {
		if (!receive && (initResp == 1 || !(t.getCurrentState() instanceof ATstartState)))
			if (complexState)
					out.println("\t\t\t\t\tm = nonblockedReadMessage(input);");
				else
					out.println("\t\t\t\t\tm = readMessage(input);");
		String[] receiveTokens = parseAction(t.getRecMessage());
		if (receiveTokens.length >= 0) {
			out.println("\t\t\t\t\tif (m.performative.equals(\"" + receiveTokens[0] + "\"))");
			out.println("\t\t\t\t\t{");
			combineReceiveParameters(out, receiveTokens, 6);
			if (t.getNextState() instanceof ATendState)
				out.println("\t\t\t\t\t\tnotDone = false;");
			else
				out.println("\t\t\t\t\t\tstate = " + t.getNextState().getName() + ";");
			out.println("\t\t\t\t\t}");
		}
	}
	/* no xmit - guard - receive */
	if ((t.getTransMessage().equals("")) && (!guard.equals("")) && (!t.getRecMessage().equals(""))) {
		if (!receive && (initResp == 1 || !(t.getCurrentState() instanceof ATstartState)))
			if (complexState)
					out.println("\t\t\t\t\tm = nonblockedReadMessage(input);");
				else
					out.println("\t\t\t\t\tm = readMessage(input);");
		String[] receiveTokens = parseAction(t.getRecMessage());
		if (receiveTokens.length >= 0) {
			out.println("\t\t\t\t\tif (m.performative.equals(\"" + receiveTokens[0] + "\") && (" + guard + "))");
			out.println("\t\t\t\t\t{");
			combineReceiveParameters(out, receiveTokens, 6);
			if (t.getNextState() instanceof ATendState)
				out.println("\t\t\t\t\t\tnotDone = false;");
			else
				out.println("\t\t\t\t\t\tstate = " + t.getNextState().getName() + ";");
			out.println("\t\t\t\t\t}");
		}
	}
	if (t.getTransMessage().equals(""))
		receive = true;
	return receive;
}
/**
 * This method was created in VisualAge.
 * @param tokens java.lang.String[]
 *
 * tokens can represent either of the following strings:
 *  <x, ... y> = action(p1, .. pn)
 *  x = action(p1, .. pn)
 *  x = action()
 *  action(p1, .. pn)
 */

private int getFirstParamLocation(String[] tokens) {
	int firstParam = 1;

	/* find end of output tuples (<x, y, z> = action(param1 ... paramn)) 
	   ... if they exist */

	if (tokens.length > 1) {
		if (tokens[0] != null) {
			if (tokens[0].startsWith("<")) {
				for (int i = 1; i < tokens.length; i++)
					if (tokens[i].endsWith(">"))
						if (tokens[i + 1].equals("="))
							firstParam = i + 3;
						else
							firstParam = i + 2;
			}

			/* check for x = action(p1 .. pn) */
			else
				if (tokens[1].equals("="))
					firstParam = 3;
		}
	}
	return firstParam;
}
/**
 * This method was created in VisualAge.
 * @param tokens java.lang.String[]
 */
private String getParameterClassName(String[] tokens) {
	int i;
	String className = "";
	int firstParam = getFirstParamLocation(tokens);

	/* print tokens if in debug mode */
	if (debug) {
		for (i = 0; i < tokens.length; i++)
			gui.write("Token[" + i + "] = " + tokens[i]);
		gui.write("firstParam = " + firstParam);
	}

	/* determine name */
	if (tokens.length > firstParam + 1) {
		for (i = firstParam; i < tokens.length - 1; i++)
			className = className + tokens[i] + "_";
		className = className + tokens[tokens.length - 1] + "_Type";
	}
	return className;
}
/**
 * Insert the method's description here.
 * Creation date: (6/6/00 3:54:27 PM)
 * @return java.lang.String
 */
public String OCLtoJava(String s)
{
	int i;
	s = s.trim();
	while ((i = s.indexOf(" AND ")) >= 0)
	{
		s = new String(s.substring(0, i - 1) + " && " + s.substring(i + 5));
	}
	while ((i = s.indexOf(" OR ")) >= 0)
	{
		s = new String(s.substring(0, i - 1) + " || " + s.substring(i + 4));
	}
	while ((i = s.indexOf("NOT")) >= 0)
	{
		if (i == 0)
			s = new String("!" + s.substring(i + 3));
		else
			s = new String(s.substring(0, i - 1) + "!" + s.substring(i + 3));
	}
	return s;
}
/**
 * This method was created in VisualAge.
 *
 * This method parses with = as a delimter.
 */
private String[] parseAction(String s)
{
	String delimiters = "\n\t\r(,)= ";
	StringTokenizer tokens = new StringTokenizer(s, delimiters);
	if (tokens.countTokens() > 0)
	{
		if (debug) gui.write("Parsing the string |" + s + "|");
		String x[] = new String[tokens.countTokens()];
		int i = 0;
		while (tokens.hasMoreTokens())
		{
			x[i++] = tokens.nextToken();
			if (debug) gui.write("     Token " + (i-1) + " = " + x[i-1]);
		}

		/* return the vector of elements */
		return x;
	} else
		return new String[0];
}
/**
 * This method was created in VisualAge.
 *
 * This method parses WITHOUT using = as a delimter.
 */
private String[] parseAction2(String s)
{
	String delimiters = "\n\t\r(,) ";
	StringTokenizer tokens = new StringTokenizer(s, delimiters);
	if (tokens.countTokens() > 0)
	{
		if (debug) gui.write("Parsing the string |" + s + "|");
		String x[] = new String[tokens.countTokens()];
		int i = 0;
		while (tokens.hasMoreTokens())
		{
			x[i++] = tokens.nextToken();
			if (debug) gui.write("     Token " + (i-1) + " = " + x[i-1]);
		}

		/* return the vector of elements */
		return x;
	} else
		return new String[0];
}
/**
 * This method was created in VisualAge.
 * @return java.util.Vector
 * @param s java.lang.String
 */
private String[] parseInitialObjects(String s)
{
	String delimiters = "\n\t\r(,) ";
	StringTokenizer tokens = new StringTokenizer(s, delimiters);
	if (tokens.countTokens() > 1)
	{
		String x[] = new String[tokens.countTokens() - 1];

		/* store all tokens (except the first) in the array */
		String dummy = tokens.nextToken();
		int i = 0;
		while (tokens.hasMoreTokens())
			x[i++] = tokens.nextToken();

		/* return the vector of elements */
		return x;
	} else
		return new String[0];
}
/**
 * This method was created in VisualAge.
 *
 * This method parses tuple notation such as <x, y, z>
 */
private String[] parseTuple(String s)
{
	String delimiters = "\n\t\r<,>= ";
	StringTokenizer tokens = new StringTokenizer(s, delimiters);
	if (tokens.countTokens() > 0)
	{
		if (debug) gui.write("Parsing the string |" + s + "|");
		String x[] = new String[tokens.countTokens()];
		int i = 0;
		while (tokens.hasMoreTokens())
		{
			x[i++] = tokens.nextToken();
			if (debug) gui.write("     Token " + (i-1) + " = " + x[i-1]);
		}

		/* return the vector of elements */
		return x;
	} else
		return new String[0];
}
/**
 * This method was created in VisualAge.
 * @param out java.io.PrintWriter
 * @param stInit system.ATstatetable
 */
private void printConvBody(PrintWriter out, ATstatetable stInit, int initResp)
{
	out.println("		while (notDone)");
	out.println("		{");
	out.println("			switch (state)");
	out.println("			{");

	// start with the start state first
	ATstate start = new ATstate(1,0);
	start.setName("StartState");
	start.setAction("");
	genConvState(out, start, stInit, initResp);

	// step through states
	Enumeration stateEnum = stInit.returnStates();
	while (stateEnum.hasMoreElements())
	{
		try
		{
			genConvState(out, (ATstate) stateEnum.nextElement(), stInit, initResp);
		} catch (ClassCastException e)
		{
			// skip start and end states ???
		}
	}
	out.println("			}");
	out.println("		}");
}
/**
 * This method was created in VisualAge.
 * @param out java.io.PrintWriter
 */
private void printRunClosing(PrintWriter out, String classname)
{
	// print out closing code

	out.println("		input.close();");
	out.println("		output.close();");
	out.println("		connection.close();");
	out.println("	} catch (UnknownHostException e)");
	out.println("	{");
	out.println("		parent.write(\"** Unknown Host exception in " + classname + ".\");");
	out.println("	} catch (EOFException eof)");
	out.println("	{");
	out.println("		parent.write(\"** Server terminated connection in " + classname + ".\");");
	out.println("	} catch (IOException e)");
	out.println("	{");
	out.println("		parent.write(\"** IO Exception in " + classname + ".\");");
	out.println("		e.printStackTrace();");
	out.println("	}");
	out.println("}");
}
/**
 * This method was created in VisualAge.
 * @param out java.io.PrintWriter
 */
private void printRunOpening(PrintWriter out, ATstatetable stInit, int initResp) {
	out.println("\n/*");
	out.println("*");
	out.println("* Conversation run method - controls conversation.");
	out.println("*");
	out.println("* automatically generated by agentTool" + version);
	out.println("*");
	out.println("*/");
	out.println("public void run()");
	out.println("{");
	out.println("	Message m = new Message();");
	out.println("	int state = 0;");
	out.println("	boolean notDone = true;");

	/* add state constant definitions here ... */
	addStateConstants(out, stInit);

	/* add combined parameter definitions */
	out.println("\n\t/* combined parameter definitions */");
	Enumeration transEnum = stInit.returnTrans();
	while (transEnum.hasMoreElements()) {
		ATtransition t = ((ATtransition) transEnum.nextElement());
		addCombinedParams(out, parseAction2(t.getRecMessage()));
		addCombinedParams(out, parseAction2(t.getTransMessage()));
	}

	/* add variable declarations used in transitions */
	out.println("\n\t/* variables used in transitions */");
	transEnum = stInit.returnTrans();
	while (transEnum.hasMoreElements()) {
		ATtransition t = ((ATtransition) transEnum.nextElement());
		addVariableDeclarations(out, parseAction2(t.getRecMessage()));
		addVariableDeclarations(out, parseAction2(t.getTransMessage()));
		if (t.getRecMessage().indexOf('=') > 0) {
			String x[] = parseAction2(t.getRecMessage());
			if (x.length > 0)
				if (!x[0].equals("=")) {
					String var = ((x[0].replace('<', ' ')).replace('>', ' ')).trim();
					if (localVariables.indexOf(var) < 0) {
						out.println("\t" + "Object " + var + " = null;");
						localVariables.addElement(var);
					}
				}
		}
		if (t.getTransMessage().indexOf('=') > 0) {
			String x[] = parseAction2(t.getTransMessage());
			if (x.length > 0)
				if (!x[0].equals("=")) {
					String var = ((x[0].replace('<', ' ')).replace('>', ' ')).trim();
					if (localVariables.indexOf(var) < 0) {
						out.println("\t" + "Object " + var + " = null;");
						localVariables.addElement(var);
					}
				}
		}
	}

	/* add state action variable declarations */
	out.println("\n\t/* state variable declarations */");
	Enumeration stateEnum = stInit.returnStates();
	while (stateEnum.hasMoreElements()) {
		try {
			ATstate s = ((ATstate) stateEnum.nextElement());
			Vector v = (Vector) s.getActions();
			Enumeration actions = v.elements();
			while (actions.hasMoreElements()) {
				String a = (String) actions.nextElement();
				addVariableDeclarations(out, parseAction2(a));

				// add declaration for output parameters
				int eqLoc = a.indexOf('=');
				if (eqLoc >= 0) {
					String[] vars = parseTuple(a.substring(0, eqLoc));
					String var = vars[0];

					/* check individual variables */
					for (int j = 0; j < vars.length; j++)
						if (localVariables.indexOf(vars[j]) < 0) {
							out.println("\tObject " + vars[j] + " = null;");
							localVariables.addElement(vars[j]);
						}

					/* build composite name if multiple output values */
					if (vars.length > 1) {
						for (int j = 1; j < vars.length; j++)
							var = var + "_" + vars[j];
						if (localVariables.indexOf(var) < 0) {
							out.println("\t" + var + "_Type " + var + " = null;");
							localVariables.addElement(var);
						}
					}
				}
			}
		} catch (ClassCastException e) {
			// skip start and end states
		}
	}

	/* add startup stuff */
	out.println("");
	out.println("	//set up conversation");
	out.println("	try");
	out.println("	{");
}
}
