package Editor;

import system.*;
/**
 * This type was created in VisualAge.
 * This Panel contains agents and conversations
 * @author: Jennifer Mifflin
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
public class AgentDiagramPanel extends ATpanel {
							   
	private ATagent selectedAgent = null; //currently selected Agent
	private ATconversation selectedConversation = null; //currently selected Conversation

	private system.ATagent first;				//the first square choosen by a line or arrow
	protected system.ATconversation conv = null, newconv = null;

	private system.ATsystem sys= null; //the system this represents

	//MenuItems
	//for agents
	private MenuItem agentDelete;
	private MenuItem agentRoles;
	private MenuItem agentName;
	//for conversations
	private MenuItem convsPref;
	private MenuItem convsDelete;
	private MenuItem convsReverse;
	private MenuItem changeInit,
					 changeResp;
	//for both
	private MenuItem description;
	
/**
 * AgentDiagramPanel constructor comment.
 */
public AgentDiagramPanel(double v) {
	super(v);

	this.enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	this.enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
	popup = new PopupMenu();

	this.add(popup);
}
/**
 * AgentDiagramPanel constructor comment.
 */
public AgentDiagramPanel(boolean isDoubleBuffered, double v) {
	super(isDoubleBuffered, v);
}
/**
 * This method was created in VisualAge.
 */
  public void actionPerformed(java.awt.event.ActionEvent event) {
	String command = event.getActionCommand();
	
	
	//all these commands come from the popup menu.
	//they are pretty self-explanitory
	if (command.equals("agentDelete")) {
		deleteAgent();
	} else if(command.equals("agentRoles")){
		defineRoles();
	} else if(command.equals("agentName")){
		ObjName objN = new ObjName(selectedAgent);
		objN.show();

		java.util.Enumeration convs = sys.returnConvs();
		while(convs.hasMoreElements()){
			conv = (system.ATconversation)convs.nextElement();
			if(conv.getInitiator()==selectedAgent || conv.getResponder()==selectedAgent) {
				//paint(this.getGraphics());
				conv.resize(selectedAgent);
			}
		}
		
	} else if (command.equals("convsPreferences")){
		changeConvPrefs();
	} else if (command.equals("convsDelete")){
		deleteConv();
	} else if (command.equals("convsReverse")){
		reverseConv();
	}
	//change a transition's Initiator
	else if (command.equals("changeInit")){
		initOrResp = init;
		change = yes;
		sysMsg("select INITIAL State");
	}
	//change a transition's Responder
	else if (command.equals("changeResp")){
		initOrResp = resp;
		change = yes;
		sysMsg("select RESPONDING State");
	} else if(command.equals("description")){
		if(selected.equals("agent")){
			ObjectProperties oP = new ObjectProperties(selectedAgent);
			oP.show();
		} else {
			ObjectProperties oP = new ObjectProperties(selectedConversation);
			oP.show();
		}
	}
	
	stopConvAdd();
	repaint();
	//paint(this.getGraphics());
}
public void changeConvPrefs() {
	//puts up another window (ConvPropertiesDialog) which allows the user
	//to change the conversation's preferences.  Initializes name so
	//user knows what they are changing it from
	ConvPropertiesDialog convDialog = 
		new ConvPropertiesDialog(selectedConversation);
	//focuses on convDialog
	convDialog.requestFocus();
	//show convDialog
	convDialog.show();
	this.repaint();	
}
/**
 * This method was created in VisualAge.
 */
public void changeInit() {
	//delete the conversation from it's agents
	selectedConversation.deleteMe();
	//set the current state
	selectedConversation.setInitiator(first);
}
/**
 * This method was created in VisualAge.
 */
public void changeResp() {
	//delete transition from its states
	selectedConversation.deleteMe();
	//set the current state
	selectedConversation.setResponder(first);
}
/**
 * Insert the method's description here.
 * Creation date: (2/1/00 9:35:31 AM)
 */
public void clear() {}
/**
 * Opens a window to let user define the roles that the
 * selected agent implements
 * Creation date: (5/19/00 8:34:13 AM)
 */
public void defineRoles() {
	RoleWindow RW = new RoleWindow(sys.returnRolesVector(),selectedAgent);
	RW.show();
}
public void deleteAgent() {
	//deletes the selected Agent from the system.
	//has a check to make sure there are no conversations
	//attached.  If there are, it tells the user to first
	//remove the conversations.
	if (selectedAgent.conversationCount() < 1) {
		if(selectedAgent.instanceCount() <1){
			sys.deleteAgent(selectedAgent);
			sysMsg("Agent deleted");
		} else {
			sysMsg("delete Agent Instances in Deployment Panel");
		}
	} else {
		sysMsg("Remove conversations first");
	}

}
public void deleteConv() {
	//Deletes the selected conversation from the system.
	ATagent first, second;
	ATagent2 f,s;
	boolean casesConnected = false;
	first = selectedConversation.getInitiator();
	second = selectedConversation.getResponder();
	Vector cases1 = first.getCases();
	Vector cases2 = second.getCases();
	Enumeration c2,c1 = cases1.elements();
	while(c1.hasMoreElements()){
		c2 = cases2.elements();
		f = (ATagent2)c1.nextElement();
		while(c2.hasMoreElements()){
			s = (ATagent2)c2.nextElement();
			if(f.isConnectedTo(s)){
				casesConnected = true;
			}
		}
	}
	if((!first.isConnectedTo(second,selectedConversation))&&casesConnected){
		sysMsg("must remove connections in Deployment Panel that corespond to this conversation");
	} else {
		sys.deleteConvs(selectedConversation);
		sysMsg("Conversation Deleted");
	}
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public boolean findSelectedObject(int x, int y) {
	//finds the selected object.
	//first, gets the agents are conversations from the system
	java.util.Enumeration agents = sys.returnAgents();
	java.util.Enumeration convs  = sys.returnConvs();
	boolean agentSel;
	selected = null;
	system.ATagent agent;
	agents = sys.returnAgents();
	convs = sys.returnConvs();
	//checks to see if any of the agents are selected.
	//it returns if one is.  It does not continue to check for
	//other agents that could be selected.
	while(agents.hasMoreElements()){
		agent = (ATagent)agents.nextElement();
		if(agent.checkSelected(x,y) == true) {
			selected = "agent"; 
			selectedAgent = agent;
		}
	}
	if(selected == "agent"){
		return true;
	}
	//checks to see if any of the conversations are selected.
	while(convs.hasMoreElements()){
		conv = (ATconversation)convs.nextElement();
		conv.checkSelected(x,y);
		if(conv.Selected) {
			selected = "conv";
			selectedConversation = conv;
		}
	}
	if(selected == "conv"){
		return true;
	}
	//nothing has been selected, so return false
	return false;
}
/**
 * Insert the method's description here.
 * Creation date: (2/2/01 4:21:32 PM)
 * @return java.awt.Image
 */
public Image formPrintImage() {
		java.util.Enumeration agents = sys.returnAgents();
		java.util.Enumeration convs = sys.returnConvs();
	
		Image i = createImage(c[3]+5, c[4]+5);//creates an image with the panel's width and height
		Graphics agentGraphics = i.getGraphics();

		agentGraphics.setColor(Color.white);			//sets background color
		agentGraphics.fillRect(0,0,c[3]+5, c[4]+5);		//to white
		agentGraphics.setColor(Color.black);
		agentGraphics.drawString("Agent Template Diagram", 10, 10);
		
		//draws all the roles/messages onto the graphics object g
		while(agents.hasMoreElements()) {
			ATagent agent = (ATagent) agents.nextElement();			
			agent.draw(agentGraphics);
	 	}
		while(convs.hasMoreElements()) {
			system.ATconversation conv = (ATconversation) convs.nextElement();
			conv.draw(agentGraphics);
		}
		return i;
}
/**
 * Insert the method's description here.
 * Creation date: (1/18/01 3:06:35 PM)
 * @return int[]
 */
public int[] getPages() {
																	//c[0]-total pages in x-direction
																	//c[1]-total pages in y-direction
																	//c[2]-total number of pages
	int a=0, b=0;
	java.util.Enumeration agents = sys.returnAgents();
	//pages across equals greatest x coordinate of any of the ATObject/PAGEWIDTH    visibile area of a standard 8.5"
	//pages down equals greatest y coordinate of any of the ATObject/PAGEHEIGHT     by 11" paper respectively w/ this method
	//																	  	     printed in landscape mode
	while(agents.hasMoreElements()) {
		ATagent agent = (ATagent) agents.nextElement();
		if(agent.getGrfx().x2() > a) {
				a = agent.getGrfx().x2();
			}
			if(agent.getGrfx().y2() > b) {
				b = agent.getGrfx().y2();
			}
	}
	
	c[0] = a/PAGEWIDTH + 1;
	c[1] = b/PAGEHEIGHT + 1;
	c[2] = c[0] * c[1];
	c[3] = a + 10;
	c[4] = b + 10;
	
	return c;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getSelected() {
	//returns either "agent" or "conversation"
	return selected;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public ATobject getSelectedObject() {
	//returns selected Object
	if (selected.equals("agent"))
	{
		return (ATobject)selectedAgent;
	}
	else if (selected.equals("line"))
	{
		return (ATobject)selectedConversation;
	}
	return null;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getSelectedObjectName() {
	//returns selected object's name

	if (selected == null) {return "";}
	
	if (selected.equals("agent"))
	{
		return "Agent: " + selectedAgent.getName();
	}
	if (selected.equals("line"))
	{
		return "Conv:" + selectedConversation.getName();
	}
	return "";
}
public void initPanel(system.ATobject object) {
	//sets the system of the panel
	if (object instanceof system.ATsystem) {
		sys = null;
		sys = (ATsystem)object;
	}
}
	//- - - - - - - - - - - - - - - - - - - - - - - - 
	// Start to move a box, draw a line or arrow
	//- - - - - - - - - - - - - - - - - - - - - - - -
	public synchronized boolean mouseDown(int x, int y, system.ATsystem sys) {
		int j = 0;
		this.sys = sys;
		x_pos = x;
		y_pos = y;
		//lists of agents and conversations
		java.util.Enumeration agents = sys.returnAgents();
		java.util.Enumeration convs  = sys.returnConvs();

		boolean agentSel;
		selected = null;
		system.ATagent agent = new system.ATagent(version,0);
		
		//deselected conversations
		while(convs.hasMoreElements()){
			conv = (system.ATconversation)convs.nextElement();
			conv.Selected = false;
		}
		//get conversations
		convs = sys.returnConvs();
		//if a transitions initiator or responder is going to be changed
		if(change == yes){
			first = null;
			selected = null;
			//loop through agents finding selected agent
			while(agents.hasMoreElements()){
				agent = (system.ATagent)agents.nextElement();
				if(agent.checkSelected(x,y) == true) {
					agent.Selected = false;
					first = agent;
				} 
			}
			//if a agent has been selected, change the corrert end to the transition
			if(first != null){
				if(initOrResp == init){
					changeInit();
				} else { 
					changeResp();
				}
				change = no;
				return true;
			}
			//otherwise, tell the user that they need to select a agent 
			else {
				sysMsg("Select Agent");
				return true;
			}
			
		}

		//if the a conversation isn't being added	
		if(linestatus == NORMAL){
			//check to see if an agent has been selected
			while(agents.hasMoreElements()){
				//find the last selected agent
				//set selectedAgent to it
				agent = (system.ATagent)agents.nextElement();
				if(agent.checkSelected(x,y) == true){
					selectedAgent = agent;
					selected = "agent";
				}
			}
			agents = sys.returnAgents();
			//if an agent was selected, then return
			//set the agent to selected
			if(selected=="agent"){
				if(selectedAgent.Selected == false){
					selectedAgent.Selected = true;
					//if the shift is not being held down 
					if(shift!= true){
						//set the agents that are not the selected agent
						//to be not selected
						while(agents.hasMoreElements()){
							agent=(system.ATagent)agents.nextElement();
							if(agent!=selectedAgent){
								agent.Selected = false;
							}
						}
					}
				} else {
					//selected agent was selected, but the shift is being
					//held down and it is being clicked on, so now it is
					//not selected 
					if(shift == true){
						selectedAgent.Selected = false;
					}
/*					if((shift == true)&&(selectedAgent.Selected == false)){
						selectedAgent.Selected = false;
					}
		*/		}
				return true;
			} else {
				agents = sys.returnAgents();
				while(agents.hasMoreElements()){
					agent = (system.ATagent)agents.nextElement();
					agent.Selected = false;
				}
			}
			//get agents
			agents = sys.returnAgents();
			
			//check conversations to see if they are selected
			while(convs.hasMoreElements()){
				conv = (system.ATconversation)convs.nextElement();
				conv.checkSelected(x,y);
				if(conv.Selected) {
					//if a conversation is selected, return
					selected = "line";
					selectedConversation = conv;
					return true;
				}
			}
			if(conv.Selected) {return true;}
	
		}
		//a conversation is going to be added
		if(linestatus == WAIT_FIRST_POINT){
			//set selected to null
			selected = null;
			first = null;
			//loop through agents to find uppermost selected agent
			while(agents.hasMoreElements()){
				agent = (system.ATagent)agents.nextElement();
				if(agent.checkSelected(x,y) == true) {
					first = agent;
					linestatus = WAIT_SECOND_POINT;
				}
			}
			//if an agent was selected, tell user to select RESPONDER
			if(first!= null){
				sysMsg("  Select RESPONDER");
				return true;
			}
		}
		//the second agent is going to be added to the component
		if(linestatus == WAIT_SECOND_POINT){
			while(convs.hasMoreElements()){
				conv = (system.ATconversation)convs.nextElement();
			}
			agents = sys.returnAgents();
			boolean sel = false;
			while(agents.hasMoreElements()){
				agent = (system.ATagent)agents.nextElement();
				//check to see if agent is selected
				if(agent.checkSelected(x,y) == true) {
					sel = true;
					selectedAgent = agent;
				}
			}
			//if an agent was selected, set the conversation's coordinates
			//and put the linestatus back to normal
			if(sel==true){
				newconv.setCoordinates(first,selectedAgent);
				linestatus = NORMAL;
				sysMsg("Conversation Added");
				return true;
			}
		}
		return true;
	}
	//- - - - - - - - - - - -
	// Moving rectangle and attached lines
	//- - - - - - - - - - - -
public synchronized boolean mouseDrag(int x, int y, system.ATsystem sys) {
	if((x>0)&&(y>0)){
		java.util.Enumeration agents = null;
		java.util.Enumeration agents2 = null;
		java.util.Enumeration convs  = null;
		//get list of agents
		if (sys.getAgentCount() >0) {
			agents = sys.returnAgents();
			agents2 = sys.returnAgents();
		}
		//get list of conversations
		if (sys.getConvCount() >0) {
			convs = sys.returnConvs();
		}

		//reset x_pos and y_pos
		x_pos = x;
		y_pos = y;
		//variable agent
		system.ATagent agent;
		
		while(agents.hasMoreElements()){		//find which agent is selected
			agent = (system.ATagent)agents.nextElement();
			if(agent.Selected == true){
				agent.move(x,y);	//if an agent is selected, move it.
				boolean connected;
				//check if any conversations are connected to the selected agent
				//if they are, move them too
				/*if(convs != null){
					while(convs.hasMoreElements()){   
						conv = (system.ATconversation)convs.nextElement();
						connected = conv.checkAgent(agent);
						if(connected){
							conv.resize(agent);
						}
					}
					convs = sys.returnConvs();
				}*/
			}
		}
		
		conv = (system.ATconversation)convs.nextElement();
		//find the first selected conversation	
		while((convs.hasMoreElements())&&(!conv.Selected)) {
			conv = (system.ATconversation)convs.nextElement();
		}
		//add a point if it is selected and one hasn't already been added this go-around
		if(conv.Selected) {
			if(newPt == 0){
				conv.addPoint(x,y);
				newPt = 1;
			}
			//if a points already been added this time, move that point
			conv.movePoint(x,y);
		}
		
	}
	return true;
}
	public synchronized boolean mouseUp(int x, int y, system.ATsystem sys) {
		//set new Point to 0 so when a line is selected, a new point can be added
		newPt = 0;
		
		//arrange points of selected conversation
		if(selectedConversation.Selected){
			selectedConversation.arrange_pts();
		}
		return true;
	}
/**
 * Insert the method's description here.
 * Creation date: (1/8/01 11:41:31 AM)
 * @param g java.awt.Graphics
 */
public void paint(Graphics g) {
	Dimension d = getSize();
	g.setColor(Color.white);
	g.fillRect(0, 0, d.width, d.height);
	g.setColor(Color.black);

	if(sys != null) {
		//get lists of agents and conversations		
		java.util.Enumeration agents = sys.returnAgents();
		java.util.Enumeration convs  = sys.returnConvs();

		//draw the conversations
		while (convs.hasMoreElements()) {
			ATconversation conv = (ATconversation)convs.nextElement();
			conv.draw(g);
		}
		//draw the agents
		while (agents.hasMoreElements()) {
			ATagent agent = (ATagent)agents.nextElement();
			agent.draw(g);
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (2/10/00 10:05:01 AM)
 */
public void printPage(Graphics g, int pageIndex, Image i) {
	try {
		int xpage = pageIndex % c[0];
		int ypage = (pageIndex/c[0]) % c[1];
		
		if(fullsize) {
			g.drawImage(i, 72, 72, 540, 720,//<--destination rectangle
				    xpage*PAGEWIDTH, ypage*PAGEHEIGHT, xpage*PAGEWIDTH+PAGEWIDTH, ypage*PAGEHEIGHT+PAGEHEIGHT, null);
		}
		else {
			g.drawImage(i, 72, 72, 540, 72 + (c[4] * 468 / c[3]),//<--destination rectangle
					        0,  0, c[3], c[4], null);
//			g.drawImage(i, 72, 72, 540, 720,//<--destination rectangle
//	        0,  0, c[0]*PAGEWIDTH, c[1]*PAGEHEIGHT, null);
		}
	}
	 catch (Exception e) {
		System.out.println(e);
	 }
}
/**
 * This method was created in VisualAge.
 * @param e MouseEvent
 */
public synchronized void processMouseEvent(java.awt.event.MouseEvent e) {
	//if right mouse button is pushed
	if(e.isPopupTrigger()) {
		//get x,y position
		x_pos = e.getX();
		y_pos = e.getY();
		//find selected object at that point
		if(findSelectedObject(x_pos,y_pos)){
			if(selected != null) {
				popup.removeAll();
				
				//add correct menu items for component or connection
				if (selected.equals("agent")) {
					setAgentPopup();
				} else if(selected.equals("conv")) {
					setConvsPopup();
				}
				
				popup.show(this, e.getX(), e.getY());
	
			//show the popup menu
			// !!! If you don't add something to popup after doing a removeAll(),
			//  and then call show(), it will lock up the popup and you won't be
			//  able to use it again.  To fix this, I added the if(selected != null)
			//  ensure that we will always have an active popup.  -JAG 1-5-01
			}
		}
	}
	//if not popup, pass on event
	else {
		super.processMouseEvent(e);
	}
}
public void reverseConv() {
	//reverse conversation
	selectedConversation.reverse();
	//tell user conversation was reversed
	sysMsg("Conversation Reversed");
}
/**
 * Insert the method's description here.
 * Creation date: (12/2/99 8:38:23 AM)
 * @param name java.lang.String
 */
public void setAgentlName(String name) {
	//set name of selected agent
	selectedAgent.setName(name + "(T)");
}
/**
 * This method was created in VisualAge.
 */
public void setAgentPopup() {
	//initializes popup menu items and adds them to
	//the popup menu
	
	agentName = new MenuItem("Name");
	agentName.setActionCommand("agentName");
	agentName.addActionListener(this);
	popup.add(agentName);

	description = new MenuItem("Description");
	description.setActionCommand("description");
	description.addActionListener(this);
	popup.add(description);

	popup.addSeparator();
	
	agentRoles = new MenuItem("Roles");
	agentRoles.setActionCommand("agentRoles");
	agentRoles.addActionListener(this);
	popup.add(agentRoles);

	popup.addSeparator();
	
	agentDelete = new MenuItem("Delete");
	agentDelete.setActionCommand("agentDelete");
	agentDelete.addActionListener(this);
	popup.add(agentDelete);
	
}
/**
 * This method was created in VisualAge.
 */
public void setConvsPopup() {
	//initializes popup menu items and adds them to
	//the popup menu
	
	convsPref = new MenuItem("Preferences");
	convsPref.setActionCommand("convsPreferences");
	convsPref.addActionListener(this);
	popup.add(convsPref);

	description = new MenuItem("Description");
	description.setActionCommand("description");
	description.addActionListener(this);
	popup.add(description);

	popup.addSeparator();
	
	convsReverse = new MenuItem("Reverse");
	convsReverse.setActionCommand("convsReverse");
	convsReverse.addActionListener(this);
	popup.add(convsReverse);

	changeInit = new MenuItem("Change Initiator");
	changeInit.setActionCommand("changeInit");
	changeInit.addActionListener(this);
	popup.add(changeInit);

	changeResp = new MenuItem("Change Responder");
	changeResp.setActionCommand("changeResp");
	changeResp.addActionListener(this);
	popup.add(changeResp);
	popup.addSeparator();
	
	convsDelete = new MenuItem("Delete");
	convsDelete.setActionCommand("convsDelete");
	convsDelete.addActionListener(this);
	popup.add(convsDelete);
	
}
public void setSelectedObject(ATobject obj) {
	if(obj instanceof ATagent){
		selectedAgent = (ATagent)obj;
		selected = "agent";
	} else 
	if(obj instanceof ATconversation){
		selectedConversation = (ATconversation)obj;
		selected = "line";
	}
}
/**
 * This method was created in VisualAge.
 */
public void startConvAdd(system.ATconversation c) {
	//sets linestatus so mouseDown(int x,int y) method
	//knows it's looking for an agent
	linestatus = WAIT_FIRST_POINT;
	//set newconv to c
	newconv = c;
	//tell user to select initiator
	sysMsg("  Select INITIATOR");
}
/**
 * This method was created in VisualAge.
 */
public void stopConvAdd() {
	if(linestatus != NORMAL) {
		linestatus = NORMAL;
		sysMsg(" <cancel operation>");
	}
}
//- - - - - - - - - - - - - - -
// Draw objects on panel
//- - - - - - - - - - - - - - -
public void update() {
	
	/*try {
		//get graphics position for this panel
		g = this.getGraphics();
		
		Dimension d = getSize();
		if ((offGraphics == null)
		 || (d.width != offDimension.width)
		 || (d.height != offDimension.height)) {
			    offDimension = d;
			    offImage = createImage(d.width, d.height);
			    offGraphics = offImage.getGraphics();
		}
		offGraphics.setColor(Color.white);
		offGraphics.fillRect(0, 0, d.width, d.height);
		offGraphics.setColor(Color.black);

		//get lists of agents and conversations		
		java.util.Enumeration agents = sys.returnAgents();
		java.util.Enumeration convs  = sys.returnConvs();

		//draw the conversations
		while (convs.hasMoreElements()) {
			ATconversation conv = (ATconversation)convs.nextElement();
			conv.draw(offGraphics);
		}

		//draw the agents
		while (agents.hasMoreElements()) {
			ATagent agent = (ATagent)agents.nextElement();
			agent.draw(offGraphics);
		}

		g.drawImage(offImage,0,0,null);
		
		} catch (java.lang.NullPointerException e) {
	}
	*/
}
}
