package Editor;

import javax.swing.*;
/**
 * This type was created in VisualAge.
 * Diplays Roles, Tasks, and Protocols
 * @author: Jennifer Mifflin
  *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import system.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
public class RolePanel extends ATpanel
{
	//MenuItems for popup menus
	private MenuItem roleName;
	private MenuItem taskName;
	private MenuItem protName;
	private MenuItem hideTasks;
	private MenuItem roleGoals;
	private MenuItem delete;
	private MenuItem description;
	private MenuItem locality;

	//true if tasks are hidden
	boolean tasksHidden = false;
	
	//seleted objects
	ATRole selectedRole = null;
	ATTask selectedTask = null;
	ATProtocol selectedProtocol = null;

	//task to be added or manipulated
	ATTask task = null;

	int AddingTask = 3;
	int AddProtocol1 = 4;
	int AddProtocol2 = 5;

	//next protocol	number
	int pNum;
/**
 * This method was created in VisualAge.
 */
public RolePanel(double v)
{
	super(v);
	this.enableEvents(AWTEvent.MOUSE_EVENT_MASK);
	this.enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
	popup = new PopupMenu();
	this.add(popup);
}
/**
 * This method was created in VisualAge.
 */
  public void actionPerformed(java.awt.event.ActionEvent event) {
	String command = event.getActionCommand();
		
	//all these commands come from the popup menu.
	//they are pretty self-explanitory
	if(command.equals("roleName")){
		//opens new window to specify name
		ObjName objN = new ObjName(selectedRole);
		objN.show();

		//connector between roles/tasks is a graphics object aspect of tasks
		java.util.Enumeration tasks = sys.returnTasks();
		while(tasks.hasMoreElements()){
			task = (system.ATTask)tasks.nextElement();
			if(task.getRole()==selectedRole) {
				task.resizeLine(selectedRole);
				//paint(this.getGraphics());
			}
		}
			

	} else if(command.equals("roleGoals")){
		defineGoals();
	} else if(command.equals("taskName")){
		//opens new window to specify name
		ObjName objN = new ObjName(selectedTask);
		objN.show();

		java.util.Enumeration prots = sys.returnProtocols();
		while(prots.hasMoreElements()){
			selectedProtocol = (system.ATProtocol)prots.nextElement();
			if(selectedProtocol.getInitiator()==selectedTask || selectedProtocol.getResponder()==selectedTask) {
				selectedProtocol.resize(selectedTask);
				//paint(this.getGraphics());
			}
		}
			

	} else if(command.equals("protName")){
		//opens new window to specify name
		ObjName objN = new ObjName(selectedProtocol);
		objN.show();
		
	} else if(command.equals("hideTasks")){
		tasksHidden = !tasksHidden;
		
	} else if(command.equals("delete")){
		if(selected.equals("prot")){
			sys.deleteProt(selectedProtocol);
		} else if (selected.equals("task")){
			if(selectedTask.nothingAttached()){
				sys.deleteTask(selectedTask);
			} else {
				sysMsg("must remove prot first");
			}
		} else if (selected.equals("role")){
			if(selectedRole.nothingAttached()){
				if(selectedRole.noSeqDiagRoles()) {
					sys.deleteRole(selectedRole);
				} else {
					sysMsg("must remove sequence diagram assignments before deleting");
				}
			} else {
				sysMsg("must remove tasks before deleting");
			}
		}
	} else if(command.equals("description")){
		//opens new window to specify discription
		if(selected.equals("prot")){
			ObjectProperties oP = new ObjectProperties(selectedProtocol);
			oP.show();
		} else if(selected.equals("task")){
			ObjectProperties oP = new ObjectProperties(selectedTask);
			oP.show();
		} else if(selected.equals("role")){
			ObjectProperties oP = new ObjectProperties(selectedRole);
			oP.show();
		}
	} else if(command.equals("locality")){
		selectedProtocol.setExternal(!selectedProtocol.external());
	}

	cancelAddProtocol();
	cancelAddTask();
	repaint();
	//paint(this.getGraphics());
}
/**
 * Insert the method's description here.
 * Creation date: (5/11/00 12:34:35 PM)
 */
public void addProtocol(int num) {
	cancelAddTask();
	if(!tasksHidden){
		linestatus = AddProtocol1;
		pNum = num;
	} else {
		sysMsg("Must show Tasks first");
	}
}
/**
 * Insert the method's description here.
 * Creation date: (5/2/00 1:28:25 PM)
 * @param t system.ATTask
 */
public void addTask(ATTask t) {
	//begin to add task
	cancelAddProtocol();
	
	if(tasksHidden){
		sysMsg("Must show Tasks first");
	} else {
		sysMsg("Select Role to add task to");
		linestatus = AddingTask;
		task = t;
	}
}
/**
 * Insert the method's description here.
 * Creation date: (11/2/00 2:17:33 PM)
 */
public void cancelAddProtocol() {
	if(linestatus == AddProtocol1 || linestatus == AddProtocol2) {
		linestatus = NORMAL;
		sysMsg(" <cancel operation>");
	}
}
/**
 * Insert the method's description here.
 * Creation date: (11/2/00 2:17:33 PM)
 */
public void cancelAddTask() {
	if(linestatus == AddingTask) {
		linestatus = NORMAL;
		sysMsg(" <cancel operation>");
	}
}
/**
 * This method was created in VisualAge.
 */
public void changeInit() {
	//delete the transition from it's states
	selectedProtocol.deleteMe();
	//set the current state
	selectedProtocol.setComponent1(task);
	//reset the coordinates for the transition
	selectedProtocol.setCoordinates();
}
/**
 * This method was created in VisualAge.
 */
public void changeResp() {
	//delete the transition from it's states
	selectedProtocol.deleteMe();
	//set the current state
	selectedProtocol.setComponent2(task);
	//reset the coordinates for the transition
	selectedProtocol.setCoordinates();
}
/**
 * Insert the method's description here.
 * Creation date: (2/1/00 9:35:31 AM)
 */
public void clear() {
	//not yet implemented
}
/**
 * Insert the method's description here.
 * Creation date: (2/1/00 9:35:31 AM)
 */
public void clear(ATsystem sys) {
	sys.removeAllProts();
	sys.removeAllTasks();
	sys.removeAllRoles();
	selectedTask = null;
}
/**
 * Insert the method's description here.
 * Creation date: (5/19/00 8:34:13 AM)
 */
public void defineGoals() {
	//opens window to select goals for selected role
	//to implement
	RoleWindow GW = new RoleWindow(sys.returnGoalsVector(),selectedRole);
	GW.show();
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public boolean findSelectedObject(int x, int y) {
	//finds the selected object.
	//first, gets the agents are Tasks from the system
	java.util.Enumeration roles = sys.returnRoles();
	java.util.Enumeration tasks = sys.returnTasks();
	java.util.Enumeration prots = sys.returnProtocols();
	
	boolean agentSel;
	selected = null;
	system.ATRole role;
	system.ATProtocol prot;
	//checks to see if any of the roles are selected.
	//it returns if one is.  It does not continue to check for
	//other agents that could be selected.
	while(roles.hasMoreElements()){
		role = (system.ATRole)roles.nextElement();
		if(role.checkSelected(x,y) == true) {
			selected = "role"; 
			selectedRole = role;
		}
	}
	if(selected == "role"){
		return true;
	}
	//checks to see if any of the Tasks are selected.
	while(tasks.hasMoreElements()){
		task = (system.ATTask)tasks.nextElement();
		task.checkSelected(x,y);
		if(task.Selected) {
			selected = "task";
			selectedTask = task;
		}
	}
	if(selected == "task"){
		return true;
	}

	//checks to see if any of the Protocols are selected.
	while(prots.hasMoreElements()){
		prot = (system.ATProtocol)prots.nextElement();
		prot.checkSelected(x,y);
		if(prot.Selected){
			selected = "prot";
			selectedProtocol = prot;
		}
	}
	if(selected == "prot"){
		return true;
	}
	return false;
}
/**
 * Insert the method's description here.
 * Creation date: (2/2/01 4:11:16 PM)
 * @return java.awt.Image
 * @param i java.awt.Image
 * @param g java.awt.Graphics
 */
public Image formPrintImage() {
		java.util.Enumeration roles = sys.returnRoles();
		java.util.Enumeration tasks = sys.returnTasks();
		java.util.Enumeration prots = sys.returnProtocols();

		Image i = createImage(c[3]+5, c[4]+5);//creates an image with the panel's width and height + 5(for a buffer zone)
		Graphics roleGraphics = i.getGraphics();

		roleGraphics.setColor(Color.white);
		roleGraphics.fillRect(0,0,c[3]+5, c[4]+5);
		roleGraphics.setColor(Color.black);
		roleGraphics.drawString("Role Diagram", 10, 10);
	
		while(roles.hasMoreElements()) {
			system.ATRole role = (ATRole) roles.nextElement();				
			role.draw(roleGraphics);
		}
		while(tasks.hasMoreElements()) {
			system.ATTask task = (ATTask) tasks.nextElement();
			task.draw(roleGraphics);
		}
		while(prots.hasMoreElements()) {
			system.ATProtocol prot = (ATProtocol) prots.nextElement();
			prot.draw(roleGraphics);
		}
		return i;
}
/**
 * Insert the method's description here.
 * Creation date: (1/18/01 3:06:35 PM)
 * @return int[]
 */
public int[] getPages() {
																	//c[0]-total pages in x-direction
																	//c[1]-total pages in y-direction
																	//c[2]-total number of pages
	int a=0, b=0;
	java.util.Enumeration roles = sys.returnRoles();
	java.util.Enumeration tasks = sys.returnTasks();
	//pages across equals greatest x coordinate of any of the ATObject/PAGEWIDTH    visibile area of a standard 8.5"
	//pages down equals greatest y coordinate of any of the ATObject/PAGEHEIGHT     by 11" paper respectively w/ this method
	//																	  	     printed in landscape mode
	while(roles.hasMoreElements()) {
		ATRole role = (ATRole) roles.nextElement();
		if(role.getGrfx().x2() > a) {
				a = role.getGrfx().x2();
			}
			if(role.getGrfx().y2() > b) {
				b = role.getGrfx().y2();
			}
	}
	int z[];
	while(tasks.hasMoreElements()) {
		system.ATTask task = (ATTask) tasks.nextElement();
		z=task.getGrfx().getCoords();
		if(z[4]+z[2] > a) {
			a = z[4]+z[2];
		}
		if(z[5]+z[3] > b) {
			b = z[5]+z[3];
		}
	}
	
	c[0] = a/PAGEWIDTH + 1;
	c[1] = b/PAGEHEIGHT + 1;
	c[2] = c[0] * c[1];
	c[3] = a + 10;
	c[4] = b + 10;
	
	return c;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public ATobject getSelectedObject()
{	//returns the currently selected object
		if(selected == "role"){
			return selectedRole;
		} else if(selected == "task"){
			return selectedTask;
		} else if(selected == "prot"){
			return selectedProtocol;
		}
	return null;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getSelectedObjectName() {
	//returns selected object's name
	return selected;
}
public void initPanel(system.ATobject object) {
	if (object instanceof system.ATsystem) {
		sys = null;
		sys = (system.ATsystem)object;
	}		
}
	//- - - - - - - - - - - - - - - - - - - - - - - - 
	// Start to move a box, draw a line or arrow
	//- - - - - - - - - - - - - - - - - - - - - - - -
	public synchronized boolean mouseDown(int x, int y, system.ATsystem sys) {
	
		newPt = 0;
	
		x_pos = x;
		y_pos = y;
	
		java.util.Enumeration roles = sys.returnRoles();
		java.util.Enumeration tasks = sys.returnTasks();
		java.util.Enumeration protocols = sys.returnProtocols();
	
		ATRole role1;
		ATTask task1;
		ATProtocol protocol1;
	
		if(change == no){
			while(tasks.hasMoreElements()){
				task1 = (ATTask)tasks.nextElement();
				task1.Selected = false;
			}
			while(protocols.hasMoreElements()){
				protocol1 = (ATProtocol)protocols.nextElement();
				protocol1.Selected = false;
				protocol1.taskShown = !tasksHidden;
			}
		}if(change == yes){
			task = null;
			while(tasks.hasMoreElements()){
				task1 = (ATTask)tasks.nextElement();
				if(task1.checkSelected(x,y) == true){
					task1.Selected = false;
					task = task1;
				}
			}
			if(task!=null){
				if(initOrResp == init){
					changeInit();
				} else {
					changeResp();
				}
				change = no;
				return true;
			}
			else {
				sysMsg("Select Agent");
				return true;
			}
		}
		tasks = sys.returnTasks();
		protocols = sys.returnProtocols();	
			
		if(linestatus == NORMAL){
			selected = "";
			while(roles.hasMoreElements()){
				role1 = (system.ATRole)roles.nextElement();
				if(role1.checkSelected(x,y)){
					if(selectedRole!=null && selectedRole!=role1){
						selectedRole.Selected = false;
					}
					role1.Selected = true;
					selectedRole = role1;
					selected = "role";
				} else {
					role1.Selected = false;
				}
			}
			if(selected == "role"){
				return true;
			}
			while(tasks.hasMoreElements()){
				task1 = (system.ATTask)tasks.nextElement();
				if(task1.checkSelected(x,y)){
					if(selectedTask!=null && selectedTask!= task1){
						selectedTask.Selected = false;
					}
					task1.Selected = true;
					selectedTask = task1;
					selected = "task";
				} else {
					task1.Selected = false;
				}
			}
			if(selected == "task"){
				return true;
			}
			while(protocols.hasMoreElements()){
				protocol1 = (ATProtocol)protocols.nextElement();
				if(protocol1.checkSelected(x,y)){
					if(selectedProtocol!=null && selectedProtocol != protocol1){
						selectedProtocol.Selected = false;
					}
					protocol1.Selected = true;
					selectedProtocol = protocol1;
					selected = "protocol";
				} else {
					protocol1.Selected = false;
				}
			}
			if(selected == "protocol"){
				return true;
			}
		} else if (linestatus == AddingTask){
			while(roles.hasMoreElements()){
				role1 = (system.ATRole)roles.nextElement();
				if(role1.checkSelected(x,y)){
					role1.Selected = false;
					linestatus = NORMAL;
					selectedRole = role1;
					sys.addTask(task);
					task.setRole(selectedRole);
					break;
				}
			}
			
		} else if (linestatus == AddProtocol1){
			while(tasks.hasMoreElements()){
				task1 = (system.ATTask)tasks.nextElement();
				if(task1.checkSelected(x,y)){
					task1.Selected = false;
					task = task1;
					linestatus = AddProtocol2;
				}
			}
			if(linestatus == AddProtocol1){
				System.out.println("select source task");
			}
		} else if (linestatus == AddProtocol2){
			ATTask hold = new ATTask(version,0);
			while(tasks.hasMoreElements()){
				task1 = (system.ATTask)tasks.nextElement();
				if(task1.checkSelected(x,y) && task1!=task){
					task1.Selected = false;
					hold = task1;
					linestatus = NORMAL;
				}
			}
			if(linestatus == NORMAL){
				ATProtocol protocol = new ATProtocol(version,sys.getNextKey());
				protocol.setCoordinates(task,hold);
				protocol.setName("Protocol" + pNum);
				sys.addProtocol(protocol);
			} else {
				System.out.println("select reciever task");
			}
		}	
		return true;
	}
public synchronized boolean mouseDrag(int x, int y, system.ATsystem sys) {
	ATRole role;
	ATProtocol protocol;
	if((x>0)&&(y>0)){
		//reset x_pos and y_pos
		x_pos = x;
		y_pos = y;
		
		java.util.Enumeration roles = sys.returnRoles();
		java.util.Enumeration tasks = sys.returnTasks();
		java.util.Enumeration protocols = sys.returnProtocols();

		ATRole selRole = null;
		ATTask selTask = null;
		ATProtocol selProtocol = null;
		
		//go through objects and if they're selected, drag them
		while(roles.hasMoreElements()){
			selRole = (ATRole)roles.nextElement();
			if(selRole.Selected){
				selRole.move(x,y);
			}
		}
		while(tasks.hasMoreElements()){
			selTask = (ATTask)tasks.nextElement();
			if(selTask.Selected){
				selTask.move(x,y);
			}
		}
		while(protocols.hasMoreElements()){
			selProtocol = (ATProtocol)protocols.nextElement();
			if(selProtocol.Selected){
				if(newPt == 0){
					selProtocol.addPoint(x,y);
					newPt = 1;
				}
				selProtocol.movePoint(x,y);
			}
		}

	}
	return true;
}
	public synchronized boolean mouseUp(int x, int y, system.ATsystem sys) {
		newPt = 0;
		//arrange points of selected transition
		if(selectedProtocol.Selected){
			selectedProtocol.arrange_pts();
		}	
		return true;
	}
/**
 * Insert the method's description here.
 * Creation date: (1/8/01 11:34:08 AM)
 * @param g java.awt.Graphics
 */
public void paint(Graphics g) {
	Dimension d = getSize();
	g.setColor(Color.white);
	g.fillRect(0, 0, d.width, d.height);
	g.setColor(Color.black);

	if(sys != null) {
		//get lists of agents and conversations		
		java.util.Enumeration roles = sys.returnRoles();
		java.util.Enumeration tasks  = sys.returnTasks();
		java.util.Enumeration protocols = sys.returnProtocols();
		
		//draw the protocols
		while (protocols.hasMoreElements()){
			ATProtocol protocol = (ATProtocol)protocols.nextElement();
			protocol.taskShown = !tasksHidden;
			protocol.draw(g);
		}	
		
		//draw the tasks
		if(!tasksHidden){
			while (tasks.hasMoreElements()) {
				ATTask task = (ATTask)tasks.nextElement();
				task.draw(g);
			}
		}

		//draw the roles
		while (roles.hasMoreElements()) {
			ATRole role = (ATRole)roles.nextElement();
			role.draw(g);
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (2/10/00 10:05:01 AM)
 */
public void printPage(Graphics g, int pageIndex, Image i) {
	try {
		int xpage = pageIndex % c[0];
		int ypage = (pageIndex/c[0]) % c[1];
		
		if(fullsize) {	
			g.drawImage(i, 72, 72, 540, 720,//<--destination rectangle
				    xpage*PAGEWIDTH, ypage*PAGEHEIGHT, xpage*PAGEWIDTH+PAGEWIDTH, ypage*PAGEHEIGHT+PAGEHEIGHT, null);	
		}
		else {
			g.drawImage(i, 72, 72, 540, 72 + (c[4] * 468 / c[3]),//<--destination rectangle
					        0,  0, c[3], c[4], null);
//			g.drawImage(i, 72, 72, 540, 720,//<--destination rectangle
//	        0,  0, c[0]*PAGEWIDTH, c[1]*PAGEHEIGHT, null);
		}
	 }   
	 catch (Exception e) {
		System.out.println(e);
	 }
}
/**
 * This method was created in VisualAge.
 * @param e MouseEvent
 */
public synchronized void processMouseEvent(java.awt.event.MouseEvent e) {
	//if right mouse button is pushed
	if(e.isPopupTrigger()) {
		//get x,y position
		x_pos = e.getX();
		y_pos = e.getY();
		findSelectedObject(x_pos,y_pos);
		//find selected object at that point
		if(selected != null) {
			popup.removeAll();
			//set popup menu according to what has been selected
		
			if(selected.equals("role")){
				setRolePopup();
			} else if(selected.equals("task")){
				setTaskPopup();
			} else if(selected.equals("prot")){
				setProtPopup();
			} else {
				if(!tasksHidden){
					hideTasks = new MenuItem("Hide Tasks");
				} else {
					hideTasks = new MenuItem("Show Tasks");
				}
				hideTasks.setActionCommand("hideTasks");
				hideTasks.addActionListener(this);
				popup.add(hideTasks);

			}
			//show popup
			popup.show(this,x_pos, y_pos);
		}
	}
	//if not popup, pass on event
	else {
		super.processMouseEvent(e);
	}
}
/**
 * This method was created in VisualAge.
 */
public void setProtPopup() {
	protName = new MenuItem("Name");
	protName.setActionCommand("protName");
	protName.addActionListener(this);
	popup.add(protName);

	description = new MenuItem("Description");
	description.setActionCommand("description");
	description.addActionListener(this);
	popup.add(description);

	if(selectedProtocol.external()) 
		locality = new MenuItem("Set internal");
	else {
		locality = new MenuItem("Set external");
	}
	locality.setActionCommand("locality");
	locality.addActionListener(this);
	popup.add(locality);
	
	popup.addSeparator();
	
	delete = new MenuItem("Delete");
	delete.setActionCommand("delete");
	delete.addActionListener(this);
	popup.add(delete);
}
/**
 * This method was created in VisualAge.
 */
public void setRolePopup() {
	roleName = new MenuItem("Name");
	roleName.setActionCommand("roleName");
	roleName.addActionListener(this);
	popup.add(roleName);

	description = new MenuItem("Description");
	description.setActionCommand("description");
	description.addActionListener(this);
	popup.add(description);

	popup.addSeparator();
	
	roleGoals = new MenuItem("Goals");
	roleGoals.setActionCommand("roleGoals");
	roleGoals.addActionListener(this);
	popup.add(roleGoals);

	popup.addSeparator();
	
	delete = new MenuItem("Delete");
	delete.setActionCommand("delete");
	delete.addActionListener(this);
	popup.add(delete);
}
/**
 * This method was created in VisualAge.
 */
public void setTaskPopup() {
	taskName = new MenuItem("Name");
	taskName.setActionCommand("taskName");
	taskName.addActionListener(this);
	popup.add(taskName);

	description = new MenuItem("Description");
	description.setActionCommand("description");
	description.addActionListener(this);
	popup.add(description);

	popup.addSeparator();
	
	delete = new MenuItem("Delete");
	delete.setActionCommand("delete");
	delete.addActionListener(this);
	popup.add(delete);
}
//- - - - - - - - - - - - - - -
// Draw objects on panel
//- - - - - - - - - - - - - - -
public void update() {
	
	/*try {
		//get graphics position for this panel
		g = this.getGraphics();
		Dimension d = getSize();
		if ((offGraphics == null)
		 || (d.width != offDimension.width)
		 || (d.height != offDimension.height)) {
			    offDimension = d;
			    offImage = createImage(d.width, d.height);
			    offGraphics = offImage.getGraphics();
		}
		offGraphics.setColor(Color.white);
		offGraphics.fillRect(0, 0, d.width, d.height);
		offGraphics.setColor(Color.black);

		//get lists of agents and conversations		
		java.util.Enumeration roles = sys.returnRoles();
		java.util.Enumeration tasks  = sys.returnTasks();
		java.util.Enumeration protocols = sys.returnProtocols();

		//draw the protocols
		while (protocols.hasMoreElements()){
			ATProtocol protocol = (ATProtocol)protocols.nextElement();
			protocol.taskShown = !tasksHidden;
			protocol.draw(offGraphics);
		}	
		
		//draw the tasks
		if(!tasksHidden){
			while (tasks.hasMoreElements()) {
				ATTask task = (ATTask)tasks.nextElement();
				task.draw(offGraphics);
			}
		}

		//draw the roles
		while (roles.hasMoreElements()) {
			ATRole role = (ATRole)roles.nextElement();
			role.draw(offGraphics);
		}
		
		g.drawImage(offImage,0,0,null);

	} catch (java.lang.NullPointerException e) {
	}
	*/
}
}
