package KBFile;

import java.io.*;

/*
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 */
 
public class RecordHeader {

  /**
   * File pointer to the first byte of record data (8 bytes).
   */
  protected long dataPointer; 

  /**
   * Actual number of bytes of data held in this record (4 bytes).
   */
  protected int dataCount;

  /**
   * Number of bytes of data that this record can hold (4 bytes).
   */
  protected int dataCapacity;  

  /**
   * Indicates this header's position in the file index.
   */
  protected int indexPosition;

  protected RecordHeader() {
  }  
  protected RecordHeader(long dataPointer, int dataCapacity) {
	if (dataCapacity < 1) {
	  throw new IllegalArgumentException("Bad record size: " + dataCapacity);
	}
	this.dataPointer = dataPointer;
	this.dataCapacity = dataCapacity;
	this.dataCount = 0;
  }  
  protected int getDataCapacity() {
	return dataCapacity;
  }  
  protected int getFreeSpace() {
	return dataCapacity - dataCount;
  }  
  protected int getIndexPosition() {
	return indexPosition;
  }  
  protected void read(DataInput in) throws IOException {
	dataPointer = in.readLong();
	dataCapacity = in.readInt();
	dataCount = in.readInt();
  }  
  protected static RecordHeader readHeader(DataInput in) throws IOException {
	RecordHeader r = new RecordHeader();
	r.read(in);
	return r;
  }  
  protected void setIndexPosition(int indexPosition) {
	this.indexPosition = indexPosition;
  }  
  /**
   * Returns a new record header which occupies the free space of this record.
   * Shrinks this record size by the size of its free space.
   */
  protected RecordHeader split() throws RecordsFileException {
	long newFp = dataPointer + (long)dataCount;
	RecordHeader newRecord = new RecordHeader(newFp, getFreeSpace());
	dataCapacity = dataCount;
	return newRecord;
  }  
  protected void write(DataOutput out) throws IOException {
	out.writeLong(dataPointer);
	out.writeInt(dataCapacity);
	out.writeInt(dataCount);
  }  
}
