package Transforms;

/**
 * This class represents the Agent Component Transformation 
 * (ACT1) in the transformation process.
 * This transform creates the agent component with the
 * basic required states and transitions.
 *
 * Creation date: (12/28/00 12:07:00 AM)
 * @author: Athie Self
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.*;

public class ACTransform1 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * ACTransform1 constructor comment.
 */
public ACTransform1(TSystem t) {
	super();

	sys = t.copy();
	currentSys = t;
	beforeSys = t.copy();
	setRationale("");
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem 
 * and that system has roles, tasks, and protocols defined.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)){
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method creates an agent component for 
 * each agent class in the system. 
 *
 * the params attribute is not used
 */

public void execute(Object params) {	
			
	Vector agents = sys.agents;

	for(int i = 0; i < agents.size(); i++){
		
		Agent ag = (Agent)agents.elementAt(i);

		// create the ATagentComponent for each agent
		ATagent atAgent = ag.getAtAgent();
		ATarchitecture atArch = atAgent.getArchitecture();
		ATagentComponent atagentComponent = new ATagentComponent(sys.version,sys.atSys.getNextKey());
		atArch.addComponent(atagentComponent);
		
		int num = atAgent.getComponentCount();

		// set the name of the AgentComponent
		atagentComponent.setName("AgentComponent");

		// set the position of the AgentComponent
		atagentComponent.setCoordinates(50+((num-1)%7)*75, 50+((num-1)/7)*75, 50, 50);

		// create the state table for the AgentComponent
		ATstatetable atStateTable = new ATstatetable(atAgent,sys.version,sys.atSys.getNextKey());
		ATtableElement atStartState = atStateTable.findStatebyname("StartState");
		ATendState atEndState = (ATendState)atStateTable.findStatebyname("EndState");
		atStartState.move(400,0);
		atEndState.setPos(100,100);
			
		// create states for state table
		ATstate atState1 = new ATstate(sys.version,sys.atSys.getNextKey());
		atState1.setName("idle");
		atState1.setCoordinates(200,200,10,10);
		atStateTable.addState(atState1);

		ATstate atState2 = new ATstate(sys.version,sys.atSys.getNextKey());
		atState2.setName("determineRecipient");
		atState2.setAction("");
		atState2.setAction("c=getComponent(message)");
		atState2.setCoordinates(400,250,10,10);
		atStateTable.addState(atState2);
		
		// create transitions for state table
		ATtransition atTransition1 = new ATtransition(sys.version,sys.atSys.getNextKey());
		atTransition1.setRecMessage("extReceive(message,agent)");
		atTransition1.setCoordinates(atState1,atState2);
		atStateTable.addTransition(atTransition1);
		
		ATtransition atTransition2 = new ATtransition(sys.version,sys.atSys.getNextKey());
		atTransition2.addAction("relay(message,c)");		
		atTransition2.setGuardCondition("c!=null");
		atTransition2.setCoordinates(atState2,atState1);
		atStateTable.addTransition(atTransition2);

		ATtransition atTransition3 = new ATtransition(sys.version,sys.atSys.getNextKey());
		atTransition3.setRecMessage("extReceive(terminate,agent)");
		atTransition3.setCoordinates(atState1,atEndState);
		atStateTable.addTransition(atTransition3);

		atagentComponent.setStateTable(atStateTable);
	}

	// now make the altered system the current system
	// can be undone by the unDo method
	currentSys = sys;
	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
