package Transforms;

/**
 * This class represents the Agent Component Transformation 
 * (ACT6) in the transformation process.
 * This transform adds the logic to start all
 * persistent components to the agent component.
 *
 * Creation date: (1/2/01 2:20:00 AM)
 * @author: Athie Self
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.*;

public class ACTransform6 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * ACTransform6 constructor comment.
 */

public ACTransform6(TSystem t) {
	super();

	sys = t.copy();
	currentSys = t;
	beforeSys = t.copy();
	setRationale("");
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem 
 * and that system has roles, tasks, and protocols defined.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */

public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)){
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * the params attribute is not used
 */

public void execute(Object params) {	

	Vector	agents = sys.agents;

	for(int i = 0; i < agents.size(); i++){
		
		Agent ag = (Agent)agents.elementAt(i);

		if (!hasTransientComponent(ag)) {
			
			// get the AgentComponent
			ATagent atAgent = ag.getAtAgent();
			ATarchitecture atArch = atAgent.getArchitecture();
			ATagentComponent atagentComponent = atArch.findComponentbyname("AgentComponent");

			// get the state table for the AgentComponent
			ATstatetable atStateTable = atagentComponent.getStateTable();
			ATtableElement atEndState = atStateTable.findStatebyname("EndState");
			
			// add new state for state table
			ATstate atStartPersistentCompsState = new ATstate(sys.version,sys.atSys.getNextKey());
			atStartPersistentCompsState.setName("startPersistentComps");
			atStartPersistentCompsState.setAction("");
			atStartPersistentCompsState.setAction("started=startComps()");
			atStartPersistentCompsState.setAction("time=getSleepTime()");
			atStartPersistentCompsState.setCoordinates(200,100,10,10);
			atStateTable.addState(atStartPersistentCompsState);
			
			// add new transition for state table
			ATtransition atTransition = new ATtransition(sys.version,sys.atSys.getNextKey());
			atTransition.setGuardCondition("NOT started");
			atTransition.setCoordinates(atStartPersistentCompsState,atEndState);
			atStateTable.addTransition(atTransition);
			
			atagentComponent.setStateTable(atStateTable);
		}	
	}

	// now make the altered system the current system
	// can be undone by the unDo method
	currentSys = sys;
	
}
/**
 * @return boolean
 * @param a Transforms.Agent
 */

public boolean hasTransientComponent(Agent a) {

	Vector components = a.getComponents();
	Vector vecTransitions = new Vector();
	
	if(components != null) {

		for(int i = 0; i < components.size(); i++) {

			Component c = (Component)components.elementAt(i);
			
			if (!c.getName().equals("AgentComponent")) {
				
				StateTable st = (StateTable)c.getStateTable();
				
				if(st != null) {
					vecTransitions = st.getTransitions();
				}
				
				if(vecTransitions != null) {
					
					for(int k = 0; k < vecTransitions.size(); k++) {
						
						Transition t = (Transition)vecTransitions.elementAt(k);
						
						if ((t.getFromName().equals("StartState"))
							&& ((t.getReceiveEvent() != null) 
							|| (t.getReceive() != null))) {

							return true;
								
						}
					}
				}
			}
		}
	}
	
	return false;	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
