package Transforms;

/**
 * This class is used to represent events that are transmitted on
 * transitions in state tables
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
 
public class Event {
	private String	name = null;				// event performative, or event type
	private Vector	parameters = null;			// SEQUENCE of Parameters
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 11:27:00 AM)
 */
public Event(String n) {
	name = n;
}
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 11:27:43 AM)
 */
public Event(String n, Vector p) {
	name = n;
	parameters = p;
}
/**
 * Insert the method's description here.
 * Creation date: (11/16/00 9:22:59 AM)
 */
public Event copy() {
	if(parameters != null)
		return new Event(name, (Vector)parameters.clone());
	else
		return new Event(name);
}
/**
 * This method is used to determine if one event
 * logically equals another event (same name, same parameters)
 *
 * Creation date: (11/10/00 9:45:01 AM)
 */
public boolean corresponds(Event e) {
	int p, p2;
	if(!name.equals(e.name))
		return false;
		
	if(parameters != null)
		p = parameters.size();
	else
		p = 0;
		
	if(e.parameters != null)
		p2 = e.parameters.size();
	else
		p2 = 0;
		
	if(p != p2)
		return false;
	
	return true;
}
/**
 * This method is used to determine if one event
 * logically equals another event (same name, same parameters)
 *
 * Creation date: (11/10/00 9:45:01 AM)
 */
public boolean equals(Event e) {
	int p, p2;
	if(!name.equals(e.name))
		return false;
		
	if(parameters != null)
		p = parameters.size();
	else
		p = 0;
		
	if(e.parameters != null)
		p2 = e.parameters.size();
	else
		p2 = 0;
		
	if(p != p2)
		return false;

	for(int i = 0; i < parameters.size(); i++){
		Parameter param1 = (Parameter)parameters.elementAt(i);
		Parameter param2 = (Parameter)e.parameters.elementAt(i);
		if(!param1.getName().equals(param2.getName()))
			return false;
	}
	
	return true;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 8:50:59 PM)
 * @return java.lang.String
 */
public java.lang.String getName() {
	return name;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 8:50:59 PM)
 * @return java.util.Vector
 */
public java.util.Vector getParameters() {
	return parameters;
}
/**
 * This method is used to determine if the event has the given parameter
 *
 * Creation date: (11/29/00 1:33:27 PM)
 */
public boolean hasParameter(Parameter p) {
	if(parameters != null)
	for(int i = 0; i < parameters.size(); i++){
		Parameter p2 = (Parameter)parameters.elementAt(i);
		if(p2.getName().equals(p.getName()))
		return true;
	}

	// default
	return false;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 8:50:59 PM)
 * @param newName java.lang.String
 */
public void setName(java.lang.String newName) {
	name = newName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 8:51:00 PM)
 * @param newParameters java.util.Vector
 */
public void setParameters(java.util.Vector newParameters) {
	parameters = newParameters;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/6/00 1:29:25 PM)
 */
public void show() {
	System.out.print(name);
	if(parameters != null){
		System.out.print("(");
		for(int i = 0; i < parameters.size(); i++){
			Parameter p = (Parameter)parameters.elementAt(i);
			System.out.print(p.getName());
			if(i+1 < parameters.size())
				System.out.print(", ");
		}
		System.out.print(")");
	}
}
/**
 * Returns a string representation of the event
 *
 * Creation date: (11/6/00 1:29:25 PM)
 */
public String toString() {
	String e = "";
	
	e = e + name;
	if(parameters != null){
		e = e + "(";
		for(int i = 0; i < parameters.size(); i++){
			Parameter p = (Parameter)parameters.elementAt(i);
			e = e + p.getName();
			if(i+1 < parameters.size())
				e = e + ", ";
		}
		e = e + ")";
	}
	return e;
}
}
