package Transforms;

/**
 * This class is used to represent a FunctionCall that can occur
 * in the rhs of an Action
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import java.util.StringTokenizer;
 
public class FunctionCall {
	private String	name = null;			// name of the function
	private Vector	parameters = null;		// SEQUENCE of parameters
/**
 * Insert the method's description here.
 * Creation date: (11/5/00 1:25:11 AM)
 */
public FunctionCall(String n) {
	name = n;
}
/**
 * Insert the method's description here.
 * Creation date: (11/5/00 1:26:18 AM)
 */
public FunctionCall(String n, Vector p) {
	name = n;
	parameters = p;
}
/**
 * This method will create a FunctionCall that is a copy of this
 *
 * Creation date: (11/16/00 9:08:45 AM)
 */
public FunctionCall copy() {
	if(parameters != null)
		return new FunctionCall(name, (Vector)parameters.clone());
	else
		return new FunctionCall(name, null);
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 8:56:13 PM)
 * @return java.lang.String
 */
public java.lang.String getName() {
	return name;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 8:56:13 PM)
 * @return java.util.Vector
 */
public java.util.Vector getParameters() {
	return parameters;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 8:56:13 PM)
 * @param newName java.lang.String
 */
public void setName(java.lang.String newName) {
	name = newName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 8:56:13 PM)
 * @param newParameters java.util.Vector
 */
public void setParameters(java.util.Vector newParameters) {
	parameters = newParameters;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/5/00 1:05:22 AM)
 */
public void show() {
	System.out.print(name);
	System.out.print("(");
	if(parameters != null){
		for(int i = 0; i < parameters.size(); i++){
			Parameter p = (Parameter)parameters.elementAt(i);
			System.out.print(p.getName());
			if(i+1 < parameters.size()){
				System.out.print(", ");
			}
		}
	}
	System.out.println(")");
}
/**
 * Returns a string representation of the FunctionCall
 *
 * Creation date: (11/5/00 1:05:22 AM)
 */
public String toString() {
	String function = "";
	
	function = function + name + "(";
	if(parameters != null){
		for(int i = 0; i < parameters.size(); i++){
			Parameter p = (Parameter)parameters.elementAt(i);
			function = function + p.getName();
			if(i+1 < parameters.size()){
				function = function + ", ";
			}
		}
	}
	function = function + ")";
	return function;
}
/**
 * This method will return a FunctionCall from its string representation
 *
 * Creation date: (11/5/00 12:09:32 AM)
 */
public static FunctionCall translateIn(String s) {
	StringTokenizer	st = new StringTokenizer(s, "(), ");
	FunctionCall 	f;
	Parameter 		parameter;
	
	if(st.countTokens() == 0){
		return null;
	}
	else{
		String name = st.nextToken();
		Vector parameters = new Vector();
		while(st.hasMoreTokens()){
			String p = st.nextToken();
			parameter = new Parameter(p);
			parameters.add(parameter);
		}
		f = new FunctionCall(name, parameters);
		return f;
	}
}
}
