package Transforms;

/**
 * This class is used to represent protocols in the system.
 * Protocols are initially determined between tasks in the analysis phase.
 * Later, they are duplicated between the agent components that are created
 * in the design.
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import system.ATProtocol;
 
public class Protocol {
	
	public static final int internal = 1;
	public static final int external = 2;

	private ATProtocol	atProtocol = null;
	private String		name = null;			// protocol name
	private int 		mode = 0;				// either "internal" or "external"
	private Task 		initiator = null;		// initiator task of the protocol
	private Task 		responder = null;		// responder task of the protocol
	private Component 	initComp = null;		// initiator component of the protocol
	private Component 	respComp = null;		// responder component of the protocol

	private boolean decided = false;
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 12:39:24 PM)
 */
public Protocol(ATProtocol p, Task i, Task r) {
	name = p.getName();
	initiator = i;
	responder = r;
	atProtocol = p;
	if(p.external())
		mode = Protocol.external;
	else
		mode = Protocol.internal;
}
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 12:36:36 PM)
 */
public Protocol(Task i, Task r) {
	initiator = i;
	responder = r;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return system.ATProtocol
 */
public system.ATProtocol getAtProtocol() {
	return atProtocol;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return int
 */
public final static int getExternal() {
	return external;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return Transforms.Component
 */
public Component getInitComp() {
	return initComp;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return Transforms.Task
 */
public Task getInitiator() {
	return initiator;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return int
 */
public final static int getInternal() {
	return internal;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return int
 */
public int getMode() {
	return mode;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return java.lang.String
 */
public java.lang.String getName() {
	return name;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return Transforms.Component
 */
public Component getRespComp() {
	return respComp;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return Transforms.Task
 */
public Task getResponder() {
	return responder;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @return boolean
 */
public boolean isDecided() {
	return decided;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @param newAtProtocol system.ATProtocol
 */
public void setAtProtocol(system.ATProtocol newAtProtocol) {
	atProtocol = newAtProtocol;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @param newDecided boolean
 */
public void setDecided(boolean newDecided) {
	decided = newDecided;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @param newInitComp Transforms.Component
 */
public void setInitComp(Component newInitComp) {
	initComp = newInitComp;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @param newInitiator Transforms.Task
 */
public void setInitiator(Task newInitiator) {
	initiator = newInitiator;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @param newMode int
 */
public void setMode(int newMode) {
	mode = newMode;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @param newName java.lang.String
 */
public void setName(java.lang.String newName) {
	name = newName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @param newRespComp Transforms.Component
 */
public void setRespComp(Component newRespComp) {
	respComp = newRespComp;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:09:05 PM)
 * @param newResponder Transforms.Task
 */
public void setResponder(Task newResponder) {
	responder = newResponder;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/6/00 1:05:36 PM)
 */
public void show() {
	System.out.println();
	System.out.println("Protocol: " + name);
	System.out.println("initiator: " + initiator.getName());
	System.out.println("responder: " + responder.getName());
	if(mode == Protocol.internal)
		System.out.println("mode: internal");
	if(mode == Protocol.external)
		System.out.println("mode: external");
	if(initComp != null)
		System.out.println("initComp: " + initComp.getName());
	if(respComp != null)
		System.out.println("respComp: " + respComp.getName());
}
/**
 * Returns a String with information about the protocol
 *
 * Creation date: (11/21/00 11:43:11 AM)
 */
public String toString() {
	String s = name;
	if((initComp != null) && (respComp != null))
		s += " [" + initComp.getName() + ", " + respComp.getName() + "]";
	return s;
}
}
