package Transforms;

/**
 * This class is used to represent externally received events in task and component diagrams
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
 
public class ReceiveEvent {
	private Event		event = null;			// Event that is received
	private String		sender = null;			// string representing the sender of the event
	private Vector		protocols = null;		// Set of protocol that it belongs to
	//private Vector		convMatches = null;		// set of SendEvents that match in other components
	private String		convName = null;		// The name of the conversation this ReceiveEvent starts
	private Vector		eventMatches = null;
	private Vector		transMatches = null;
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 11:28:20 AM)
 */
public ReceiveEvent(Event e, String s) {
	event = e;
	sender = s;
}
/**
 * This method will create a logically equivalent copy of the receive event
 *
 * Creation date: (11/16/00 9:28:06 AM)
 */
public ReceiveEvent copy() {
	ReceiveEvent re = new ReceiveEvent(event.copy(), String.valueOf(sender));
	if(protocols != null)
		re.protocols = (Vector)protocols.clone();
	return re;
}
/**
 * This method determines if the two ReceiveEvents are logically equivalent
 *
 * Creation date: (11/10/00 9:42:48 AM)
 */
public boolean equals(ReceiveEvent re) {
	if(!event.equals(re.event))
		return false;
	if(!sender.equals(re.sender))
		return false;
	return true;
}
/**
 * Insert the method's description here.
 * Creation date: (1/3/01 2:43:52 PM)
 * @return java.lang.String
 */
public java.lang.String getConvName() {
	return convName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:29:46 PM)
 * @return Transforms.Event
 */
public Event getEvent() {
	return event;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:42:13 AM)
 * @return java.util.Vector
 */
public java.util.Vector getEventMatches() {
	return eventMatches;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:29:46 PM)
 * @return java.util.Vector
 */
public java.util.Vector getProtocols() {
	return protocols;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:29:46 PM)
 * @return java.lang.String
 */
public java.lang.String getSender() {
	return sender;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:42:13 AM)
 * @return java.util.Vector
 */
public java.util.Vector getTransMatches() {
	return transMatches;
}
/**
 * Insert the method's description here.
 * Creation date: (1/3/01 2:43:52 PM)
 * @param newConvName java.lang.String
 */
public void setConvName(java.lang.String newConvName) {
	convName = newConvName;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:29:46 PM)
 * @param newEvent Transforms.Event
 */
public void setEvent(Event newEvent) {
	event = newEvent;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:42:13 AM)
 * @param newEventMatches java.util.Vector
 */
public void setEventMatches(java.util.Vector newEventMatches) {
	eventMatches = newEventMatches;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:29:46 PM)
 * @param newProtocols java.util.Vector
 */
public void setProtocols(java.util.Vector newProtocols) {
	protocols = newProtocols;
}
/**
 * Insert the method's description here.
 * Creation date: (12/5/00 9:29:46 PM)
 * @param newSender java.lang.String
 */
public void setSender(java.lang.String newSender) {
	sender = newSender;
}
/**
 * Insert the method's description here.
 * Creation date: (1/5/01 10:42:13 AM)
 * @param newTransMatches java.util.Vector
 */
public void setTransMatches(java.util.Vector newTransMatches) {
	transMatches = newTransMatches;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/6/00 1:33:34 PM)
 */
public void show() {
	System.out.print("receive(");
	event.show();
	System.out.print(", " + sender + ")");
	if(protocols != null){
		System.out.print("\t(Protocols: ");
		for(int i = 0; i < protocols.size(); i++){
			Protocol p = (Protocol)protocols.elementAt(i);
			System.out.print(p.getName());
			if(p.getInitComp() != null){
				System.out.print("[" + p.getInitComp().getName() + ", " + p.getRespComp().getName() + "]");
			}
			if(i+1 < protocols.size())
				System.out.print(" , ");
		}
		System.out.print(")");
	}
}
/**
 * Returns a string representation of the ReceiveEvent
 *
 * Creation date: (11/6/00 1:33:34 PM)
 */
public String toString() {
	return ("receive(" + event.toString() + ", " + sender + ")");	
}
}
