package Transforms;

/**
 * This class represents the third phase of the transformation process.  During
 * this phase, the states and transitions that belong to conversations are
 * removed from the components and placed in the conversation halfs.  Transitions
 * with actions are created in the components in place of the removed conversations.
 *
 * Creation date: (11/10/00 11:16:23 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import system.ATconversation;
import system.ATstatetable;
import system.ATtransition;
import system.ATtableElement;
import system.ATstartState;
import system.ATendState;
import system.ATagentComponent;
import system.ATagentComponentAttribute;
import system.ATagentComponentMethod;

public class Stage3 extends Thread{
	TSystem sys = null;
	boolean debug = false;
/**
 * Phase1 constructor comment.
 */
public Stage3(TSystem s) {
	sys = s;
	start();	
}
/**
 * This method adds attributes and methods to components.
 *
 * Creation date: (1/27/01 9:02:39 AM)
 */
void attributesAndComponents() {
	Vector 		agents = sys.agents;
	Agent		agent;
	Vector 		components;
	Component 	c;
	StateTable 	st;
	Vector 		convs;
	Vector 		states;
	State 		s;
	Vector 		transitions;
	Transition 	t;
	Vector 		actions;
	Action 		a;
	ATagentComponent atComp;
	ATagentComponentAttribute attribute;
	ATagentComponentMethod method;
	String lhsType;
	Vector parameters;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		agent = (Agent)agents.elementAt(i);
		components = agent.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			atComp = c.getAtComponent();
			st = c.getStateTable();
			states = st.getStates();
			transitions = st.getTransitions();
			// look at every state
			if(states != null)
			for(int k = 0; k < states.size(); k++){
				s = (State)states.elementAt(k);
				actions = s.getActions();
				if(actions != null)
				for(int l = 0; l < actions.size(); l++){
					a = (Action)actions.elementAt(l);
					lhsType = "void";
					// look at lhs
					Vector lhs = a.getLhs();
					if(lhs != null){
						if(lhs.size() > 1){
							// create a composite attribute
							String compType = "";
							for(int m = 0; m < lhs.size(); m++){
								String attr = (String)lhs.elementAt(m);
								compType = compType + attr;
								if(m < lhs.size()-1)
									compType = compType + "_";
							}
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(compType);
							attribute.setType(compType + "_Type");
							lhsType = compType + "_Type";
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
						}
						else if(lhs.size() == 1){
							// create a single Object attribute
							String attrName = (String)lhs.elementAt(0);
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(attrName);
							attribute.setType("Object");
							lhsType = "Object";
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
						}
					}
					// look at rhs
					Vector rhs = a.getLhs();
					if(rhs != null){
						if(rhs.size() > 1){
							// create a composite attribute
							String compType = "";
							for(int m = 0; m < rhs.size(); m++){
								String attr = (String)rhs.elementAt(m);
								compType = compType + attr;
								if(m < rhs.size()-1)
									compType = compType + "_";
							}
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(compType);
							attribute.setType(compType + "_Type");
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
						}
						else if(rhs.size() == 1){
							// create a single Object attribute
							String attrName = (String)lhs.elementAt(0);
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(attrName);
							attribute.setType("Object");
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
						}
					}
					// look at function
					FunctionCall function = a.getFunction();
					if(function != null){
						method = new ATagentComponentMethod(sys.atSys.getNextKey());
						String name = function.getName();
						method.setName(name);
						parameters = function.getParameters();
						String params = "";
						if(parameters != null)
						for(int m = 0; m < parameters.size(); m++){
							Parameter p = (Parameter)parameters.elementAt(m);
							String pname = p.getName();
							// add as attribute of component
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(pname);
							attribute.setType("Object");
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
							// add as parameter for method
							params = params + pname + ":Object";
							if(m < parameters.size()-1)
								params = params + ", ";
						}
						method.setParams(params);
						method.setReturnType(lhsType);
						method.setMethodView();
						if(!hasMethod(method, atComp))
							atComp.addMethod(method);
					}
				}
			}
			// look at every transition
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				
				// look at receive Event
				Event receive = t.getReceive();
				if(receive != null){
					parameters = receive.getParameters();
					if(parameters != null)
					for(int l = 0; l < parameters.size(); l++){
						Parameter p = (Parameter)parameters.elementAt(l);
						String pname = p.getName();
						attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
						attribute.setName(pname);
						attribute.setType("Object");
						attribute.setAttrView();
						if(!hasAttribute(attribute, atComp))
							atComp.addAttribute(attribute);
					}
				}
				
				// look at ReceiveEvent
				ReceiveEvent re = t.getReceiveEvent();
				if(re != null){
					Event e = re.getEvent();
					parameters = e.getParameters();
					if(parameters != null)
					for(int l = 0; l < parameters.size(); l++){
						Parameter p = (Parameter)parameters.elementAt(l);
						String pname = p.getName();
						attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
						attribute.setName(pname);
						attribute.setType("Object");
						attribute.setAttrView();
						if(!hasAttribute(attribute, atComp))
							atComp.addAttribute(attribute);
					}
				}
				
				// look at Guard - NOT DONE				
				// look at actions
				actions = t.getActions();
				if(actions != null)
				for(int l = 0; l < actions.size(); l++){
					a = (Action)actions.elementAt(l);
					lhsType = "void";
					// look at lhs
					Vector lhs = a.getLhs();
					if(lhs != null){
						if(lhs.size() > 1){
							// create a composite attribute
							String compType = "";
							for(int m = 0; m < lhs.size(); m++){
								String attr = (String)lhs.elementAt(m);
								compType = compType + attr;
								if(m < lhs.size()-1)
									compType = compType + "_";
							}
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(compType);
							attribute.setType(compType + "_Type");
							lhsType = compType + "_Type";
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
						}
						else if(lhs.size() == 1){
							// create a single Object attribute
							String attrName = (String)lhs.elementAt(0);
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(attrName);
							attribute.setType("Object");
							lhsType = "Object";
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
						}
					}
					// look at rhs
					Vector rhs = a.getLhs();
					if(rhs != null){
						if(rhs.size() > 1){
							// create a composite attribute
							String compType = "";
							for(int m = 0; m < rhs.size(); m++){
								String attr = (String)rhs.elementAt(m);
								compType = compType + attr;
								if(m < rhs.size()-1)
									compType = compType + "_";
							}
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(compType);
							attribute.setType(compType + "_Type");
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
						}
						else if(rhs.size() == 1){
							// create a single Object attribute
							String attrName = (String)lhs.elementAt(0);
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(attrName);
							attribute.setType("Object");
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
						}
					}
					// look at function
					FunctionCall function = a.getFunction();
					if(function != null){
						method = new ATagentComponentMethod(sys.atSys.getNextKey());
						String name = function.getName();
						method.setName(name);
						parameters = function.getParameters();
						String params = "";
						if(parameters != null)
						for(int m = 0; m < parameters.size(); m++){
							Parameter p = (Parameter)parameters.elementAt(m);
							String pname = p.getName();
							// remove brackets from list parameters
							if(pname.startsWith("<") && pname.endsWith(">")){
								int length = pname.length();
								pname = pname.substring(1, length-1);
							}
							// add as attribute of component
							attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
							attribute.setName(pname);
							attribute.setType("Object");
							attribute.setAttrView();
							if(!hasAttribute(attribute, atComp))
								atComp.addAttribute(attribute);
							// add as parameter for method
							params = params + pname + ":Object";
							if(m < parameters.size()-1)
								params = params + ", ";
						}
						method.setParams(params);
						method.setReturnType(lhsType);
						method.setMethodView();
						if(!hasMethod(method, atComp))
							atComp.addMethod(method);
					}
				}
				
				// look at sends
				Vector sends = t.getSends();
				if(sends != null)
				for(int l = 0; l < sends.size(); l++){
					Event e = (Event)sends.elementAt(l);
					parameters = e.getParameters();
					if(parameters != null)
					for(int m = 0; m < parameters.size(); m++){
						Parameter p = (Parameter)parameters.elementAt(m);
						String pname = p.getName();
						attribute = new ATagentComponentAttribute(sys.atSys.getNextKey());
						attribute.setName(pname);
						attribute.setType("Object");
						attribute.setAttrView();
						if(!hasAttribute(attribute, atComp))
							atComp.addAttribute(attribute);
					}
				}
				// should be no SendEvents 
			}
			// look for methods in conversation halves
			// don't need to look for attributes, because every one that
			// belongs to the component has already been prepended
			// If it wasn't prepended, then it belongs to the conversation
			convs = c.getConvs();
			if(convs != null)
			for(int k = 0; k < convs.size(); k++){
				ConversationHalf ch = (ConversationHalf)convs.elementAt(k);
				attributesAndComponents(ch, c);
			}
			// remove attributes that use the dot notation
			Vector compAttrs = atComp.getAttributes();
			if(compAttrs != null)
			for(int k = 0; k < compAttrs.size(); k++){
				ATagentComponentAttribute compAttr = (ATagentComponentAttribute)compAttrs.elementAt(k);
				String attrName = compAttr.getName();
				if(attrName.indexOf('.') != -1){
					compAttrs.removeElementAt(k);
					k--;
				}
			}
		}
	}
}
/**
 * This method adds methods to components from the conversation halves.
 * No attributes need to be added from the conversation halves.  The ones
 * that belong to the components have already been prepended with "parent.".
 * The ones that were not prepended belong to the conversation half.
 *
 * Creation date: (1/27/01 9:02:39 AM)
 */
void attributesAndComponents(ConversationHalf ch, Component c) {
	StateTable 	st = ch.getStateTable();
	Vector 		states;
	State 		s;
	Vector 		transitions;
	Transition 	t;
	Vector 		actions;
	Action 		a;
	ATagentComponent atComp = c.getAtComponent();
	ATagentComponentAttribute attribute;
	ATagentComponentMethod method;
	String lhsType;
	Vector parameters;

	
	states = st.getStates();
	transitions = st.getTransitions();
	// look at every state
	if(states != null)
	for(int k = 0; k < states.size(); k++){
		s = (State)states.elementAt(k);
		actions = s.getActions();
		if(actions != null)
		for(int l = 0; l < actions.size(); l++){
			a = (Action)actions.elementAt(l);
			lhsType = "void";
			// look at lhs
			Vector lhs = a.getLhs();
			if(lhs != null){
				if(lhs.size() > 1){
					// create a composite attribute
					String compType = "";
					for(int m = 0; m < lhs.size(); m++){
						String attr = (String)lhs.elementAt(m);
						if(attr.startsWith("parent."))
							attr = attr.substring(7);
						compType = compType + attr;
						if(m < lhs.size()-1)
							compType = compType + "_";
					}
					lhsType = compType + "_Type";
				}
				else if(lhs.size() == 1){
					// create a single Object attribute
					lhsType = "Object";
				}
			}
			
			// look at function
			FunctionCall function = a.getFunction();
			if(function != null){
				method = new ATagentComponentMethod(sys.atSys.getNextKey());
				String name = function.getName();
				method.setName(name);
				parameters = function.getParameters();
				String params = "";
				if(parameters != null)
				for(int m = 0; m < parameters.size(); m++){
					Parameter p = (Parameter)parameters.elementAt(m);
					String pname = p.getName();
					if(pname.startsWith("parent."))
						pname = pname.substring(7);
					// add as parameter for method
					params = params + pname + ":Object";
					if(m < parameters.size()-1)
						params = params + ", ";
				}
				method.setParams(params);
				method.setReturnType(lhsType);
				method.setMethodView();
				if(!hasMethod(method, atComp))
					atComp.addMethod(method);
			}
		}
	}
	// look at every transition
	if(transitions != null)
	for(int k = 0; k < transitions.size(); k++){
		t = (Transition)transitions.elementAt(k);
		
		// look at actions
		actions = t.getActions();
		if(actions != null)
		for(int l = 0; l < actions.size(); l++){
			a = (Action)actions.elementAt(l);
			lhsType = "void";
			// look at lhs
			Vector lhs = a.getLhs();
			if(lhs != null){
				if(lhs.size() > 1){
					// create a composite attribute
					String compType = "";
					for(int m = 0; m < lhs.size(); m++){
						String attr = (String)lhs.elementAt(m);
						if(attr.startsWith("parent."))
							attr = attr.substring(7);
						compType = compType + attr;
						if(m < lhs.size()-1)
							compType = compType + "_";
					}
					lhsType = compType + "_Type";
				}
				else if(lhs.size() == 1){
					// create a single Object attribute
					lhsType = "Object";
				}
			}
			// look at function
			FunctionCall function = a.getFunction();
			if(function != null){
				method = new ATagentComponentMethod(sys.atSys.getNextKey());
				String name = function.getName();
				method.setName(name);
				parameters = function.getParameters();
				String params = "";
				if(parameters != null)
				for(int m = 0; m < parameters.size(); m++){
					Parameter p = (Parameter)parameters.elementAt(m);
					String pname = p.getName();
					if(pname.startsWith("parent."))
						pname = pname.substring(7);
					// add as parameter for method
					params = params + pname + ":Object";
					if(m < parameters.size()-1)
						params = params + ", ";
				}
				method.setParams(params);
				method.setReturnType(lhsType);
				method.setMethodView();
				if(!hasMethod(method, atComp))
					atComp.addMethod(method);
			}
		}
	}
}
/**
 * This method checks to see if a component already has the attribute.
 *
 * Creation date: (1/27/01 9:02:39 AM)
 */
boolean hasAttribute(ATagentComponentAttribute a, ATagentComponent c) {
	Vector attributes = c.getAttributes();
	if(attributes != null)
	for(int i = 0; i < attributes.size(); i++){
		ATagentComponentAttribute attr = (ATagentComponentAttribute)attributes.elementAt(i);
		if((attr.getName() != null)
			&& (attr.getName().equals(a.getName()))
			&& (attr.getType() != null)
			&& (attr.getType().equals(a.getType())))
			return true;
	}
	// not found
	return false;
}
/**
 * This method checks to see if a component already has the method.
 *
 * Creation date: (1/27/01 9:02:39 AM)
 */
boolean hasMethod(ATagentComponentMethod m, ATagentComponent c) {
	Vector methods = c.getMethods();
	if(methods != null)
	for(int i = 0; i < methods.size(); i++){
		ATagentComponentMethod meth = (ATagentComponentMethod)methods.elementAt(i);
		if((meth.getName() != null)
			&& (meth.getName().equals(m.getName()))){
				String params1 = meth.getParams();
				String params2 = m.getParams();
				if((params1 == null) && (params2 == null))
					return true;
				if((params1 != null) && (params2 != null)){
					StringTokenizer st = new StringTokenizer(params1, ",");
					StringTokenizer st2 = new StringTokenizer(params2, ",");
					if(st.countTokens() == st2.countTokens())
						return true;
				}
			}
	}
	// not found
	return false;
}
/**
 * Insert the method's description here.
 * Creation date: (1/27/01 9:02:39 AM)
 */
void newMethod() {
}
/**
 * This method is simply used to remove all existing states from the given
 * state table
 *
 * Creation date: (12/5/00 11:17:18 AM)
 */
void removeStates(ATstatetable st) {
	Enumeration e = st.returnStates();
	Vector elements = new Vector();
	while(e.hasMoreElements())
		elements.add(e.nextElement());
	
	for(int i = 0; i < elements.size(); i++){
		ATtableElement te = (ATtableElement)elements.elementAt(i);
		st.removeState(te);
	}
}
/**
 * This method contains the flow of events throughout the phase
 *
 * Creation date: (12/5/00 11:17:18 AM)
 */
public synchronized void run() {
	while(!sys.phaseTwoDone){
		try{
			sleep(100);
		}catch(Exception e){}
	}
	
	// prepare conversations that exit to multiple states
	Transform33 t33 = new Transform33(sys);

	// add actions to transitions when a received parameter is
	// used outside the current conversation
	Transform36 t36 = new Transform36(sys);

	// prepend parameters used in actions inside of states or transitions
	// that are also used outside of the current conversation
	Transform38 t38 = new Transform38(sys);
	Transform40 t40 = new Transform40(sys);
	Transform42 t42 = new Transform42(sys);
	
	// create the transition with action for conversations
	Transform44 t44 = new Transform44(sys);
	Transform45 t45 = new Transform45(sys);
	Transform46 t46 = new Transform46(sys);

	// create the conversation halves for the components / conversations
	Transform47 t47 = new Transform47(sys);

	// add states to the conversation halves
	Transform48 t48 = new Transform48(sys);

	// add transitions to the conversation halves
	Transform49 t49 = new Transform49(sys);

	// change the conversation external events to internal events
	Transform51 t51 = new Transform51(sys);
	
	// set the ATconversation statetables to the atStateTables of the conversation halfs
	setConversationStateTables();

	// add component attributes and methods
	attributesAndComponents();
	
	if(debug){
		System.out.println();
		System.out.println("After phase 3: ");
		sys.show();
	}
}
/**
 * This method is used to create the state tables associated with each
 * conversation as defined by the conversation halves.
 *
 * Creation date: (12/5/00 11:17:18 AM)
 */
void setConversationStateTables() {
	Vector conversations = sys.conversations;
	if(conversations != null)
	for(int i = 0; i < conversations.size(); i++){
		Conversation c = (Conversation)conversations.elementAt(i);
		ConversationHalf initiator = c.getInitiator();
		ConversationHalf responder = c.getResponder();
		StateTable initStateTable = initiator.getStateTable();
		ATstatetable atInitTable = initStateTable.getAtStateTable();
		StateTable respStateTable = responder.getStateTable();
		ATstatetable atRespTable = respStateTable.getAtStateTable();		
		ATconversation atConv = c.getAtConversation();
		ATstatetable init = atConv.getStateTable(1);
		removeStates(init);
		// add the states from atInitTable to init
		Enumeration e = atInitTable.returnStates();
		while(e.hasMoreElements()){
			ATtableElement te = (ATtableElement)e.nextElement();
			//if(!(te instanceof ATstartState) && !(te instanceof ATendState))
				init.addState(te);
		}
		// add the transitions from atInitTable to init
		e = atInitTable.returnTrans();
		while(e.hasMoreElements()){
			ATtransition t = (ATtransition)e.nextElement();
			init.addTransition(t);
		}
		
		ATstatetable resp = atConv.getStateTable(2);
		removeStates(resp);
		// add the states from atInitTable to resp
		e = atRespTable.returnStates();
		while(e.hasMoreElements()){
			ATtableElement te = (ATtableElement)e.nextElement();
			//if(!(te instanceof ATstartState) && !(te instanceof ATendState))
				resp.addState(te);
		}
		// add the transitions from atInitTable to resp
		e = atRespTable.returnTrans();
		while(e.hasMoreElements()){
			ATtransition t = (ATtransition)e.nextElement();
			resp.addTransition(t);
		}
		
	}
}
}
