package Transforms;

/**
 * This class is used to represent a state in a state table
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import system.ATtableElement;
import system.ATstate;
import system.ATstateGRFX;
import system.ATstartState;
import system.ATstartStateGRFX;
import system.ATendState;
import system.ATendStateGRFX;
 
public class State {
	private ATtableElement	element = null;
	private String	name = null;			// name of the state
	private Vector	actions = null;			// SEQUENCE of actions
	private Vector	conversations = null;	// SET of conversations the state belongs to
	private TSystem sys = null;
/**
 * Insert the method's description here.
 * Creation date: (11/11/00 12:55:42 PM)
 */
public State(String n, TSystem tSys) {
	name = n;
	sys = tSys;
}
/**
 * Insert the method's description here.
 * Creation date: (11/4/00 11:59:09 PM)
 */
public State(ATtableElement t, Vector a, TSystem tSys) {
	name = t.getName();
	actions = a;
	element = t;
	sys = tSys;
}
/**
 * This method will add its actions to the actions of its ATtableElement
 *
 * Creation date: (11/29/00 1:30:54 PM)
 */
public void addATActions() {
	ATstate atState = (ATstate)element;
	if(actions != null){
		Vector acts = getActionsAsStrings();
		for(int i = 0; i < acts.size(); i++){
			String a = (String)acts.elementAt(i);
			atState.addAction(a);
		}
		atState.setActions(acts);
	}	
}
/**
 * This method will create and return a logically equivalent state to this one
 *
 * Creation date: (11/15/00 5:21:45 PM)
 */
public State copy() {
	State s = new State(name, sys);
	if((actions != null) && (actions.size() > 0)){
		Vector actions2 = new Vector();
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			actions2.add(a.copy());
		}
		s.actions = actions2;
	}

	// now create the ATstate
	if(name.equals("StartState")){
		// already has a start state by default
		// but this one needs to be there instead for position
		ATstartState old = (ATstartState)element;
		ATstartStateGRFX oldGrfx = (ATstartStateGRFX)old.getGrfx();
		double version = old.getVersion();

		// create a new ATstartState
		ATstartState newState = new ATstartState(version, sys.atSys.getNextKey());

		// copy the coordinates of the old graphics object
		int[] c = oldGrfx.getCoords();
		newState.setPos(c[0], c[1]);

		s.element = newState;
	}
	else if(name.equals("EndState")){
		// already has an end state by default
		// but this one needs to be there instead for position
		ATendState old = (ATendState)element;
		ATendStateGRFX oldGrfx = (ATendStateGRFX)old.getGrfx();
		double version = old.getVersion();

		// create a new ATendState
		ATendState newState = new ATendState(version, sys.atSys.getNextKey());

		// copy the coordinates of the old graphics object
		int[] c = oldGrfx.getCoords();
		newState.setPos(c[0], c[1]);

		s.element = newState;
	}
	else{
		// get the original ATstate and its graphics object
		ATstate oldATstate = (ATstate)element;
		ATstateGRFX oldGrfx = (ATstateGRFX)oldATstate.getGrfx();
		double version = oldATstate.getVersion();
		
		// create a new ATstate
		ATstate newATstate = new ATstate(version, sys.atSys.getNextKey());
		
		// copy the coordinates of the old graphics object
		int[] c = oldGrfx.getCoords();
		newATstate.setCoordinates(c[0], c[1], c[2], c[3]);
		
		// set the new States element to the newly created ATstate
		s.element = newATstate;
		
		// make a copy of the actions and set give it to the new ATstate
		//Vector oldActions = oldATstate.getActions();
		//Vector newActions = (Vector)oldActions.clone();
		//newATstate.setActions(newActions);
		s.addATActions();
			
		// make a copy of the state's name
		String newName = String.valueOf(oldATstate.getName());
		newATstate.setName(newName);
	}
		
	return s;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:47:58 AM)
 * @return java.util.Vector
 */
public java.util.Vector getActions() {
	return actions;
}
/**
 * This method returns the state's actions as a vector of strings
 *
 * Creation date: (11/22/00 8:39:31 PM)
 */
public Vector getActionsAsStrings() {
	// Vector of strings representing the actions
	Vector acts = new Vector();		

	if(actions != null)
	for(int i = 0; i < actions.size(); i++){
		Action a = (Action)actions.elementAt(i);
		String s = a.toString();
		acts.add(s);
	}

	acts.insertElementAt("", 0);
	
	return acts;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:47:58 AM)
 * @return java.util.Vector
 */
public java.util.Vector getConversations() {
	return conversations;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:47:58 AM)
 * @return system.ATtableElement
 */
public system.ATtableElement getElement() {
	return element;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:47:58 AM)
 * @return java.lang.String
 */
public java.lang.String getName() {
	return name;
}
/**
 * This method determines if it has all of the conversations that are in the provided vector
 *
 * Creation date: (11/22/00 8:39:31 PM)
 */
public boolean hasAllConversations(Vector convs) {
	if(convs == null)
	return true;

	if(conversations == null)
	return false;

	// loop through and test each cid in cids
	for(int i = 0; i < convs.size(); i++){
		Conversation conv = (Conversation)convs.elementAt(i);
		if(!conversations.contains(conv))
			return false;
	}
	
	// default
	return true;	
}
/**
 * This method determines if the given conversation is in its conversation set
 *
 * Creation date: (11/29/00 1:30:54 PM)
 */
public boolean hasConversation(Conversation conv) {
	if((conversations != null) && conversations.contains(conv))
		return true;

	// default
	return false;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:47:58 AM)
 * @param newActions java.util.Vector
 */
public void setActions(java.util.Vector newActions) {
	actions = newActions;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:47:58 AM)
 * @param newConversations java.util.Vector
 */
public void setConversations(java.util.Vector newConversations) {
	conversations = newConversations;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:47:58 AM)
 * @param newElement system.ATtableElement
 */
public void setElement(system.ATtableElement newElement) {
	element = newElement;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 8:47:58 AM)
 * @param newName java.lang.String
 */
public void setName(java.lang.String newName) {
	name = newName;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/6/00 1:17:06 PM)
 */
public void show() {
	System.out.println("State: " + name);
	if(actions != null){
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			if(a != null)
				a.show();
		}
	}
	if(conversations != null){
		System.out.print("\tconversations: ");
		for(int i = 0; i < conversations.size(); i++){
			Conversation conv = (Conversation)conversations.elementAt(i);
			String cname = conv.getName();
			System.out.print(cname);
			if(i+1 < conversations.size())
			System.out.print(", ");
		}
		System.out.println();
	}
}
/**
 * Returns a string representation of the state
 *
 * Creation date: (11/6/00 1:17:06 PM)
 */
public String toString() {
	String s = "State: " + name + "\n";
	if(actions != null){
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			if(a != null)
				s = s + a.toString() + "\n";
		}
	}
	return s;
}
/**
 * This method will create and return a state that corresponds to the provided ATtableElement
 *
 * Creation date: (11/3/00 3:57:59 PM)
 */
public static State translateIn(ATtableElement atElement, TSystem tSys) {
	State 	state;
	Vector	atActions;
	String	atAction;
	Action	action;
	Vector	actions = null;

	if(atElement instanceof ATstate){
		ATstate atState = (ATstate)atElement;
		atActions = atState.getActions();
		if((atActions != null) && (atActions.size() > 0)){
			for(int i = 0; i < atActions.size(); i++){
				atAction = (String)atActions.elementAt(i);			
				action = Action.translateIn(atAction);
				if(action != null){
					if(actions == null)
						actions = new Vector();
					actions.add(action);
				}	
			}
		}	
		state = new State(atState, actions, tSys);
		return state;
	}

	else if(atElement instanceof ATstartState){
		state = new State(atElement, null, tSys);
		return state;
	}

	else if(atElement instanceof ATendState){
		state = new State(atElement, null, tSys);
		return state;
	}

	else{
		System.out.println("Error translating state");
		return null;
	}
}
}
