package Transforms;

/**
 * This class is used to represent state tables, or state diagrams
 * StateTables are used in tasks, components, and conversations
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import java.util.Enumeration;
import system.ATstatetable;
import system.ATtableElement;
import system.ATtransition;
 
public class StateTable {
	private ATstatetable atStateTable = null;
	private Vector	states = null;			// SET of States
	private Vector	transitions = null; 	// SET of Transitions
	private int		convName;
	private TSystem	sys = null;
/**
 * Insert the method's description here.
 * Creation date: (11/3/00 4:05:45 PM)
 */
public StateTable(ATstatetable st, Vector s, Vector t, TSystem tSys) {
	states = s;
	transitions = t;
	atStateTable = st;
	convName = 1;
	sys = tSys;
}
/**
 * This method will add a state to the state table, as well as the state's ATtableElement
 * to the ATstatetable
 *
 * Creation date: (11/22/00 12:18:12 PM)
 */
public void addState(State s) {
	if(states == null)
		states = new Vector();
	states.add(s);
	if(s.getElement() != null)
		atStateTable.addState(s.getElement());
}
/**
 * This method will add a transition to the state table, as well as the transition's ATtransition
 * to the ATstatetable
 *
 * Creation date: (11/22/00 12:18:12 PM)
 */
public void addTransition(Transition t) {
	if(transitions == null)
		transitions = new Vector();

	transitions.add(t);
	if(t.getAtTransition() != null)
		atStateTable.addTransition(t.getAtTransition());	
}
/**
 * This method will create and return a logically equivalent statetable to this one
 *
 * Creation date: (11/10/00 12:16:42 PM)
 */
public StateTable copy() {
	// create a new ATstatetable
	ATstatetable newTable = new ATstatetable(atStateTable.getVersion(), sys.atSys.getNextKey());
	// clear the start and end states
	Enumeration e = newTable.returnStates();
	Vector sv = new Vector();
	while(e.hasMoreElements()){
		sv.add(e.nextElement());
	}
	for(int i = 0; i < sv.size(); i++){
		ATtableElement te = (ATtableElement)sv.elementAt(i);
		newTable.removeState(te);
	}

	// create a new StateTable	
	StateTable st = new StateTable(newTable, null, null, sys);
		
	// make a copy of all of the states
	if(states != null)
	for(int i = 0; i < states.size(); i++){
		State s = (State)states.elementAt(i);
		State s2 = s.copy();
		st.addState(s2);
	}

	// make a copy of all of the transitions
	if(transitions != null)
	for(int i = 0; i < transitions.size(); i++){
		Transition t = (Transition)transitions.elementAt(i);
		Transition t2 = t.copy();
		st.addTransition(t2);
	}

	// match up transition from and to states to the new states
	Vector transitions2 = st.getTransitions();
	Vector states2 = st.getStates();
	if(transitions2 != null)
	for(int i = 0; i < transitions2.size(); i++){
		Transition t = (Transition)transitions2.elementAt(i);
		ATtransition atTrans = t.getAtTransition();
		ATtableElement from = null;
		ATtableElement to = null;		
		for(int j = 0; j < states2.size(); j++){
			State s = (State)states2.elementAt(j);
			if(s.getName().equals(t.getFromName())){
				t.setFrom(s);
				// set from to the ATtableElement of s
				from = s.getElement();
			}
			if(s.getName().equals(t.getToName())){
				t.setTo(s);
				// set to to the ATtableElement of s
				to = s.getElement();
			}
		}
		// now set the from and to of the ATtransitions
		atTrans.setCurrentState(from);
		atTrans.setNextState(to);
		atTrans.setCoordinates(from, to);
	}

	return st;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:21:33 AM)
 * @return system.ATstatetable
 */
public system.ATstatetable getAtStateTable() {
	return atStateTable;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:21:33 AM)
 * @return java.util.Vector
 */
public java.util.Vector getStates() {
	return states;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:21:33 AM)
 * @return java.util.Vector
 */
public java.util.Vector getTransitions() {
	return transitions;
}
/**
 * This method will determine if the state table has a state with the given name
 *
 * Creation date: (11/11/00 12:53:30 PM)
 */
public boolean hasState(String n) {
	if(states != null)
	for(int i = 0; i < states.size(); i++){
		State s = (State)states.elementAt(i);
		if(s.getName().equals(n))
			return true;
	}
	return false;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:21:33 AM)
 * @param newAtStateTable system.ATstatetable
 */
public void setAtStateTable(system.ATstatetable newAtStateTable) {
	atStateTable = newAtStateTable;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:21:33 AM)
 * @param newStates java.util.Vector
 */
public void setStates(java.util.Vector newStates) {
	states = newStates;
}
/**
 * Insert the method's description here.
 * Creation date: (12/6/00 9:21:33 AM)
 * @param newTransitions java.util.Vector
 */
public void setTransitions(java.util.Vector newTransitions) {
	transitions = newTransitions;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/6/00 1:13:52 PM)
 */
public void show() {
	System.out.println("State Table:");
	System.out.println("States:");
	for(int i = 0; i < states.size(); i++){
		State s = (State)states.elementAt(i);
		s.show();
	}
	System.out.println("Transitions:");
	for(int i = 0; i < transitions.size(); i++){
		Transition t = (Transition)transitions.elementAt(i);
		System.out.println();
		t.show();
	}
}
/**
 * Returns a string representation of the state table
 *
 * Creation date: (11/6/00 1:13:52 PM)
 */
public String toString() {
	String st = "State Table\n";
	st = st + "States:\n";
	for(int i = 0; i < states.size(); i++){
		State s = (State)states.elementAt(i);
		st = st + s.toString();
	}
	st = st + "Transitions:\n";
	for(int i = 0; i < transitions.size(); i++){
		Transition t = (Transition)transitions.elementAt(i);
		st = st + t.toString() + "\n\n";
	}
	return st;
}
/**
 * This method will create and return a StateTable that corresponds to the given ATstatetable
 *
 * Creation date: (11/3/00 3:46:32 PM)
 */
public static StateTable translateIn(ATstatetable atStateTable, TSystem tSys) {
	Enumeration 	eStates;
	Enumeration 	eTransitions;
	ATtableElement	atState;
	State			state;
	ATtransition	atTransition;
	Transition		transition;
	
	eStates = atStateTable.returnStates();
	eTransitions = atStateTable.returnTrans();

	// build vector of states
	Vector states = new Vector();
	while(eStates.hasMoreElements()){
		atState = (ATtableElement)eStates.nextElement();
		state = State.translateIn(atState, tSys);
		states.add(state);
	}

	// build vector of transitions
	Vector transitions = new Vector();
	while(eTransitions.hasMoreElements()){
		atTransition = (ATtransition)eTransitions.nextElement();
		transition = Transition.translateIn(atTransition, tSys);
		transitions.add(transition);
	}

	// need to match up the from and to states of transitions
	for(int i = 0; i < transitions.size(); i++){
		transition = (Transition)transitions.elementAt(i);
		for(int j = 0; j < states.size(); j++){
			state = (State)states.elementAt(j);
			if(transition.getFromName().equals(state.getName())){
				transition.setFrom(state);
			}
			if(transition.getToName().equals(state.getName())){
				transition.setTo(state);
			}
		}
	}
	
	return new StateTable(atStateTable, states, transitions, tSys);
}
}
