package Transforms;

/**
 * This class is used to store all of the information in the system
 * throughout the transformation process
 *
 * Creation date: (11/3/00 10:50:21 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;
import java.util.Enumeration;
import system.*;
 
public class TSystem {
	int 			convName;
	Vector			roles = null;
	Vector			tasks = null;
	Vector			protocols = null;
	Vector			agents = null;
	Vector			conversations = null;
	ATsystem		atSys = null;
	Editor.ATeditor	editor = null;
	double			version;
	public boolean phaseOneStarted = false;
	public boolean phaseTwoStarted = false;
	public boolean phaseOneDone = false;
	public boolean phaseTwoDone = false;
/**
 * Insert the method's description here.
 * Creation date: (11/3/00 3:15:44 PM)
 */
public TSystem(ATsystem s, Editor.ATeditor ed, double v) {
	atSys = s;
	roles = new Vector();
	tasks = new Vector();
	protocols = new Vector();
	agents = new Vector();
	conversations = new Vector();
	editor = ed;
	version = v;
	loadATsystem();
	convName = 1;
	//show();
}
/**
 * This method makes a copy of the TSystem and returns it
 *
 * Creation date: (11/8/00 10:55:02 AM)
 */
public TSystem copy() {
	TSystem t = new TSystem(atSys, editor, version);
	t.roles = roles;
	t.tasks = tasks;
	t.protocols = protocols;
	t.agents = agents;
	return t;
}
/**
 * This method will return the conversation in the system with the given name
 *
 * Creation date: (12/1/00 2:35:36 PM)
 */
public Conversation getConv(String cname) {
	if(conversations != null)
	for(int i = 0; i < conversations.size(); i++){
		Conversation c = (Conversation)conversations.elementAt(i);
		if(cname.equals(c.getName()))
			return c;
	}
	return null;
}
/**
 * This method will return a conversation name that is unique to the state table
 * but not necessarily to the entire system
 *
 * Creation date: (11/13/00 1:02:41 PM)
 */
public String getConvName() {
	String c = "Conversation" + String.valueOf(convName);
	convName++;
	return c;
}
/**
 * This method will return a new conversation name, based on the string given
 * that is unique throughout the system.
 *
 * Creation date: (12/1/00 2:35:36 PM)
 */
public String getNewConversationName(String s) {
	for(int i = 1; true; i++){
		String newConvName = s + "_" + String.valueOf(i);
		if(!hasConv(newConvName))
		return newConvName;
	}
}
/**
 * This method will determine if there is a conversation in the system that
 * has the name given
 *
 * Creation date: (12/1/00 2:35:36 PM)
 */
public boolean hasConv(String cname) {
	if(conversations != null)
	for(int i = 0; i < conversations.size(); i++){
		Conversation c = (Conversation)conversations.elementAt(i);
		if(cname.equals(c.getName()))
			return true;
	}
	return false;
}
/**
 * This method is used to load agents into the system
 *
 * Creation date: (11/6/00 12:47:35 PM)
 */

void loadAgents(Enumeration e) {
	ATagent		atAgent;
	Agent		agent;
	Vector vecStates 		= new Vector();
	Vector vecTransitions 	= new Vector();
	Vector vecNewComps		= new Vector();
	
	while(e.hasMoreElements()){
		atAgent = (ATagent)e.nextElement();
		String name = atAgent.getName();
		Vector atRoles = atAgent.getRoles();
		Vector newRoles = new Vector();
		for(int i = 0; i < atRoles.size(); i++){
			ATRole r = (ATRole)atRoles.elementAt(i);
			for(int j = 0; j < roles.size(); j++){
				Role role = (Role)roles.elementAt(j);
				if(r.getName() == role.getName()){
					newRoles.add(role);
				}
			}
		}

		vecNewComps.clear();

		agent = new Agent(atAgent, newRoles);
		
		ATarchitecture atArch = atAgent.getArchitecture();

		if (atArch.getComponentCount() != 0) {
			
			Enumeration eComps = atArch.returnComponents();

			while(eComps.hasMoreElements()) {
					
				ATagentComponent atAgentComp = (ATagentComponent)eComps.nextElement();
				ATstatetable atStateTable = atAgentComp.getStateTable();

				StateTable st = new StateTable(atStateTable,vecStates,vecTransitions, this);
				st = StateTable.translateIn(atStateTable, this);
				
				Component compNew = new Component(atAgentComp.getName(),st,agent);
				compNew.setAtComponent(atAgentComp);
				vecNewComps.add(compNew);
			}
		}
		
		agent.setComponents(vecNewComps);
		agents.add(agent);		
	}
}
/**
 * This method will load translate in the system, based on the current ATsystem
 *
 * Creation date: (11/3/00 3:19:03 PM)
 */
void loadATsystem() {
	loadRoles(atSys.returnRoles());
	loadProtocols(atSys.returnProtocols());
	loadAgents(atSys.returnAgents());
}
/**
 * This method is used to load protocols into the system
 *
 * Creation date: (11/6/00 12:31:39 PM)
 */
void loadProtocols(Enumeration e) {
	ATProtocol 	atProtocol;
	Protocol	protocol;
	ATTask		initiator, responder;
	Task		init = null, resp = null;
	String		name;
	
	while(e.hasMoreElements()){
		atProtocol = (ATProtocol)e.nextElement();
		initiator = atProtocol.getInitiator();
		responder = atProtocol.getResponder();
		name = atProtocol.getName();

		for(int i = 0; i < tasks.size(); i++){
			Task t = (Task)tasks.elementAt(i);
			if(t.getName().equals(initiator.getName())){
				init = t;
			}
			if(t.getName().equals(responder.getName())){
				resp = t;
			}
		}

		protocol = new Protocol(atProtocol, init, resp);
				
		protocols.add(protocol);
	}
}
/**
 * This method is used to load roles into the system
 *
 * Creation date: (11/3/00 3:25:50 PM)
 */
void loadRoles(Enumeration e) {
	ATRole 	atRole;
	Role 	role;
	while(e.hasMoreElements()){
		atRole = (ATRole)e.nextElement();
		role = new Role(atRole);
		roles.add(role);
		Vector atTasks = atRole.getTasks();
		loadTasks(atTasks.elements(), role);		
	}
}
/**
 * This method is used to load tasks into the system
 *
 * Creation date: (11/3/00 3:40:45 PM)
 */
void loadTasks(Enumeration e, Role r) {
	ATTask atTask;
	ATstatetable atStateTable;
	StateTable stateTable;
	Vector roleTasks = new Vector();
	
	while(e.hasMoreElements()){
		atTask = (ATTask)e.nextElement();
		atStateTable = atTask.getStateTable();
		stateTable = StateTable.translateIn(atStateTable, this);
		tasks.add(new Task(atTask, stateTable));
		roleTasks.add(new Task(atTask, stateTable));
	}
	r.setTasks(roleTasks);
}
/**
 * This method will return true if there are no two tasks in the system with the same name
 *
 * Creation date: (11/10/00 1:52:06 PM)
 */
public boolean noDuplicateTasks() {
	if(tasks != null){
		for(int i = 0; i < tasks.size(); i++){
			Task t = (Task)tasks.elementAt(i);
			for(int j = 0; j < tasks.size(); j++){
				Task t2 = (Task)tasks.elementAt(i);
				if((t != t2) && (t.getName().equals(t2.getName()))){
					return false;
				}
			}
		}
	}
	return true;
}
/**
 * This method will return true if there is at least one agent playing every
 * role as defined in the role model
 *
 * Creation date: (11/10/00 11:02:20 AM)
 */
public boolean rolesCovered() {
	boolean covered;
	
	for(int i = 0; i < roles.size(); i++){
		Role r = (Role)roles.elementAt(i);
		covered = false;
		for(int j = 0; j < agents.size(); j++){
			Agent a = (Agent)agents.elementAt(j);
			Vector agentRoles = a.getRoles();
			if((agentRoles != null) && (agentRoles.contains(r))){
				covered = true;
			}
		}
		if(!covered){
			System.out.println();
			System.out.println("Role " + r.getName() + " is not played by any agent.");
			return false;
		}
	}
	
	return true;
}
/**
 * Show method used for debugging
 *
 * Creation date: (11/6/00 12:58:41 PM)
 */
public void show() {
	// show Roles
	System.out.println();
	System.out.println();
	System.out.println("***************************");
	System.out.println("*** Roles in the System ***");
	System.out.println("***************************");	
	for(int i = 0; i < roles.size(); i++){
		Role r = (Role)roles.elementAt(i);
		r.show();
	}

	// show Protocols
	System.out.println();
	System.out.println("*******************************");
	System.out.println("*** Protocols in the System ***");
	System.out.println("*******************************");	
	for(int i = 0; i < protocols.size(); i++){
		Protocol p = (Protocol)protocols.elementAt(i);
		p.show();
	}

	// show Agents
	System.out.println();
	System.out.println("****************************");
	System.out.println("*** Agents in the System ***");
	System.out.println("****************************");	
	for(int i = 0; i < agents.size(); i++){
		Agent a = (Agent)agents.elementAt(i);
		System.out.println();
		a.show();
	}
}
}
