package Transforms;

/**
 * This is the abstract Transform class that all transformations will
 * inherit from
 *
 * Creation date: (11/3/00 10:18:53 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;

public abstract class Transform {
	protected String rationale;
	protected boolean debug = false;
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 8:47:08 AM)
 */
public Transform() {
	rationale = "Duh!";
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	return false;
}
/**
 * This method must be overridden by the concrete subclass
 * to actually perform the transform
 *
 * Creation date: (11/6/00 8:56:21 AM)
 */
public abstract void execute(Object params);
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 8:48:19 AM)
 */
public String getRationale() {
	return rationale;
}
/**
 * This method must be overridden by the concrete subclass
 * to replay the transform based on stored target an params
 *
 * Creation date: (11/6/00 8:56:21 AM)
 */
public abstract void replay(Object top);
/**
 * Insert the method's description here.
 * Creation date: (11/6/00 8:48:19 AM)
 */
public void setRationale(String r) {
	rationale = r;
}
/**
 * This method must be overridden by the concrete subclass
 * to undo the transform if possible
 *
 * Creation date: (11/6/00 8:56:21 AM)
 */
public abstract void unDo(Object params);
}
