package Transforms;

/**
 * This class represents transformations 12-18 in the transformation process.
 * This transform is used to determine the protocol set for transitions.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;

public class Transform12 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform12(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * Insert the method's description here.
 * Creation date: (12/28/00 1:37:46 PM)
 */
public boolean allDeterminedIn(Transition t, StateTable st) {
	Vector transitions = st.getTransitions();
	Transition t2;

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if((t2.getTo() == t.getFrom()) && !t2.isDeterminedProtocols())
			return false;
	}
	
	return true;
}
/**
 * Insert the method's description here.
 * Creation date: (12/28/00 1:37:46 PM)
 */
public boolean allDeterminedOut(Transition t, Transition t1, StateTable st) {
	Vector transitions = st.getTransitions();
	Transition t2;

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if((t2 != t)  
			&& (t2.getFrom() == t1.getTo()) 
			&& !t2.isDeterminedProtocols())
				return false;
	}
	
	return true;
}
/**
 * This method will determine if for every protocol in t's protocols vector if there
 * is at least one transition into it's from state that has that protocol
 *
 * Creation date: (11/20/00 11:32:31 AM)
 */
public boolean allFoundIn(Transition t, StateTable st) {
	Vector		transitions = st.getTransitions();
	Transition	t2;
	Vector		protocols = t.getProtocols();
	Protocol 	p;
	boolean		found;
	
	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		found = false;
		for(int j = 0; j < transitions.size(); j++){
			t2 = (Transition)transitions.elementAt(j);
			if((t2 != t) && (t2.getTo() == t.getFrom())){
				Vector protocolsIn = t2.getProtocols();
				if((protocolsIn != null) && protocolsIn.contains(p)){
					// p was found
					found = true;					
				}
			}
		}
		// if p wasn't found in at least one of the protocols in
		// a transition in, then return false
		if(!found)
			return false;
	}

	return true;
}
/**
 * This method will determine if all of the transitions in the vector have had
 * their protocols determined
 *
 * Creation date: (11/12/00 3:13:59 PM)
 */
public boolean allLabeled(Vector transitions) {
	Transition t;
	for(int i = 0; i < transitions.size(); i++){
		t = (Transition)transitions.elementAt(i);
		if(!t.isDeterminedProtocols())
			return false;
	}
	return true;
}
/**
 * This method will determine if every transition leaving the to state of the given
 * transition have all of t's protocols
 *
 * Creation date: (11/20/00 11:32:31 AM)
 */
public boolean allMatchOut(Transition t, Transition t1, StateTable st) {
	Vector 		transitions = st.getTransitions();
	Transition 	t2;
	Vector		protocols = t1.getProtocols();
	Protocol	p;
	
	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if((t2 != t) && (t2 != t1) && (t2.getFrom() == t1.getTo())){
			Vector protocolsOut = t2.getProtocols();
			if(protocolsOut != null){
				for(int j = 0; j < protocols.size(); j++){
					p = (Protocol)protocols.elementAt(j);
					if(!protocolsOut.contains(p))
						return false;
				}
			}
		}
	}

	// default - didn't find any that didn't match up
	return true;
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method determines if the transition has no events on it.
 *
 * Creation date: (11/20/00 11:10:44 AM)
 */
public boolean automatic(Transition t) {
	if((t.getReceive() == null)
		&& (t.getReceiveEvent() == null)
		&& (t.getSends() == null)
		&& (t.getSendEvents() == null))
		return true;
	else
		return false;
}
/**
 * This method determines the protocol set for transitions
 * that have no external events
 *
 * Creation date: (11/11/00 2:52:41 PM)
 */
public void determine(Transition t, StateTable st) {
	Vector	transitions = st.getTransitions();
	Vector	prots = new Vector();
	boolean	allSame = true;
	boolean determined = true;
	Transition t2;

	State s = t.getFrom();
	if(s.getName().equals("StartState")){
		allSame = false;
		t.setProtocols(null);
		t.setDeterminedProtocols(true);
	}
	else{	
		for(int i = 0; i < transitions.size(); i++){
			t2 = (Transition)transitions.elementAt(i);
			if(t2 != t){
				// case 1 - there's a transition into the from state with
				// a null protocols vector and either has 
				// events or has already been checked
				if((t2.getTo() == t.getFrom()) 
					&& (t2.getProtocols() == null)){
						if(!t2.isDeterminedProtocols())
							determined = false;
						else
							allSame = false;
							t.setProtocols(null);
							t.setDeterminedProtocols(true);
					}
				// case 2 - there's a transition out of the from state with 
				// a null protocols vector that has events
				else if((t2.getFrom() == t.getFrom())
					&& (t2.getProtocols() == null)
					&& !automatic(t2)){
						if(!t2.isDeterminedProtocols())
							determined = false;
						else{
							allSame = false;
							t.setProtocols(null);
							t.setDeterminedProtocols(true);
						}
					}
				// case 3 - there's a transition into the from state that has a
				// protocol not found in all exiting protocols
				// that are not automatic 
				else if((t2.getTo() == t.getFrom())
					&& (t2.getProtocols() != null)){
						if(!allMatchOut(t, t2, st)){
							if(allDeterminedOut(t, t2, st)){
								allSame = false;
								t.setProtocols(null);
								t.setDeterminedProtocols(true);
							}
							else
								determined = false;
						}
						else{
							// add the protocols to prots
							Vector protocols = t2.getProtocols();
							for(int j = 0; j < protocols.size(); j++){
								Protocol p = (Protocol)protocols.elementAt(j);
								if(!prots.contains(p))
									prots.add(p);
							}
						}
					}
				// case 4 - there's a transition out of the from that has a
				// protocol not found in at least one transition into the from state
				else if((t2.getFrom() == t.getFrom())
					&& (t2.getProtocols() != null)
					&& !allFoundIn(t2, st)){
						if(!allDeterminedIn(t2, st))
							determined = false;
						else{
							allSame = false;
							t.setProtocols(null);
							t.setDeterminedProtocols(true);
						}
					}
				// else t2 is not relevant
			}		
		}
	}

	if(determined && allSame){
		t.setProtocols(prots);
		t.setDeterminedProtocols(true);
	}
}
/**
 * This method will add a protocols set to transitions in component state tables
 *
 * the params attribute is not used
 */
public void execute(Object params) {
	Vector			agents = sys.agents;
	Vector			components = null;
	Vector			transitions = null;
	Agent			a;
	Component 		c;
	Transition		t;
	StateTable		st;
	boolean			allMatch;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			if(st != null)
				transitions = st.getTransitions();
			if(transitions != null)
			while(!allLabeled(transitions)){
				for(int k = 0; k < transitions.size(); k++){
					t = (Transition)transitions.elementAt(k);
					// if(t.agentID == null)
					if((t.getReceive() != null) || (t.getSends() != null)){
						// there are internal events 
						// (Transformation 16)
						t.setProtocols(null);
						t.setDeterminedProtocols(true);
					}
					else{
						// there are no internal events
						if(t.getReceiveEvent() != null){
							ReceiveEvent re = t.getReceiveEvent();
							Vector reProtocols = re.getProtocols();
							if(t.getSendEvents() != null){
								// check to make sure that all sendEvents correspond
								// to the receiveEvent 
								Vector sendEvents = t.getSendEvents();
								allMatch = true;
								for(int x = 0; x < sendEvents.size(); x++){
									SendEvent se = (SendEvent)sendEvents.elementAt(x);
									// (Transformation 15)
									// if there is a SendEvent to a list, then the protocols
									// will be null, even if all events belong to the same
									// protocols 
									if(isList(se.getRecipient()))
										allMatch = false;
									Vector seProtocols = se.getProtocols();
									if(!protocolsMatch(reProtocols, seProtocols)){
										allMatch = false;
									}
								}
								if(allMatch){
									// (Transformation 12)
									t.setProtocols(reProtocols);
									t.setDeterminedProtocols(true);
								}
								else{
									// This should never happen
									System.out.println("ERROR: transition was not split up correctly");
									System.out.println(t);
									t.setProtocols(null);
									t.setDeterminedProtocols(true);
								}
							}
							else{
								// make the protocols the protocols set from the
								// the receive event (Transformation 12)
								t.setProtocols(reProtocols);
								t.setDeterminedProtocols(true);
							}
						}
						else{
							// there is no receiveEvent
							if(t.getSendEvents() != null){
								// check to make sure that all sendEvents correspond
								// to each other (Transformation 13 and Transformation 14)
								// Also check that there is not a SendEvent to a list
								Vector sendEvents = t.getSendEvents();
								allMatch = true;
								Vector prots = null;
								for(int x = 0; x < sendEvents.size(); x++){
									SendEvent se = (SendEvent)sendEvents.elementAt(x);
									// (Transformation 15)
									// if there is a SendEvent to a list, then the protocols
									// will be null, even if all events belong to the same
									// protocols
									if(isList(se.getRecipient()))
										allMatch = false;
									
									Vector seProtocols = se.getProtocols();
									if(prots == null){
										prots = se.getProtocols();
									}
									else if(seProtocols != null){
										for(int y = 0; y < seProtocols.size(); y++){
											Protocol prot = (Protocol)seProtocols.elementAt(y);
											prots.add(prot);
										}
									}
									if(!protocolsMatch(prots, se.getProtocols()) || isList(se.getRecipient())){
										allMatch = false;
									}
								}
								if(allMatch){
									// (Transformation 13)  
									t.setProtocols(prots);
									t.setDeterminedProtocols(true);
								}
								else{
									// (Transformation 14 and 15)
									t.setProtocols(null);
									t.setDeterminedProtocols(true);
								}
								
							}
							else{
								// check to see if all transitions to and from the from state
								// have the same protocols
								// (Transformation 17 and Transformation 18)
								determine(t, st);
							}
						}		
					}
				}
			}
		}	
	}
}
/**
 * This method is used to determine if a recipient is to a list
 *
 * Creation date: (11/11/00 2:46:50 PM)
 */
public boolean isList(String s) {
	char first = s.charAt(0);
	char last = s.charAt(s.length()-1);
	if((first == '<') && (last == '>'))
		return true;
	else
		return false;
}
/**
 * This method determines if the two vectors of protocols are the same
 *
 * Creation date: (11/21/00 1:36:20 PM)
 */
public boolean protocolsMatch(Vector seProtocols, Vector reProtocols) {

	if((seProtocols != null) && (reProtocols != null)){
		if(seProtocols.size() != reProtocols.size())
			return false;
		for(int i = 0; i < seProtocols.size(); i++){
			Protocol p = (Protocol)seProtocols.elementAt(i);
			if(!reProtocols.contains(p))
				return false;
		}
	}
	
	return true;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
