package Transforms;

/**
 * This class represents transformations 19-23 in the transformation process.
 * The result of this transform is that all needed transitions will receive the
 * start label.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;

public class Transform19 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform19(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This protocols determin if for all of the protocols in t's protocols set
 * there is at least one transition into t's from state with that protocol
 *
 * Creation date: (11/20/00 11:32:31 AM)
 */
public boolean allFoundIn(Transition t, StateTable st) {
	Vector		transitions = st.getTransitions();
	Transition	t2;
	Vector		protocols = t.getProtocols();
	Protocol 	p;
	boolean		found;
	
	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		found = false;
		for(int j = 0; j < transitions.size(); j++){
			t2 = (Transition)transitions.elementAt(j);
			if((t2 != t) && (t2.getTo() == t.getFrom())){
				Vector protocolsIn = t2.getProtocols();
				if((protocolsIn != null) && protocolsIn.contains(p)){
					// p was found
					found = true;					
				}
			}
		}
		// if p wasn't found in at least one of the protocols in
		// a transition in, then return false
		if(!found)
			return false;
	}

	return true;
}
/**
 * This method is used to determine if all of the transitions out of t's from state
 * have the same protocols as t does
 *
 * Creation date: (11/20/00 12:17:16 PM)
 */
public boolean allMatchOut(Transition t, StateTable st) {
	Vector transitions = st.getTransitions();
	Transition t2;
	Vector protocols = t.getProtocols();
	
	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if((t2 != t) && (t2.getFrom() == t.getFrom()) && !protocolsMatch(protocols, t2.getProtocols())){
			return false;
			/*Vector protocolsOut = t2.getProtocols();
			if(protocolsOut == null)
				return false;
			else if(protocols.size() != protocolsOut.size())
				return false;
			for(int j = 0; j < protocols.size(); j++){
				Protocol p = (Protocol)protocols.elementAt(j);
				if(!protocolsOut.contains(p))
					return false;
			}
			*/
		}
	}
	return true;
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will add an start labels to transitions in component state tables
 *
 * the params attribute is not used
 */
public void execute(Object params) {
	Vector			agents = sys.agents;
	Vector			components = null;
	Vector			transitions = null;
	Agent			a;
	Component 		c;
	Transition		t;
	StateTable		st;
	boolean			allMatch;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			if(st != null)
				transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if(t.getProtocols() != null){
					Vector protocols = t.getProtocols();
					if(!allFoundIn(t, st) // (Transformation 19)
						|| !allMatchOut(t, st) // (Transformation 21)
						|| foundOtherIn(t, st) // (Transformation 22)
						|| nullIn(t, st)
						|| nullOut(t, st)){
							t.setStart(true);
							// give the conversation a new name
							t.setConvNames(new Vector());
							t.getConvNames().add(sys.getConvName());
					}
					// only look for assignments if the transition wasn't already
					// labeled a start transition
					else if(!t.isStart()){
						Vector sendEvents = t.getSendEvents();
						if(sendEvents != null)
						for(int l = 0; l < sendEvents.size(); l++){
							SendEvent se = (SendEvent)sendEvents.elementAt(l);
							if(isAssigned(st, t, se, null)){
								// (Transformation 20)
								t.setStart(true);
								// give the conversation a new name
								t.setConvNames(new Vector());
								t.getConvNames().add(sys.getConvName());
							}
						}
					}					
				}
				else if((t.getProtocols() == null) && (t.getSendEvents() != null)){
						// (Transformation 23)
						// either there are multiple SendEvents to different
						// recipients or there is a SendEvent to a list
						//
						// All SendEvents with the same protocols must have
						// the same convName
						
						t.setStart(true);

						// iterate through sendEvents
						Vector sendEvents = t.getSendEvents();
						for(int l = 0; l < sendEvents.size(); l++){
							SendEvent se = (SendEvent)sendEvents.elementAt(l);
							if(se.getConvName() == null){
								se.setConvName(sys.getConvName());
								if(t.getConvNames() == null)
									t.setConvNames(new Vector());
								t.getConvNames().add(se.getConvName());
								// give same convName to all matching sendEvents
								for(int m = 0; m < sendEvents.size(); m++){
									SendEvent se2 = (SendEvent)sendEvents.elementAt(m);
									if((se2 != se) && se.sameProtocols(se2)){
										se2.setConvName(se.getConvName());
									}
								}
							}
						}
					}				
			}
		}	
	}
}
/**
 * This method determines if there is a transition into t's from state that has a
 * protocol not found in t's protocol set
 *
 * Creation date: (11/20/00 12:17:16 PM)
 */
public boolean foundOtherIn(Transition t, StateTable st) {
	Vector	protocols = t.getProtocols();
	Vector	transitions = st.getTransitions();
	Transition t2;

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.getTo() == t.getFrom()){
			Vector protocolsIn = t2.getProtocols();
			if(protocolsIn != null){
				for(int j = 0; j < protocolsIn.size(); j++){
					Protocol p = (Protocol)protocolsIn.elementAt(j);
					if(!protocols.contains(p))
						return true;
				}
			}
		}
	}
	
	return false;
}
/**
 * This method determines if the recipient of the send event has been assigned in a
 * previous state.
 *
 * Creation date: (11/13/00 1:11:20 PM)
 */
public boolean isAssigned(StateTable st, Transition t, SendEvent se, Vector checked) {
	State 	s = t.getFrom();
	Action 	a;
	Vector	actions = s.getActions();
	Vector	transitions = st.getTransitions();
	Transition t2;
	boolean	sent = false;
	boolean received = false;

	// add the transition to the checked vector
	if(checked == null)
		checked = new Vector();
	checked.add(t);
	
	// check the actions in the from state to see if they
	// make an assignment to the recipient
	if(actions != null)
	for(int i = 0; i < actions.size(); i++){
		a = (Action)actions.elementAt(i);
		Vector lhs = a.getLhs();
		if(lhs != null)
		for(int j = 0; j < lhs.size(); j++){
			String p = (String)lhs.elementAt(j);
			if(p.equals(se.getRecipient()))
				return true;
		}
	}

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(!checked.contains(t2) && (t2 != t) && (t2.getTo() == t.getFrom())){
			// check to see if there was a ReceiveEvent with
			// sender equal to the original SendEvent's recipient
			ReceiveEvent re = t2.getReceiveEvent();
			if((re != null) && (re.getSender().equals(se.getRecipient()))){
				received = true;
			}
			else{
				// check to see if there is a SendEvent with
				// the same recipient as the original SendEvent
				Vector sendEvents = t2.getSendEvents();
				if(sendEvents != null)
				for(int j = 0; j < sendEvents.size(); j++){
					SendEvent se2 = (SendEvent)sendEvents.elementAt(j);
					if((se2 != null) && (se2.getRecipient().equals(se.getRecipient())))
						sent = true;
				}
			}
			if(!sent && !received){
				// if there was no message sent to or received from
				// the recipient of the original SendEvent se, then check for
				// actions that make the assignment
				actions = t.getActions();
				if(actions != null)
				for(int j = 0; j < actions.size(); j++){
					a = (Action)actions.elementAt(j);
					Vector lhs = a.getLhs();
					if(lhs != null)
					for(int k = 0; k < lhs.size(); k++){
						String p = (String)lhs.elementAt(k);
						if(p.equals(se.getRecipient()))
							return true;
					}
				}
				if(isAssigned(st, t2, se, checked)){
					// make a recursive call on t2
					return true;
				}
			}
		}
	}	
	
	return false;
}
/**
 * This method will determine if there is a transition into the from state
 * of transition t with null protocols
 *
 * Creation date: (12/28/00 1:03:40 PM)
 */
public boolean nullIn(Transition t, StateTable st) {
	Vector	protocols = t.getProtocols();
	Vector	transitions = st.getTransitions();
	Transition t2;

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.getTo() == t.getFrom()){
			Vector protocolsIn = t2.getProtocols();
			if(protocolsIn == null){
					return true;
			}
		}
	}
	
	return false;
}
/**
 * This method will determine if there is a transition into the from state
 * of transition t with null protocols
 *
 * Creation date: (12/28/00 1:03:40 PM)
 */
public boolean nullOut(Transition t, StateTable st) {
	Vector	protocols = t.getProtocols();
	Vector	transitions = st.getTransitions();
	Transition t2;

	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.getFrom() == t.getFrom()){
			Vector protocolsIn = t2.getProtocols();
			if(protocolsIn == null){
					return true;
			}
		}
	}
	
	return false;
}
/**
 * This method determines if the two vectors of protocols are the same
 *
 * Creation date: (11/21/00 1:36:20 PM)
 */
public boolean protocolsMatch(Vector seProtocols, Vector reProtocols) {

	if((seProtocols != null) && (reProtocols != null)){
		if(seProtocols.size() != reProtocols.size())
			return false;
		for(int i = 0; i < seProtocols.size(); i++){
			Protocol p = (Protocol)seProtocols.elementAt(i);
			if(!reProtocols.contains(p))
				return false;
		}
	}
	
	return true;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
