package Transforms;

/**
 * This class represents transformation 28a in the transformation process.
 * The result of this transform is that any transition that is the start and not the 
 * end of a conversation before a state containing a move activity
 * is changed to be the end of the conversation as well.  This ensures that
 * the state containing the move activity stays in the component when the
 * conversations are harvested from the components.
 *
 * Creation date: (2/15/01 11:03:00 PM)
 * @author: Athie Self
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.*;

public class Transform28a extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform28a(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will add end labels to transitions in component state tables
 *
 * the params attribute is not used 
 */

public void execute(Object params) {	

	Vector agents = sys.agents;
	
	for(int i = 0; i < agents.size(); i++) {
	
		Agent ag = (Agent)agents.elementAt(i);

		if(isMobility_Specified(ag)) {
			
			Vector components = ag.getComponents();

			for(int j = 0; j < components.size(); j++) {

				Component c = (Component)components.elementAt(j);
										
				if(isMobility_Specified_Component(c)) {

					StateTable st = (StateTable)c.getStateTable();
					
					Vector VecStates = st.getStates();

					for(int k = 0; k < VecStates.size(); k++) {

						State s = (State)VecStates.elementAt(k);
						
						if(isMobility_Specified_State(s)) {
							
							Vector transitions = st.getTransitions();

							// Get the transitions that have a to state that is 
							// state with the move activity and see if any of those
							// transitions are the start of a conversation.  If so,
							// then mark the transition as an end of a conversation.
							
							for(int l = 0; l < transitions.size(); l++) {
						
								Transition t = (Transition)transitions.elementAt(l);

								if(t.getToName()!=null) {
									
									if(t.getToName().equals(s.getName())
										&& isConversation_Before(st,t)) {

										t.setEnd(true);
										
										ATtransition atTrans = (ATtransition)t.getAtTransition();

										atTrans.setEnd(true);

									}
								}
							}
						}
					}
				}
			}
		}
	}
		
	// now make the altered system the current system
	// can be undone by the unDo method
	currentSys = sys;
}
/**
 * Insert the method's description here.
 * Creation date: (2/15/2001 11:26:27 PM)
 * @return boolean
 * @param trans Transform.Transition
 */

public boolean isConversation_Before(StateTable stateT, Transition trans) {
	
	if ((trans.isStart()==true) && (trans.isEnd()==false)) {

		return true;
		
	} else {
		
		Vector vecTrans = stateT.getTransitions();
		
		for(int m=0; m<vecTrans.size(); m++) {

			Transition t2 = (Transition)vecTrans.elementAt(m);

			if (trans.getFromName() != null) {
				
				if (trans.getFromName() == t2.getToName()) {
			
					isConversation_Before(stateT,t2);
				
				}
			}
		}
	}
	
	return false;
	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param ag Transforms.Agent
 */
public boolean isMobility_Specified(Agent a) {

	Vector actions    = new Vector();
	Vector states     = new Vector();
	
	Vector components = a.getComponents();
	
	if(components != null) {

		for(int j = 0; j < components.size(); j++) {

			Component c = (Component)components.elementAt(j);

			if (!c.getName().equals("AgentComponent")) {

				StateTable st = c.getStateTable();

				if(st != null) {
					states = st.getStates();
				}

				if(states != null) {					

					for(int k = 0; k < states.size(); k++) {

						State s = (State)states.elementAt(k);

						actions = s.getActions();

						if(actions != null) {

							for(int l = 0; l < actions.size(); l++) {

								Action act = (Action)actions.elementAt(l);

								FunctionCall fc = (FunctionCall)act.getFunction();

								if(fc!=null) {

									if(fc.getName().equals("move")) {
										return true;
									}
								}	
							}		
						}	
					}
				}
			}
		}
	}
	
	return false;	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param ag Transforms.Agent
 */
public boolean isMobility_Specified_Component(Component c) {

	Vector			actions = new Vector();
	Vector			states = new Vector();
	StateTable		st;
	Action			act;
	State			s = null;
	FunctionCall	fc;

	if (!c.getName().equals("AgentComponent")) {
		st = c.getStateTable();
		if(st != null) {
			states = st.getStates();
		}
		if(states != null) {					
			for(int k = 0; k < states.size(); k++) {
				s = (State)states.elementAt(k);
				actions = s.getActions();
				if(actions != null) {
					for(int l = 0; l < actions.size(); l++) {
						act = (Action)actions.elementAt(l);
						fc = act.getFunction();
						if(fc!=null) {
							if(fc.getName().equals("move")) {
								return true;
							}
						}	
					}		
				}	
			}
		}
	}
	
	return false;	
}
/**
 * Insert the method's description here.
 * Creation date: (12/29/2000 9:53:42 AM)
 * @return boolean
 * @param ag Transforms.Agent
 */
public boolean isMobility_Specified_State (State s) {

	Vector vecActions = s.getActions();

	if(vecActions != null) {
	
		for(int i = 0; i < vecActions.size(); i++) {
		
			Action act = (Action)vecActions.elementAt(i);

			FunctionCall fc = (FunctionCall)act.getFunction();
			
			if(fc != null) {
			
				if(fc.getName().equals("move")) {
				
					return true;
					
				}
			}	
		}		
	}	
	
	return false;	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
