package Transforms;

/**
 * This class represents transformations 28-30 in the transformation process.
 * The result of the transform is that in some cases send events in one component
 * will be matched up in another component and they will receive the same convName.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */

import java.util.Vector;

public class Transform29 extends Transform {	
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform29(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method will only return true if the target is a TSystem and that system has
 * roles, tasks, and protocols defined.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will determine if condition 1 has been met.  That is, do the events
 * equal each other and there are not other protocols that either event could possibly
 * belong to and either the send event or the receive event is the only event in
 * their state tables with their event.
 *
 * Creation date: (12/7/00 7:46:09 PM)
 */
boolean condition1(Protocol p, Component c, Component c2, 
	Transition trans, Transition trans2, SendEvent se, ReceiveEvent re) {
		StateTable st = c.getStateTable();
		StateTable st2 = c2.getStateTable();
		if((re != null) 
			&& (re.getSender() != null) 
			&& re.getEvent().corresponds(se.getEvent())
			&& isUnique(sys, p, c, c2, trans, trans2, se, re)
			&& (onlySend(st, se) || onlyReceive(st2, re)) ){
				return true;
			}
		return false;
}
/**
 * This method will determine if condition 2 has been met.  That is, do the events
 * equal each other and there are not other protocols that the receive event could possibly
 * belong to and is the send event the only send event in its state table with that event.
 *
 * Creation date: (12/7/00 7:46:09 PM)
 */
boolean condition2(Protocol p, Component c, Component c2, 
	Transition trans, Transition trans2, SendEvent se, ReceiveEvent re) {
		StateTable st = c.getStateTable();
		StateTable st2 = c2.getStateTable();
		if((re != null) 
			&& (re.getSender() != null) 
			&& re.getEvent().corresponds(se.getEvent())
			&& onlySend(st, se)
			&& isUnique(sys, p, c2, re)){
				return true;
			}
		return false;
}
/**
 * This method will determine if condition 3 has been met.  That is, do the events
 * equal each other and there are not other protocols that the send event could possibly
 * belong to and is the receive event the only receive event in its state table with that event.
 *
 * Creation date: (12/7/00 7:46:09 PM)
 */
boolean condition3(Protocol p, Component c, Component c2, 
	Transition trans, Transition trans2, SendEvent se, ReceiveEvent re) {
		StateTable st = c.getStateTable();
		StateTable st2 = c2.getStateTable();
		if((re != null) 
			&& (re.getSender() != null) 
			&& re.getEvent().corresponds(se.getEvent())
			&& onlyReceive(st2, re)
			&& isUnique(sys, p, c, se)){
				return true;
			}
		return false;
}
/**
 * This method will now check all external events and try
 * determine the protocol they belong to.
 *
 * The params attribute is not used.
 */
public void execute(Object params) {
	// get the protocols from the system 
	Vector			protocols = sys.protocols;
	Component		c, c2;
	Protocol		p;

	if(debug){
		System.out.println();
		System.out.println("*** Transformation 29 ***");
	}
	
	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		if((p.getInitComp() != null) && (p.getRespComp() != null)){
			c = p.getInitComp();
			c2 = p.getRespComp();
			findMatch(sys, p, c, c2);

			c = p.getRespComp();
			c2 = p.getInitComp();
			findMatch(sys, p, c, c2);
		}
	}
}
/**
 * This method will now check all external events and try
 * determine the protocol they belong to.
 *
 * The params attribute is not used.
 */
public void findMatch(TSystem sys, Protocol p, Component c, Component c2) {
	StateTable 		st = c.getStateTable();
	StateTable		st2 = c2.getStateTable();
	Transition		trans, trans2;
	SendEvent		se;
	ReceiveEvent	re;
	Vector transitions = st.getTransitions();
	Vector transitions2 = st2.getTransitions();

	// check all send events in the transitions vector
	// for task t's state table
	if(transitions != null)
	for(int j = 0; j < transitions.size(); j++){
		trans = (Transition)transitions.elementAt(j);
		if(trans.isStart()){
		
			// get the send events for this transition
			Vector sendEvents = trans.getSendEvents();
		
			// check each individual send event
			if(sendEvents != null){
				for(int k = 0; k < sendEvents.size(); k++){
					se = (SendEvent)sendEvents.elementAt(k);
					// check the send event against all receiveEvents
					// for transitions in task t2's state table
					if(transitions2 != null)
					for(int l = 0; l < transitions2.size(); l++){
						trans2 = (Transition)transitions2.elementAt(l);
						if(trans2.isStart()){
							re = trans2.getReceiveEvent();
							if(/*condition1(p, c, c2, trans, trans2, se, re)
								|| */condition2(p, c, c2, trans, trans2, se, re)
								|| condition3(p, c, c2, trans, trans2, se, re)){
									String newConvName = sys.getConvName();
									if(se.getEventMatches() == null)
										se.setEventMatches(new Vector());
									if(re.getEventMatches() == null)
										re.setEventMatches(new Vector());
									if(se.getTransMatches() == null)
										se.setTransMatches(new Vector());
									if(re.getTransMatches() == null)
										re.setTransMatches(new Vector());
									se.getEventMatches().add(re);
									re.getEventMatches().add(se);
									re.getTransMatches().add(trans);
									se.getTransMatches().add(trans2);			

									// need to match up the names with the other
									String cname = se.getConvName();
									// if the send event does not have a convName,
									// then the transition should only have one convName
									// that can be used
									if(cname == null){
										if(trans.getReceiveEvent() == null)
											cname = (String)trans.getConvNames().elementAt(0);
										else{
											cname = newConvName;
											if(trans.getConvNames() == null)
												trans.setConvNames(new Vector());
											trans.getConvNames().add(newConvName);
										}
									}

									// the transition with the receive event will only have
									// one convName that needs to be replaced
									String cname2 = (String)trans2.getConvNames().elementAt(0);
									
									// update everything to the new ConvName
									// this should happen EVERY time
									Vector updated = new Vector();
									updated.add(trans);
									updated.add(trans2);
									se.setConvName(newConvName);
									re.setConvName(newConvName);
									trans.updateConvName(cname, newConvName);
									updateMatches(trans, cname, newConvName, updated);
									trans2.updateConvName(cname2, newConvName);
									updateMatches(trans2, cname2, newConvName, updated);
									trans.setMatched(true);
									trans2.setMatched(true);
									if(debug){
										System.out.println("matching transitions with new convName: " + newConvName);
										System.out.println(trans);
										System.out.println(trans2);
									}
								}
						}
					}
				}
			}
		}
	}
}
/**
 * This method will determine if there are no other protocols that either the send
 * event or receive event could possibly belong to.
 *
 * Creation date: (11/9/00 3:44:46 PM)
 */
public boolean isUnique(TSystem sys, Protocol p, Component c, Component c2, Transition trans, Transition trans2, SendEvent se, ReceiveEvent re) {
	Protocol 	p2;
	Component	c3 = null;
	boolean 	checkFirst, checkSecond;
	StateTable	st;
	Vector transitions;
	
	// check all protocols
	Vector protocols = sys.protocols;
	for(int i = 0; i < protocols.size(); i++){
		p2 = (Protocol)protocols.elementAt(i);
		if(p2 != p){
			checkFirst = false;
			checkSecond = false;
			// check the first case: Component c cannot have another protocol
			// with a Component that has a receive event equal to re
			if((p2.getInitComp() == c) && (p2.getRespComp() != c2)){
				c3 = p2.getRespComp();
				checkFirst = true;
			}
			if((p2.getRespComp() == c) && (p2.getInitComp() != c2)){
				c3 = p2.getInitComp();
				checkFirst = true;
			}

			if(checkFirst){
				st = c3.getStateTable();
				transitions = st.getTransitions();
				if(transitions != null){
					for(int j = 0; j < transitions.size(); j++){
						Transition trans3 = (Transition)transitions.elementAt(j);
						ReceiveEvent re2 = trans3.getReceiveEvent();
						if((re2 != null) && re2.getEvent().corresponds(re.getEvent())){
							if(debug){
								System.out.println();
								System.out.println("The send event " + se + " in component " + c.getName() + " cannot be matched");
								System.out.println("There is a matching receive event in both " + c2.getName() + " and " + c3.getName());
							}
							return false;
						}
					}
				}
			}
				
			// check the second case: Component c2 cannot have another protocol 
			// with a Component that has a send event equal to se
			if((p2.getInitComp() == c2) && (p2.getRespComp() != c)){
				c3 = p2.getRespComp();
				checkSecond = true;
			}
			if((p2.getRespComp() == c2) && (p2.getInitComp() != c)){
				c3 = p2.getInitComp();
				checkSecond = true;
			}

			if(checkSecond){
				st = c3.getStateTable();
				transitions = st.getTransitions();
				if(transitions != null){
					for(int j = 0; j < transitions.size(); j++){
						Transition trans4 = (Transition)transitions.elementAt(j);
						Vector sendEvents = trans4.getSendEvents();
						if(sendEvents != null){
							for(int k = 0; k < sendEvents.size(); k++){
								SendEvent se2 = (SendEvent)sendEvents.elementAt(k);
								if(se2.getEvent().corresponds(se.getEvent())){
									if(debug){
										System.out.println();
										System.out.println("The receive event " + re + " in component " + c2.getName() + " cannot be matched");
										System.out.println("There is a matching send event in both " + c.getName() + " and " + c3.getName());
									}
									return false;
								}
							}
						}
					}
				}
			}
	
		}
	}
	
	return true;
}
/**
 * This method will determine if there are no other protocols that the receive
 * event could possibly belong to.
 *
 * Creation date: (11/9/00 3:44:46 PM)
 */
public boolean isUnique(TSystem sys, Protocol p, Component c2, ReceiveEvent re) {
	Vector		protocols = sys.protocols;
	Protocol 	p2;
	Component 	c3;
	
	for(int i = 0; i < protocols.size(); i++){
		p2 = (Protocol)protocols.elementAt(i);
		if((p2 != p) && ((p2.getInitComp() == c2) || (p2.getRespComp() == c2))){
			if(p2.getInitComp() == c2)
				c3 = p2.getRespComp();
			else
				c3 = p2.getInitComp();
			StateTable st3 = c3.getStateTable();
			Vector transitions = st3.getTransitions();
			if(transitions != null)
			for(int j = 0; j < transitions.size(); j++){
				Transition trans3 = (Transition)transitions.elementAt(j);
				Vector sendEvents = trans3.getSendEvents();
				if(sendEvents != null)
				for(int k = 0; k < sendEvents.size(); k++){
					SendEvent se = (SendEvent)sendEvents.elementAt(k);
					if(se.getEvent().corresponds(re.getEvent()))
						return false;
				}
			}	
		}
	}
	
	return true;
}
/**
 * This method will determine if there are no other protocols that the send
 * event could possibly belong to.
 *
 * Creation date: (11/9/00 3:44:46 PM)
 */
public boolean isUnique(TSystem sys, Protocol p, Component c2, SendEvent se) {
	Vector		protocols = sys.protocols;
	Protocol 	p2;
	Component	c3;
	
	for(int i = 0; i < protocols.size(); i++){
		p2 = (Protocol)protocols.elementAt(i);
		if((p2 != p) && ((p2.getInitComp() == c2) || (p2.getRespComp() == c2))){
			if(p2.getInitComp() == c2)
				c3 = p2.getRespComp();
			else
				c3 = p2.getInitComp();
			StateTable st3 = c3.getStateTable();
			Vector transitions = st3.getTransitions();
			if(transitions != null)
			for(int j = 0; j < transitions.size(); j++){
				Transition trans3 = (Transition)transitions.elementAt(j);
				ReceiveEvent re = trans3.getReceiveEvent();
				if((re != null) && (se.getEvent().corresponds(re.getEvent())))
					return false;
			}	
		}
	}
	
	return true;
}
/**
 * This method will determine if the receive event is the only receive event in the
 * state table with the same event.
 *
 * Creation date: (11/15/00 10:51:28 AM)
 */
public boolean onlyReceive(StateTable st, ReceiveEvent re) {
	Vector			transitions = st.getTransitions();
	Transition	 	t;
	Vector			sendEvents;
	ReceiveEvent	re2;
	
	for(int i = 0; i < transitions.size(); i++){
		t = (Transition)transitions.elementAt(i);
		re2 = t.getReceiveEvent();
		if((re2 != null) && (re2 != re) && (re2.getEvent().corresponds(re.getEvent())))
			return false;
	}
	return true;
}
/**
 * This method will determine if the send event is the only send event in the
 * state table with the same event.
 *
 * Creation date: (11/15/00 10:51:28 AM)
 */
public boolean onlySend(StateTable st, SendEvent se) {
	Vector		transitions = st.getTransitions();
	Transition 	t;
	Vector		sendEvents;
	SendEvent	se2;
	
	for(int i = 0; i < transitions.size(); i++){
		t = (Transition)transitions.elementAt(i);
		sendEvents = t.getSendEvents();
		if(sendEvents != null)
		for(int j = 0; j < sendEvents.size(); j++){
			se2 = (SendEvent)sendEvents.elementAt(j);
			if((se2 != se) && (se2.getEvent().corresponds(se.getEvent())))
				return false;
		}	
	}
	return true;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * This method will set the system back to the 
 * system before the transformation took place.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
/**
 * This method updates conversation names of all matching transitions
 * with the new conversation name, removing the old one.  It is recursive
 * and must maintain and pass a vector of transitions this has already
 * been called on.
 * 
 * Creation date: (12/6/00 7:11:34 PM)
 */
void updateMatches(Transition t, String oldName, String newName, Vector updated) {
	Vector matches = t.getTransMatches();
	if(matches != null)
	for(int i = 0; i < matches.size(); i++){
		Transition t2 = (Transition)matches.elementAt(i);
		t2.updateConvName(oldName, newName);
		ReceiveEvent re = t2.getReceiveEvent();
		if((re != null) && (re.getConvName() != null) && re.getConvName().equals(oldName))
			re.setConvName(newName);
		Vector sendEvents = t2.getSendEvents();
		if(sendEvents != null)
		for(int j = 0; j < sendEvents.size(); j++){
			SendEvent se = (SendEvent)sendEvents.elementAt(j);
			String cname = se.getConvName();
			if((cname != null) && (cname.equals(oldName)))
				se.setConvName(newName);
		}

		if(!updated.contains(t2)){
			updated.add(t2);
			updateMatches(t2, oldName, newName, updated);
		}
	}
}
}
