package Transforms;

/**
 * This class represents transformations 33-34 in the transformation process.
 * The result is that any conversation that exits to multiple states will be changed
 * so that they all exit to a newly created null state.  
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.ATtableElement;
import system.ATstate;
import system.ATstateGRFX;
import system.ATstartState;
import system.ATstartStateGRFX;
import system.ATendState;
import system.ATendStateGRFX;
import system.ATtransition;

public class Transform33 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform33(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will create a new null state between the two given transitions
 *
 * Creation date: (11/22/00 7:13:47 PM)
 */
void createNullState(State s, Transition t, Transition t2) {	
	State from = t.getTo();
	State to = t2.getTo();
	ATtableElement fromElement = from.getElement();
	ATtableElement toElement = to.getElement();
	ATtableElement atFrom, atTo;
	int newX;
	int newY;
	int[] fromCoords = new int[4];
	int[] toCoords = new int[4];
	
	if(fromElement instanceof ATstate){
		fromCoords = ((ATstate)fromElement).getGrfx().getCoords();
	}
	if(fromElement instanceof ATstartState){
		fromCoords = ((ATstartState)fromElement).getGrfx().getCoords();
	}
	if(fromElement instanceof ATendState){
		fromCoords = ((ATendState)fromElement).getGrfx().getCoords();
	}

	if(toElement instanceof ATstate){
		toCoords = ((ATstate)toElement).getGrfx().getCoords();
	}
	if(toElement instanceof ATstartState){
		toCoords = ((ATstartState)toElement).getGrfx().getCoords();
	}
	if(toElement instanceof ATendState){
		toCoords = ((ATendState)toElement).getGrfx().getCoords();
	}

	// determine newX
	newX = (toCoords[0] + fromCoords[0]) / 2;
	
	// determind newY
	newY = (toCoords[1] + fromCoords[1]) / 2;
	
	ATstate newState = new ATstate(sys.version, sys.atSys.getNextKey());
	newState.setName(s.getName());
	newState.setCoordinates(newX, newY, 50, 50);
	s.setElement(newState);
	
}
/**
 * This method will look for conversations that exit to multiple states and
 * call the method to prepare that conversation.
 *
 * the params attribute is not used
 */
public void execute(Object params) {	
	Vector 			agents = sys.agents;
	Agent 			a;
	Vector			components;
	Component		c;
	StateTable		st;
	Vector			transitions;
	Transition		t;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if(t.isStart()){
					Vector endStates = getEndStates(t, st);
					if(endStates.size() > 1)
						prepareConv(t, st);	
				}
			}
		}
	}	
}
/**
 * This method will find all states that the given start transition exits to.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
Vector getEndStates(Transition t, StateTable st) {
	Vector endStates = new Vector();
	Vector convs = t.getConversations();
	Vector transitions = st.getTransitions();

	if(convs == null){
		System.out.println();
		System.out.println("Error - T33: trying to find end states, but no conversations");
		System.out.println();
		return endStates;
	}
	for(int i = 0; i < transitions.size(); i++){
		Transition t2 = (Transition)transitions.elementAt(i);
		if(t2.isEnd() && !endStates.contains(t2.getTo()) && t2.hasAllConversations(convs)){
			endStates.add(t2.getTo());
		}
	}

	if(endStates.size() == 0){
		System.out.println();
		System.out.println("Error - T33: found no end states with all of the conversations");
		System.out.println();
	}
	
	return endStates;
}
/**
 * This method will find all end transitions for the given start transition.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
Vector getEndTransitions(Transition t, StateTable st) {
	Vector endTransitions = new Vector();
	Vector convs = t.getConversations();
	Vector transitions = st.getTransitions();

	if(convs == null){
		System.out.println();
		System.out.println("Error - T33: trying to find end transitions");
		System.out.println();
		return endTransitions;
	}
	for(int i = 0; i < transitions.size(); i++){
		Transition t2 = (Transition)transitions.elementAt(i);
		if(t2.isEnd() && t2.hasAllConversations(convs)){
			endTransitions.add(t2);
		}
	}

	if(endTransitions.size() == 0){
		System.out.println();
		System.out.println("Error - T33: found no end transitions");
		System.out.println();
	}
	
	return endTransitions;
}
/**
 * This method will prepare the conversation.  A new null state is created.  All exiting
 * transitions will then go to the null state and an action is added to set the branch.
 * From the null state, new automatic transitions are created to the original exit states,
 * each with a guard that test to see if the corresponding branch was taken.
 * Creation date: (11/26/00 3:14:16 PM)
 */
void prepareConv(Transition t, StateTable st) {
	Vector 	transOut = getEndTransitions(t, st);
	Transition t1 = (Transition)transOut.elementAt(0);
	Transition t2 = (Transition)transOut.elementAt(1);
	State s = null;
	
	boolean added = false;
	for(int n = 0; added == false; n++){
		String stateName = "Null" + String.valueOf(n);
		if(!st.hasState(stateName)){
			added = true;
			// need to create the corresponding objects that
			// go in the AT system
			s = new State(stateName, sys);
			createNullState(s, t1, t2);
			st.getStates().add(s);
			// add s's ATtableElement to the ATstatetable
			st.getAtStateTable().addState(s.getElement());
		}
	}

	// now change all of the exiting transitions to go to the new state
	for(int i = 0; i < transOut.size(); i++){
		Transition t3 = (Transition)transOut.elementAt(i);
		
		// save the original to state
		State to = t3.getTo();
		State from = t3.getFrom();
		
		// set t3's to state to the new null state
		t3.setTo(s);
		
		// create the action associated with the transition
		Vector lhs = new Vector();
		lhs.add("parent.BRANCH");
		Vector rhs = new Vector();
		rhs.add(String.valueOf(i));
		Action act = new Action(lhs, rhs);
		if(t3.getActions() == null)
			t3.setActions(new Vector());
		t3.getActions().add(act);
		// set the actions for the ATtransition object
		t3.getAtTransition().setAction(t3.getActionsAsStrings());
		
		// set the associated graphical objects in agentTool
		ATtransition atTrans1 = t3.getAtTransition();
		atTrans1.setNextState(s.getElement());
		atTrans1.setCoordinates(from.getElement(), s.getElement());

		// create a new null transition from the null state to the original to state
		Transition newTrans = new Transition(sys);
		newTrans.setGuard("BRANCH == " + String.valueOf(i));
		newTrans.setFrom(s);
		newTrans.setTo(to);
		ATtransition newATtrans = new ATtransition(sys.version, sys.atSys.getNextKey());
		newATtrans.setGuardCondition(newTrans.getGuard());
		newATtrans.setCurrentState(s.getElement());
		newATtrans.setNextState(to.getElement());
		newATtrans.setCoordinates(s.getElement(), to.getElement());
		newTrans.setAtTransition(newATtrans);

		// now add the new transition
		st.getTransitions().add(newTrans);
		st.getAtStateTable().addTransition(newATtrans);
	}

}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
