package Transforms;

/**
 * This class represents transformation 4 in the transformation process
 * It will create an agent component for every task of every role that
 * agents play.  The task state tables are copied as the state tables of
 * the components.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.ATagent;
import system.ATarchitecture;
import system.ATagentComponent;

public class Transform4 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform4(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method does the creation of the component for the given agent based on
 * the given task
 *
 * Creation date: (11/10/00 12:10:45 PM)
 */
public void createComponent(Agent agent, Task task) {
	StateTable st = null;
	Vector components = agent.getComponents();
	
	if(components == null)
		components = new Vector();
	StateTable stateTable = task.getStateTable();
	if(stateTable != null){
		st = stateTable.copy();
	}
	Component c = new Component(task.getName(), st, agent);
	components.add(c);
	int num = components.size();
	agent.setComponents(components);

	// need to create the ATagentComponent that corresponds
	ATagent atAgent = agent.getAtAgent();
	ATarchitecture atArch = atAgent.getArchitecture();
	ATagentComponent atComp = new ATagentComponent(sys.version, sys.atSys.getNextKey());
	atArch.addComponent(atComp);
	c.setAtComponent(atComp);
	atComp.setName(c.getName());
	// set the position of the component
	atComp.setCoordinates(50+((num-1)%7)*75, 50+((num-1)/7)*75, 50, 50);
	
	atComp.setStateTable(st.getAtStateTable());

}
/**
 * This method will add a component to agents for each task
 * for every role they play
 *
 * the params attribute is not used
 */
public void execute(Object params) {
	Vector	agents = sys.agents;
	Vector 	roles;
	Vector	tasks;

	if(debug){
		System.out.println();
		System.out.println("*** Transformation 4 ***");
	}
		
	for(int i = 0; i < agents.size(); i++){
		Agent agent = (Agent)agents.elementAt(i);
		roles = agent.getRoles();
		if(roles != null){
			for(int j = 0; j < roles.size(); j++){
				Role role = (Role)roles.elementAt(j);
				tasks = role.getTasks();
				if(tasks != null){
					for(int k = 0; k < tasks.size(); k++){
						Task task = (Task)tasks.elementAt(k);
						createComponent(agent, task);
					}
				}
			}
		}
		// show the agent after creating its components
		if(debug){
			System.out.println();
			agent.show();
		}
	}
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
