package Transforms;

/**
 * This class represents transformation 45 in the transformation process
 * The result of the transform is that transitions will be made for initiator
 * conversations that begin with a transition that has protocols = null. 
 * That is, the transition has either send events that belong to different conversations
 * or there is a send event to a list.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import java.util.StringTokenizer;
import system.ATtransition;

public class Transform45 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform45(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * Insert the method's description here.
 * Creation date: (1/19/01 7:10:46 AM)
 */
boolean containsParameter(Vector parameters, Parameter p) {
	if(parameters != null)
	for(int i = 0; i < parameters.size(); i++){
		Parameter param = (Parameter)parameters.elementAt(i);
		if(param.getName().equals(p.getName()))
			return true;
	}
	return false;
}
/**
 * This method will add transitions that start initiator conversations 
 * with more than one conversation.
 *
 * the params attribute is not used
 */
public void execute(Object params) {	
	Vector 			agents = sys.agents;
	Agent 			a;
	Vector			components;
	Component		c;
	StateTable		st;
	Vector			transitions;
	Transition		t;
	Vector 			sendEvents;
	String 			actName = "";

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				if(t.isStart() && (t.getProtocols() == null) && (t.getReceiveEvent() == null)){
					State from = t.getFrom();
					State to = getEndState(t, st);
					Transition newTrans = new Transition(sys);
					newTrans.setGuard(t.getGuard());
					t.setGuard(null);
					t.getAtTransition().setGuardCondition("");
					newTrans.setFrom(from);
					newTrans.setTo(to);
					Vector newActions = t.getActions();
					if(newActions == null)
						newActions = new Vector();
					t.setActions(null);
					sendEvents = t.getSendEvents();
					if(sendEvents != null)
					for(int l = 0; l < sendEvents.size(); l++){
						SendEvent se = (SendEvent)sendEvents.elementAt(l);
						String convName = "conv";
						if((se.getConversations() != null) && (se.getConversations().size() > 1)){
							actName = (String)se.getConvName();
							actName = actName + "-" + getConvNumbers(se.getConversations());
							Conversation conv = (Conversation)se.getConversations().elementAt(0);
							convName = conv.getName();
						}
						if((se.getConversations() != null) && (se.getConversations().size() == 1)){
							Conversation conv = (Conversation)se.getConversations().elementAt(0);
							convName = conv.getName();
							actName = convName;
						}
						// determine the parameters necessary for the function
						Vector	parameters = getNeededParameters(convName, t, st);
						// create a function with the name of the conversation being started
						FunctionCall f = new FunctionCall(actName, parameters);
						// create the action for the new transition
						Action newAction = new Action(f);
						newActions.add(newAction);
					}
					/*
					Vector convs = t.getConvNames();
					if(convs != null)
					for(int l = 0; l < convs.size(); l++){
						String convName = (String)convs.elementAt(l);
						// determine the parameters necessary for the function
						Vector	parameters = getNeededParameters(convName, t, st);
						// create a function with the name of the conversation being started
						FunctionCall f = new FunctionCall(convName, parameters);
						// create the action for the new transition
						Action newAction = new Action(f);
						newActions.add(newAction);
					}
					*/
					newTrans.setActions(newActions);
					// add the transition to the stateTable					
					st.getTransitions().add(newTrans);

					// now create the corresponding ATtransition
					ATtransition newATtransition = new ATtransition(sys.version, sys.atSys.getNextKey());
					newATtransition.setCurrentState(from.getElement());
					newATtransition.setNextState(to.getElement());
					newATtransition.setCoordinates(from.getElement(), to.getElement());
					newATtransition.setGuardCondition(newTrans.getGuard());
					newATtransition.setRecMessage(t.getRecMessage());
					st.getAtStateTable().addTransition(newATtransition);
					newTrans.setAtTransition(newATtransition);
					newTrans.addATActions();
				}
			}
		}
	}
}
/**
 * Insert the method's description here.
 * Creation date: (1/15/01 9:05:26 AM)
 */
String getConvNumbers(Vector conversations) {
	String numbers = "";
	if(conversations != null)
	for(int i = 0; i < conversations.size(); i++){
		Conversation c = (Conversation)conversations.elementAt(i);
		String name = c.getName();
		int index = name.lastIndexOf("-");
		if(index != -1){
			index++;
			String num = name.substring(index);
			numbers = numbers + num;
			if(i < conversations.size()-1)
				numbers = numbers + "_";
		}
	}
	return numbers;
}
/**
 * This method will find and return the end state for the given start state
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
State getEndState(Transition t, StateTable st) {
	Vector transitions = st.getTransitions();
	Transition t2;
	State endState;
	Vector convs = t.getConversations();
	
	if(transitions != null)
	for(int i = 0; i < transitions.size(); i++){
		t2 = (Transition)transitions.elementAt(i);
		if(t2.isEnd() && t2.hasAllConversations(convs)){
			endState = t2.getTo();
			return endState;
		}
	}
	
	return null;
}
/**
 * This method will determine the parameters needed for the action to start the conversation
 * for the given transition.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
Vector getNeededParameters(String convName, Transition t, StateTable st) {
	Vector transitions = st.getTransitions();
	Vector parameters = new Vector();
	
	// first parameter is a recipient of one of the sendEvents
	// that has the given convName
	String p1 = null;
	Vector sendEvents = t.getSendEvents();
	if(sendEvents != null)
	for(int i = 0; i < sendEvents.size(); i++){
		SendEvent se = (SendEvent)sendEvents.elementAt(i);
		// only look at send events that have the same convName
		if((se.getConversations() != null) && se.hasConvName(convName)){
			if(p1 == null){
				p1 = se.getRecipient();
				parameters.add(new Parameter(p1));
			}
			// check to see if any of the parameters in the event are needed
			Vector params = se.getEvent().getParameters();
			if(params != null)
			for(int j = 0; j < params.size(); j++){
				Parameter p = (Parameter)params.elementAt(j);
				if(!p.isPrepended() 
					&& isNeeded(p, t.getConversations(), st) 
					&& !containsParameter(parameters, p))
						parameters.add(p);
			}
		}
	}
	
	return parameters;
}
/**
 * This method will check to see if the parameter is used somewhere in the guard
 * condition.  Since guard conditions are loosely defined as a string, the method
 * simply checks to see if it is a token in the string.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
public static boolean inGuard(Parameter p, String guard) {
	String param = p.getName();
	StringTokenizer st = new StringTokenizer(guard, " ,=<>()");
	while(st.hasMoreTokens()){
		String token = st.nextToken();
		if(token.equals(param))
			return true;
	}
	
	// default
	return false;
}
/**
 * This method will determine if the parameter is needed
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean isNeeded(Parameter p, Vector convs, StateTable st) {
	Vector			transitions = st.getTransitions();
	Transition 		t;
	Vector			sendEvents;
	Vector			states = st.getStates();
	State			s;
				
	// check all states
	if(states != null)
	for(int k = 0; k < states.size(); k++){
		s = (State)states.elementAt(k);
		Vector actions = s.getActions();
		if(!s.hasAllConversations(convs) && (actions != null)){
			// check all actions in the state
			for(int l = 0; l < actions.size(); l++){
				Action a = (Action)actions.elementAt(l);
				if(usedInAction(p, a))
					return true;
			}
		}
	}

	// check all transitions
	for(int k = 0; k < transitions.size(); k++){
		Transition t2 = (Transition)transitions.elementAt(k);
		if(!t2.hasAllConversations(convs) && usedInTransition(p, t2))
			return true;
	}					
				/*}
			}
		}
	}*/

	// default
	return false;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
/**
 * This method checks to see if the parameter is used somewhere in the action.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean usedInAction(Parameter p, Action a) {
	Vector lhs = a.getLhs();
	Vector rhs = a.getRhs();
	FunctionCall f = a.getFunction();
	
	
	if(lhs != null){
		for(int i = 0; i < lhs.size(); i++){
			String s = (String)lhs.elementAt(i);
			if(s.equals(p.getName()) 
				|| s.equals("parent." + p.getName())
				|| ("parent." + s).equals(p.getName()))
				return true;
		}
	}
	if(rhs != null){
		for(int i = 0; i < rhs.size(); i++){
			String s = (String)rhs.elementAt(i);
			if(s.equals(p.getName()) 
				|| s.equals("parent." + p.getName())
				|| ("parent." + s).equals(p.getName()))
				return true;
		}
	}
	if(f != null){
		Vector parameters = f.getParameters();
		if(parameters != null)
		for(int i = 0; i < parameters.size(); i++){
			Parameter p2 = (Parameter)parameters.elementAt(i);
			if(p2.getName().equals(p.getName()) 
				|| p2.getName().equals("parent." + p.getName())
				|| ("parent." + p2.getName()).equals(p.getName()))
				return true;
		}
	}
		
	// default
	return false;
}
/**
 * This method checks to see if the parameter is used somewhere in the transition.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
boolean usedInTransition(Parameter p, Transition t) {
	Event receive = t.getReceive();
	if(receive != null){
		if(receive.hasParameter(p))
		return true;
	}
	if(t.getReceiveEvent() != null){
		ReceiveEvent re = t.getReceiveEvent();
		Event e = re.getEvent();
		if(e.hasParameter(p))
		return true;
	}
	if((t.getGuard() != null) && inGuard(p, t.getGuard())){
		return true;		
	}
	Vector sends = t.getSends();
	if(sends != null){
		for(int x = 0; x < sends.size(); x++){
			Event e = (Event)sends.elementAt(x);
			if(e.hasParameter(p))
			return true;
		}
	}
	Vector sendEvents = t.getSendEvents();
	if(sendEvents != null){
		for(int x = 0; x < sendEvents.size(); x++){
			SendEvent se = (SendEvent)sendEvents.elementAt(x);
			Event e = se.getEvent();
			if(e.hasParameter(p))
			return true;
		}
		
	}
	Vector actions = t.getActions();
	if(actions != null){
		for(int i = 0; i < actions.size(); i++){
			Action a = (Action)actions.elementAt(i);
			if(usedInAction(p, a))
				return true;
		}
	}
		
	// default
	return false;
}
}
