package Transforms;

/**
 * This class represents transformation 49 and 50 in the transformation process.
 * The result of the transform is that transitions in component state tables that
 * belong to conversations will be placed in the conversation halves and removed
 * from the component.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import system.ATtransition;
import system.ATtableElement;
import system.ATstatetable;

public class Transform49 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform49(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will determine which conversation half of a conversation the
 * component has.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
ConversationHalf determineConvHalf(Conversation conv, Component c) {
	if(c.getConvs().contains(conv.getInitiator()))
		return conv.getInitiator();
	else if(c.getConvs().contains(conv.getResponder()))
		return conv.getResponder();
	else{
		System.out.println("Error in T41");
		return null;
	}
}
/**
 * This method will add transitions to the appropriate conversation halves and remove them
 * from the component state tables.
 *
 * the params attribute is not used
 */
public void execute(Object params) {	
	Vector 			agents = sys.agents;
	Agent 			a;
	Vector			components;
	Component		c;
	StateTable		st;
	Vector			transitions;
	Transition		t, t2 = null;
	Conversation 	conv;
	ConversationHalf ch;

	if(agents != null)
	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				Vector conversations = t.getConversations();
				if(conversations != null){
					for(int l = 0; l < conversations.size(); l++){
						conv = (Conversation)conversations.elementAt(l);
						ch = determineConvHalf(conv, c);
						StateTable chStateTable = ch.getStateTable();
						if(t.getProtocols() != null){
							// make a copy of the complete transition
							t2 = t.copy();
						}
						else{		
							// make a copy of the transition with only the appropriate send Events
							t2 = t.copy();
							Vector sendEvents = t2.getSendEvents();
							Vector newSends = new Vector();
							if(sendEvents != null)
							for(int m = 0; m < sendEvents.size(); m++){
								SendEvent se = (SendEvent)sendEvents.elementAt(m);
								if((se.getConversations() != null) 
									&& se.getConversations().contains(conv))
										newSends.add(se);
							}
							t2.setSendEvents(newSends);
							t2.getAtTransition().setTransMessage(t2.getTransMessage());
						}					
						
						// add the transition to the state table
						if(chStateTable.getTransitions() == null)
							chStateTable.setTransitions(new Vector());
						chStateTable.getTransitions().add(t2);
						// add t2's ATtransition to the ATstatetable
						ATtransition atTransition = t2.getAtTransition();
						ATstatetable atStateTable = chStateTable.getAtStateTable();
						atStateTable.addTransition(atTransition);
						// match up the from and to states within the state table(s)
						matchUp(chStateTable);
					}
					// now remove the transition from the component's state table
					// first remove the transition from the atStateTable
					ATstatetable atStateTable = st.getAtStateTable();
					ATtransition atTransition = t.getAtTransition();
					atStateTable.removeTransition(atTransition);
					st.getTransitions().remove(t);		
					k--;	
				}				
			}
		}
	}	
}
/**
 * This method will match up transitions to and from states to the new states in the
 * conversation half state tables.
 *
 * Creation date: (11/26/00 3:14:16 PM)
 */
void matchUp(StateTable st) {
	// match up transition from and to states to the new states
	Vector transitions2 = st.getTransitions();
	Vector states2 = st.getStates();
	if(transitions2 != null)
	for(int i = 0; i < transitions2.size(); i++){
		Transition t = (Transition)transitions2.elementAt(i);
		ATtransition atTrans = t.getAtTransition();
		ATtableElement from = null;
		ATtableElement to = null;		
		for(int j = 0; j < states2.size(); j++){
			State s = (State)states2.elementAt(j);
			if(s.getName().equals(t.getFromName())){
				t.setFrom(s);
				// set from to the ATtableElement of s
				from = s.getElement();
			}
			if(s.getName().equals(t.getToName())){
				t.setTo(s);
				// set to to the ATtableElement of s
				to = s.getElement();
			}
		}

		if(from == null){
			for(int j = 0; j < states2.size(); j++){
				State s = (State)states2.elementAt(j);
				if(s.getName().equals("StartState")){
					t.setFrom(s);
					from = s.getElement();
				}
			}
		}

		if(to == null){
			for(int j = 0; j < states2.size(); j++){
				State s = (State)states2.elementAt(j);
				if(s.getName().equals("EndState")){
					t.setTo(s);
					to = s.getElement();
				}
			}
		}
		
		// now set the from and to of the ATtransitions
		if(from != null)
			atTrans.setCurrentState(from);
		if(to != null)
			atTrans.setNextState(to);
		if((from != null) && (to != null))
			atTrans.setCoordinates(from, to);
	}
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
