package Transforms;

/**
 * This class represents transformation 5 in the transformation process
 * The result of this transformation is that the protocols in the analysis
 * phase between the tasks will be replicated in the design phase between
 * the components that were created from the tasks.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;

public class Transform5 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform5(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method actually creates the new Protocol between the components
 *
 * Creation date: (11/10/00 2:37:18 PM)
 */
public Protocol createProtocol(Protocol p, Task t, Task t2, Component c, Component c2) {
	Protocol p2 = new Protocol(t, t2);
	p2.setName(p.getName());
	p2.setMode(p.getMode());
	p2.setInitComp(c);
	p2.setRespComp(c2);

	return p2;
	// don't need to create an ATProtocol for this - not used in agentTool
	// might need to create component connection for the internal ones
}
/**
 * This method will create new protocols in the design that
 * correspond to the tasks that the components were created from
 *
 * the params attribute is not used
 */
public void execute(Object params) {
	Vector	protocols = sys.protocols;
	Vector 	roles = sys.roles;
	Vector	agents = sys.agents;

	Task	t, t2;
	Role	r, r2;
	
	// enumerate through the protocols to find where they
	// should be duplicated in the design phase
	if(protocols != null) {
	int numProtocols = protocols.size();
	for(int i = 0; i < numProtocols; i++){
		Protocol p = (Protocol)protocols.elementAt(i);
		t = p.getInitiator();
		t2 = p.getResponder();
		
		// find the roles that have the tasks
		for(int j = 0; j < roles.size(); j++){
			Role role = (Role)roles.elementAt(j);
			Vector tasks = role.getTasks();
			if((tasks != null) && tasks.contains(t))
				r = role;
			if((tasks != null) && tasks.contains(t2))
				r2 = role;
		}
		
		// now find the agents that play the roles	
		for(int	k = 0; k < agents.size(); k++){
			Agent a = (Agent)agents.elementAt(k);
			Vector components = a.getComponents();
			for(int l = 0; l < components.size(); l++){
				Component c = (Component)components.elementAt(l);
				if(c.getName().equals(t.getName())){
					for(int m = 0; m < agents.size(); m++){
						Agent a2 = (Agent)agents.elementAt(m);
						Vector components2 = a2.getComponents();
						for(int n = 0; n < components2.size(); n++){
							Component c2 = (Component)components2.elementAt(n);
							if(c2.getName().equals(t2.getName())){
								// create the new protocol
								Protocol p2 = createProtocol(p, t, t2, c, c2);
								// add the protocol to the system vector
								sys.protocols.add(p2);
							}
						}
					}
				}
			}
		}
			
	}}
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
