package Transforms;

/**
 * This class represents transformation 7 in the transformation process.
 * The result of this transform is that external receive events in components will
 * now belong to the newly created protocols between the components, instead
 * of the protocols between the tasks.  
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;
import java.util.Enumeration;

public class Transform7 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform7(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will automatically change the protocols for send events 
 * in component state tables from protocols between tasks to the corresponding 
 * protocols between components in the design phase.
 *
 * the params attribute is not used
 */
public void execute(Object params) {
	Vector			agents = sys.agents;
	Agent			a;
	Vector			components;
	Component		c;
	StateTable		st;
	Vector			transitions;
	Transition		t;
	Vector 			sendEvents;
	SendEvent		se;
	Protocol		p, p2;

	for(int i = 0; i < agents.size(); i++){
		a = (Agent)agents.elementAt(i);
		components = a.getComponents();
		if(components != null)
		for(int j = 0; j < components.size(); j++){
			c = (Component)components.elementAt(j);
			st = c.getStateTable();
			transitions = st.getTransitions();
			if(transitions != null)
			for(int k = 0; k < transitions.size(); k++){
				t = (Transition)transitions.elementAt(k);
				sendEvents = t.getSendEvents();
				if(sendEvents != null)
				for(int x = 0; x < sendEvents.size(); x++){
					se = (SendEvent)sendEvents.elementAt(x);
					if(se.getProtocols() != null){
						Vector prots = (Vector)se.getProtocols().clone();
						Enumeration protocols = prots.elements();
						while(protocols.hasMoreElements()){
							p = (Protocol)protocols.nextElement();
							p2 = findMatchingProtocol(p, c);
							se.getProtocols().remove(p);
							se.getProtocols().add(p2);
						}
					}
				}
			}
		}	
	}
}
/**
 * This method finds and returns the Protocol in the design phase
 * that was replicated from the given protocol and component
 *
 * Creation date: (11/15/00 4:05:41 PM)
 */
public Protocol findMatchingProtocol(Protocol p, Component c) {
	Task t1, 		t2;
	Protocol 		p2;
	Vector 			protocols = sys.protocols;

	for(int i = 0; i < protocols.size(); i++){
		p2 = (Protocol)protocols.elementAt(i);
		if((p != p2)
			&& (p.getName().equals(p2.getName()))
			&& (p.getInitiator() == p2.getInitiator())
			&& (p.getResponder() == p2.getResponder())
			&& ((p2.getInitComp() == c) || (p2.getRespComp() == c))){
				return p2;
			}
	}

	System.out.println();
	System.out.println("Couldn't find matching protocol for:");
	p.show();
	return null;
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
