package Transforms;

/**
 * This class represents transformations 8 and 9 in the transformation process.
 * The result of this transform is that any events that belong to all internal
 * protocols will be changed into an internal event.
 *
 * Creation date: (11/8/00 10:37:49 AM)
 * @author: Clint Sparkman
 *
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
 
import java.util.Vector;

public class Transform8 extends Transform {
	TSystem	currentSys = null;	// A pointer to the system being transformed
	TSystem beforeSys = null; 	// A copy of the system before the transformation
	TSystem sys = null;			// A copy of the system that is used to work on
/**
 * Transform1 constructor comment.
 */
public Transform8(TSystem t) {
	super();

	sys = t;
	
	if(applicable(sys, null)){
		execute(null);
	}
}
/**
 * This method should be overridden by the concrete subclass
 * to determine if the transform is applicable to the target.
 *
 * Creation date: (11/6/00 8:52:43 AM)
 */
public static boolean applicable(Object target, Object params) {
	TSystem t;
	if(target instanceof TSystem){
		t = (TSystem)target;
		if((t.roles != null)
			&& (t.tasks != null)
			&& (t.protocols != null)
			&& (t.agents != null)
			&& t.rolesCovered()
			&& t.noDuplicateTasks()){	
				return true;
			}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}
/**
 * This method will change all external events in Component state tables 
 * that belong to internal protocols into the corresponding internal events
 *
 * the params attribute is not used
 */
public void execute(Object params) {
	Vector			protocols = sys.protocols;
	Vector			agents = sys.agents;
	Vector			transitions;
	
	Component 		c;
	Protocol 		p;
	StateTable		st;
	Transition 		t;
	SendEvent		se;
	ReceiveEvent	re;

	for(int i = 0; i < protocols.size(); i++){
		p = (Protocol)protocols.elementAt(i);
		
		if((p.getMode() == Protocol.internal) 
			&& (p.getInitComp() != null) 
			&& (p.getRespComp() != null)){

				// change all of them in the initiator component's state table
				c = p.getInitComp();
				st = c.getStateTable();
				transitions = st.getTransitions();
				if(transitions != null)
				for(int j = 0; j < transitions.size(); j++){
					t = (Transition)transitions.elementAt(j);
					// change the receiveEvent to the receive Event
					re = t.getReceiveEvent();
					if((re != null) && (re.getProtocols() != null) && (re.getProtocols().contains(p))){
						t.setReceiveEvent(null);
						t.setReceive(re.getEvent());
						t.getAtTransition().setRecMessage(t.getRecMessage());
					}
					// change all of the sendEvents to send Events
					Vector sendEvents = t.getSendEvents();
	
					if(sendEvents != null){
						Vector tempSends = (Vector)sendEvents.clone();
						for(int k = 0; k < tempSends.size(); k++){
							se = (SendEvent)tempSends.elementAt(k);
							if((se != null) && (se.getProtocols() != null) && (se.getProtocols().contains(p))){
								if(t.getSends() == null)
									t.setSends(new Vector());
								t.getSendEvents().removeElementAt(k);
								if(t.getSendEvents().size() == 0)
									t.setSendEvents(null);
								t.getSends().add(se.getEvent());
								t.getAtTransition().setTransMessage(t.getTransMessage());
							}
						}
					}
				}
				// change all of them in the responder component's state table
				c = p.getRespComp();
				st = c.getStateTable();
				transitions = st.getTransitions();
				if(transitions != null)
				for(int j = 0; j < transitions.size(); j++){
					t = (Transition)transitions.elementAt(j);
					// change the receiveEvent to the receive Event
					re = t.getReceiveEvent();
					if((re != null) && (re.getProtocols() != null) && (re.getProtocols().contains(p))){
						t.setReceiveEvent(null);
						t.setReceive(re.getEvent());
						t.getAtTransition().setRecMessage(t.getRecMessage());
					}
					// change all of the sendEvents to send Events
					Vector sendEvents = t.getSendEvents();
					if(sendEvents != null){
						Vector tempSends = (Vector)sendEvents.clone();
						for(int k = 0; k < tempSends.size(); k++){
							se = (SendEvent)tempSends.elementAt(k);
							if((se != null) && (se.getProtocols() != null) && (se.getProtocols().contains(p))){
								if(t.getSends() == null)
									t.setSends(new Vector());
								t.getSendEvents().removeElementAt(k);
								if(t.getSendEvents().size() == 0)
									t.setSendEvents(null);
								t.getSends().add(se.getEvent());
								t.getAtTransition().setTransMessage(t.getTransMessage());
							}
						}
					}
				}
			}
	}	
}
/**
 * replay method comment.
 */
public void replay(Object top) {}
/**
 * unDo method comment.
 */
public void unDo(Object params) {
	currentSys = beforeSys;
}
}
