package Verify;

/**
 * A Class for getting message sequence specifications from a file, <tt>msctable.txt</tt>
 * or from a vector that is passed when running the VMSCRun.verify(Vector, Vector) method
 * @version 1.0 6 Oct 1999
 * @author Capt Tim Lacey
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import java.util.*;
class MSCTable
{
	private String convName = null;
	private String convInit = null;
	private String convResp = null;
	private String message = null;
/**
 * Constructor that reads a line of the msctable file, <tt>msctable.txt</tt>
 * or a line of the vector passed by agentTool
 * @param line the message sequence chart
 * @return nothing
 * @throws nothing
 */
public MSCTable(String line)
{
	// Each entry in a line of the message sequence chart is separated by a semicolon
	StringTokenizer t = new StringTokenizer(line, ";"); 
	convName = t.nextToken();
	convInit = t.nextToken();
	convResp = t.nextToken();
	message = t.nextToken();
}
/**
 * Get the value of the conversation initiator
 * @param nothing
 * @return the conversation initiator
 * @throws nothing
 */
public String getConvInit()
{
	return convInit;
}
/**
 * Get the value of the conversation name
 * @param nothing
 * @return the conversation name
 * @throws nothing
 */
public String getConvName()
{
	return convName;
}
/**
 * Get the value of the conversation respondent
 * @param nothing
 * @return the conversation respondent
 * @throws nothing
 */
public String getConvResp()
{
	return convResp;
}
/**
 * Get the value of the message in the sequence chart trace
 * @param nothing
 * @return the message in the trace
 * @throws nothing
 */
public String getMessage()
{
	return message;
}
/**
 * Set the value of the conversation initiator
 * @param the value of the initiator
 * @return nothing
 * @throws nothing
 */
public void setConvInit(String n)
{
	convInit = n;
}
/**
 * Set the value of the conversation name
 * @param the conversation name for a particular message
 * @return nothing
 * @throws nothing
 */
public void setConvName(String n)
{
	convName = n;
}
/**
 * Set the value of the conversation respondent
 * @param the conversation respondent
 * @return nothing
 * @throws nothing
 */
public void setConvResp(String n)
{
	convResp = n;
}
/**
 * Set the value of the message in the sequence chart trace
 * @param the message in the trace
 * @return nothing
 * @throws nothing
 */
public void setMessage(String s)
{
	message = s;
}
}
