package Verify;

/**
 * A Class for getting each line of the state table from either a file, <tt>statetbl.txt</tt>
 * or a vector passed by agentTool
 * @version 1.0 6 Oct 1999
 * @author Capt Tim Lacey
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import java.util.*;
class StateTable
{
	private String convName = null;
	private String partName = null;
	private String currentState = null;
	private String recMessage = null;
	private String guardCondition = null;
	private String transMessage = null;
	private String nextState = null;
/**
 * Constructor that reads a line of the state table file, <tt>statetbl.txt</tt>,
 * or a vector passed by agentTool
 * @param line a line of the state table.  
 * @return nothing
 * @throws nothing
 */
public StateTable(String line)
{
	// Each element is delimited by a semicolon
	StringTokenizer t = new StringTokenizer(line, ";");
	String tconvName = t.nextToken();
	tconvName = cleanUp(tconvName);
	convName = CleanupString.convert(tconvName);
	String tpartName = t.nextToken();
	tpartName = cleanUp(tpartName);
	partName = CleanupString.convert(tpartName);
	String tcurrentState = t.nextToken();
	tcurrentState = cleanUp(tcurrentState);
	currentState = CleanupString.convert(tcurrentState);
	String trecMessage = t.nextToken();
	trecMessage = cleanUp(trecMessage);
	recMessage = CleanupString.convert(trecMessage);
	String tguardCondition = t.nextToken();
	tguardCondition = cleanUp(tguardCondition);
	guardCondition = CleanupString.convert(tguardCondition);
	String ttransMessage = t.nextToken();
	ttransMessage = cleanUp(ttransMessage);
	transMessage = CleanupString.convert(ttransMessage);
	String tnextState = t.nextToken();
	tnextState = cleanUp(tnextState);
	nextState = CleanupString.convert(tnextState);
}
/**
 * Strip the parenthesis from Promela code
 * @param the input from the state table
 * @return stripped string for state table processing
 * @throws nothing
 */
public String cleanUp(String s)
{
	// Return this variable with stripped parenthesis
	String newS = null;
	// Look for an open parenthesis
	StringTokenizer t = new StringTokenizer(s, "(");
	// If there is an open parenthesis...
	if (t.hasMoreTokens())
	{
		// Get the text up to the open parenthesis
		newS = t.nextToken();
	}
	// Else there are no parameters in this message
	else
	{
		// Copy the input to string to the output string
		newS = s;
	}
	// Return the corrected string variable
	return newS;
}
/**
 * Get the value of the conversation name
 * @param nothing
 * @return the conversation name
 * @throws nothing
 */
public String getConvName()
{
	return convName;
}
/**
 * Get the value of the current state in a conversation
 * @param nothing
 * @return the current state name
 * @throws nothing
 */
public String getCurrentState()
{
	return currentState;
}
/**
 * Get the value of the guard condition
 * @param nothing
 * @return the guard condition
 * @throws nothing
 */
public String getGuardCondition()
{
	return guardCondition;
}
/**
 * Get the value of the next state in a conversation
 * @param nothing
 * @return the next state name
 * @throws nothing
 */
public String getNextState()
{
	return nextState;
}
/**
 * Get the value of the participant name in a conversation
 * @param nothing
 * @return the participant name
 * @throws nothing
 */
public String getPartName()
{
	return partName;
}
/**
 * Get the value of the received message in a conversation
 * @param nothing
 * @return the received message
 * @throws nothing
 */
public String getRecMessage()
{
	return recMessage;
}
/**
 * Get the value of the transmitted message in a conversation
 * @param nothing
 * @return the transmitted message
 * @throws nothing
 */
public String getTransMessage()
{
	return transMessage;
}
/**
 * Set the value of the conversation name
 * @param the conversation name
 * @return nothing
 * @throws nothing
 */
public void setConvName(String n)
{
	convName = n;
}
/**
 * Set the value of the current state in a conversation
 * @param the current state name
 * @return nothing
 * @throws nothing
 */
public void setCurrentState(String s)
{
	currentState = s;
}
/**
 * Set the value of the guard condition
 * @param the guard condition
 * @return nothing
 * @throws nothing
 */
public void setGuardCondition(String g)
{
	guardCondition = g;
}
/**
 * Set the value of the next state in a conversation
 * @param the next state name
 * @return nothing
 * @throws nothing
 */
public void setNextState(String s)
{
	nextState = s;
}
/**
 * Set the value of the participant name in a conversation
 * @param the participant name
 * @return nothing
 * @throws nothing
 */
public void setPartName(String n)
{
	partName = n;
}
/**
 * Set the value of the received message in a conversation
 * @param the received message
 * @return nothing
 * @throws nothing
 */
public void setRecMessage(String m)
{
	recMessage = m;
}
/**
 * Set the value of the transmitted message in a conversation
 * @param the transmitted message
 * @return nothing
 * @throws nothing
 */
public void setTransMessage(String m)
{
	transMessage = m;
}
}
