package afit.mom;

/**
 * AFIT agentMom - version 1.1
 *
 * 4 October 1999
 * 12 May 2000 - added nonblockReadMessage method
 *
 * Abstract Conversation class.
 */
 
import java.net.*;
import java.io.*;

public abstract class Conversation extends Thread {
	protected Agent parent;
	protected Socket connection;
	protected ObjectInputStream input;
	protected ObjectOutputStream output;
	protected Message m;
	protected String connectionHost;
	protected int connectionPort;
/**
 * Default Conversation constructor.
 * @param a afit.mom.Agent
 */
public Conversation(Agent a) {
	super();
	parent = a;
}
/**
 * Default Conversation constructor.
 * @param a afit.mom.Agent
 * @param hostName java.lang.String
 * @param portNum int
 */
public Conversation(Agent a, String hostName, int portNum) {
	super(a);
	parent = a;
	connectionHost = hostName;
	connectionPort = portNum;
}
/**
 * Server_Register constructor comment.
 * @param s java.io.Socket
 * @param i java.io.ObjectInputStream
 * @param o java.io.ObjectOutputStream
 * @param a afit.mom.Agent
 * @param m1 afit.mom.Message
 */
public Conversation(Socket s, ObjectInputStream i, ObjectOutputStream o, Agent a, Message m1)
{
	super(a);
	parent = a;
	connection = s;
	input = i;
	output = o;
	m = m1;
}
/**
 * This method was created in VisualAge.
 *
 * This read method allows the read to "timeout" thus allowing
 * the conversation to check to see if it has a message without
 * waiting forever!!! 12 May 2000
 *
 * @param input java.io.ObjectInputStream
 * @return afit.at.mom.Message
 */
public Message nonblockedReadMessage(ObjectInputStream input)
{
	Message m = null;
	try
	{
		// set timeout for reading a message
		connection.setSoTimeout(0100);
		m = (Message) input.readObject();

	//
	} catch (InterruptedIOException iiex)
	{
		System.out.println(" ** Interrupted IO Exception (timeout) on READ!!!!");
	} catch (ClassNotFoundException cnfex)
	{
		System.out.println(" ** ClassNotFoundException ");
	} catch (IOException cnfex)
	{
		System.out.println(" ** IOException on port " + connectionPort);
	}
	return m;
}
/**
 * This method was created in VisualAge. - blocked read!
 * @param input java.io.ObjectInputStream
 * @return afit.at.mom.Message
 */
public Message readMessage(ObjectInputStream input)
{
	Message m = null;
	try
	{
		m = (Message) input.readObject();
	} catch (InterruptedIOException iiex)
	{
		System.out.println(" ** Interrupted IO Exception on READ!!!!");
	} catch (ClassNotFoundException cnfex)
	{
		System.out.println(" ** ClassNotFoundException ");
	} catch (IOException cnfex)
	{
		System.out.println(" ** IOException on port " + connectionPort);
	}
	return m;
}
/**
 * receiveMessage method comment.
 */
public void receiveMessage(Message m)
{
}
/**
 * This method was created in VisualAge.
 */
public abstract void run();
/**
 * @param m1 afit.mom.Message
 * @param o java.io.ObjectOutputStream
 */
public void sendMessage(Message m, ObjectOutputStream output)
{
	try
	{
		m.sender = parent.name;
		System.out.println(" ** sender is " + parent.name);
		m.host = connection.getInetAddress().getLocalHost().getHostName();
		System.out.println(" ** host is " + connection.getInetAddress().getLocalHost().getHostName());
		m.port = parent.port;
		System.out.println(" ** Sending message from " + parent.port);
		System.out.println(" ** Returning a " + m.getContent().getClass().getName());

		// don't timeout on outputs
		connection.setSoTimeout(0);
		output.writeObject((Object) m);
	} catch (IOException e)
	{
		System.out.println(" ** Exception: " + e);
	}
}
}
