package afit.mom;

/**
 * AFIT agentMom - version 1.0
 *
 * 4 October 1999
 *
 * agentMom MessageHandler class definition.
 */

import java.net.*;
import java.io.*;
import java.lang.*;

public class MessageHandler extends Thread {
	public int portNo;
	public ServerSocket server;
	public Agent parent;
	public String endToken = "END";
/*
 * MessageHandler constructor
 *
 * @param port int
 * @param p afit.mom.Agent
 */

public MessageHandler(int port, Agent p) {
	super("MessageHandler");
	parent = (Agent) p;
	if (port <= 1024)
		throw new IllegalArgumentException(" ** Bad port number given to Listener constructor for agent " + p.name);

	// Try making a ServerSocket to the given port
	try {
		server = new ServerSocket(port);
	} catch (IOException e) {
		System.out.println(" ** Agent " + p.name + " Failed to start ServerSocket on port " + port);
		System.exit(1);
	}

	// Made the connection, so set the local port number
	portNo = port;
	System.out.println("Agent " + p.name + " messageHandler Started on port " + port);
}
/**
 * Code to perform when this object is garbage collected.
 * 
 * Any exception thrown by a finalize method causes the finalization to
 * halt. But otherwise, it is ignored.
 */
protected void finalize() throws Throwable {
	// Insert code to finalize the receiver here.

	// This implementation simply forwards the message to super.  You may replace or supplement this.
	super.finalize();
}
/**
 * This method was created in VisualAge.
 */
public void run()
{
	Socket connection;
	int counter = 1;
	ObjectOutputStream output;
	ObjectInputStream input;
	System.out.println("Server Running on " + portNo);

	/* capture a connection, initialize input and output streams
	 * and pass the connection to the parent agent's receiveMessage
	 * message
	*/

	while (true)
	{
		try
		{
			connection = server.accept();
			System.out.println("Connection " + counter++ +" received from " + connection.getInetAddress().getHostName());
			output = new ObjectOutputStream(connection.getOutputStream());
			output.flush();
			input = new ObjectInputStream(connection.getInputStream());
			parent.receiveMessage(connection, input, output);
		} catch (InterruptedIOException iioex)
		{
			System.out.println(" ** Socket Server timeout on port " + portNo);
		} catch (IOException cnfex)
		{
			System.out.println(" ** IOException on port " + portNo);
		}
	}
}
}
