package system;

import java.util.*;
/**
 * This type was created in VisualAge.
 *
 * THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION IS RELEASED "AS IS."
 * THE U.S. GOVERNMENT MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * CONCERNING THIS SOFTWARE AND ANY ACCOMPANYING DOCUMENTATION, INCLUDING,
 * WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 * PARTICULAR PURPOSE.  IN NO EVENT WILL THE U.S. GOVERNMENT BE LIABLE
 * FOR ANY DAMAGES, INCLUDING ANY LOST PROFITS, LOST SAVINGS OR OTHER
 * INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE, OR 
 * INABILITY TO USE, THIS SOFTWARE OR ANY ACCOMPANYING DOCUMENTATION,
 * EVEN IF INFORMED IN ADVANCE OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 */
import java.util.Vector;
public class ATagentComponent extends ATobject {
	//vectors of attributes and methods
	private Vector attributes = new Vector();
	private Vector methods = new Vector();
	//basic if static, hasSub if this comp has a subArchitecture, sub
	//if this component is a sub-component
	private boolean basic;
	private boolean hasSub = false;
	private boolean sub;
	
	//state table and architecture for component
	private ATstatetable stateDiag;
	private ATarchitecture architecture;// = new ATarchitecture();
	// grfx info
	private ATagentComponentGRFX grfxObj;
	public boolean Selected;
	private static final String ukh = "comp@";
/**
 * ATagentComponent constructor comment.
 */
public ATagentComponent(double ver, String[] tokens, ATsystem sys)
{
	this(ver,0); //*** Version 1 of ATagent ***
	
	int loc = ATtokenOps.findnextinstanceof("version",tokens,0);
  	double v=Double.parseDouble(tokens[loc+3]);
	//setVersion(v);
	//architecture = new ATarchitecture(v);
	//stateDiag = new ATstatetable(v);

  //Code for constructing object model from MAML tokens
	if (v>=1) constructFromV1(tokens, sys);
	else {
		System.out.println("Loading a version other than 1.");	
	}
 	//	System.out.println("Loaded an agent");

}
/**
 * ATagentComponent constructor comment.
 */
public ATagentComponent(double v, int keynum)
{
	super(v);
	setKey(ukh + keynum);
	grfxObj   = new ATagentComponentGRFX();
	architecture = new ATarchitecture(v, keynum);
	stateDiag = new ATstatetable(v,keynum);
}
/**
 * ATagentComponent constructor comment.
 */
public ATagentComponent(double ver, String maml, ATsystem sys)
{
	this(ver,ATtokenOps.parseLine(maml),sys);
}
/**
 * ATagentComponent constructor comment.
 */
public ATagentComponent(ATagentComponent cp){
	super(cp);
}
public void addAttribute(ATagentComponentAttribute a)
{
	attributes.addElement(a);
	//set attributes for the graphics object
	grfxObj.setAttr(attributes);
	//resize accordingly
	grfxObj.resetMax();
}
public void addMethod(ATagentComponentMethod m)
{
	methods.addElement(m);
	//set methods of the graphics object
	grfxObj.setMethods(methods);
	//resize accordingly
	grfxObj.resetMax();
}
/**
 * Insert the method's description here.
 * Creation date: (11/18/99 2:41:27 PM)
 */
public void addState(ATstate s) {
	stateDiag.addState(s);
}
/**
 * Insert the method's description here.
 * Creation date: (11/18/99 3:11:20 PM)
 * @param t afit.system.ATtransition
 */
public void addTran(ATtransition t) {
	stateDiag.addTransition(t);
}
public boolean checkSelected(int x, int y) {
	//check to see if the point (x,y) is in the graphics object
	if(grfxObj.dist(x,y)) {
		return true;
	} else {
		return false;
	}
}
public int connectionCount() {
	//return the number of connections to this component
	return grfxObj.lineCount();
}
/**
 * Insert the method's description here.
 * Creation date: (10/25/99 11:28:36 PM)
 */
public void constructFromV1(String[] tokens, ATsystem sys) {
	    int end=0;
	    int loc = ATtokenOps.findnextinstanceof("version",tokens,0);
  		double v=Double.parseDouble(tokens[loc+3]);
		loc = ATtokenOps.findnextinstanceof("name",tokens,0);
  		setName(tokens[loc+3]);
  		if (tokens[loc+9].equals("true")) setBasic(true); 
		else setBasic(false);
		if (tokens[loc+15].equals("true")) setIsSub(true); 
		else setIsSub(false);
		if (tokens[loc+21].equals("true")) setHasSub(true) ;
		else setHasSub(false);		
		
		// parent is never specified so is always null.  I have removed
		// it's definition from this Object and from the MAML definitions.
		// It will simply be ignored where it appears in old MAML's.  -JAG (2-2-01)
		//parent = tokens[36];
		
		loc  = ATtokenOps.findnextinstanceof("key", tokens, loc);
		if(loc != -1) {
			setKey(tokens[loc+3]);
		} else {
			setKey(ukh + sys.getNextKey());
		}
		
		int start=ATtokenOps.findnextinstanceof("description", tokens, loc+3);
	    loc  = ATtokenOps.findnextinstanceof("/description", tokens, start+2);
		description=ATtokenOps.extract(tokens,(start+2), (loc-2));
		//Get agent coordinates for graphics display
		loc  = ATtokenOps.findnextinstanceof("view", tokens, loc+2);
		if (loc>0){ //KB-stored agents do not have view information
					//because they appear at a location the operator selects
			int x = Integer.parseInt(tokens[loc+5]);
			int y = Integer.parseInt(tokens[loc+11]);
			int h = Integer.parseInt(tokens[loc+17]);
			int w = Integer.parseInt(tokens[loc+23]);
			loc+=23;
			setCoordinates(x,y,h,w);
		}
		//Load State Information
		  start = ATtokenOps.findnextinstanceof("statetable", tokens, loc);
		  end   = ATtokenOps.findnextinstanceof("/statetable", tokens,loc);
		  if (start != end){
		    String[] submaml= ATtokenOps.subTokens(tokens, (start-1), (end+1));
	//	    ATagent i = getInitiator();		  //getowner??
		    stateDiag= new ATstatetable(v,submaml,sys); 
		  }
		for(int i=(loc+2);i<tokens.length;++i) {
	     if (tokens[i].equals("attribute")) {
		    loc=ATtokenOps.findnextinstanceof("/", tokens, i);
		    ATagentComponentAttribute m = new ATagentComponentAttribute(ATtokenOps.extract(tokens,(i-1), (loc+1)),sys);
		    addAttribute(m);
	   	 }
	    }
		for(int i=(loc+2);i<tokens.length;++i) {
	     if (tokens[i].equals("method")) {
		    loc=ATtokenOps.findnextinstanceof("/", tokens, i);
		    ATagentComponentMethod m = new ATagentComponentMethod(ATtokenOps.extract(tokens,(i-1), (loc+1)), sys);
		    addMethod(m);
	   	 }
	   }
		//Load component architecture knowledge/code
		start   = ATtokenOps.findnextinstanceof("architecture", tokens, end);
		//Need to locate "match" for this architecture, not for possible sub-architectures.
		if (start!=-1) {
			end= ATtokenOps.findmatch("architecture", tokens, start);	
			if (start != end) { //If a non-empty architecture  ELSE?...
				String[] submaml = ATtokenOps.subTokens(tokens, (start-1), (end+1));
				ATarchitecture a = new ATarchitecture(version, submaml, sys);
				setArchitecture(a);
			}
		}
}
/**
 * Insert the method's description here.
 * Creation date: (12/28/00 2:53:57 PM)
 */
public void deleteSubArchs() {
	if(gethasSub()) {
		//access the sub-ATagent panel from here; then go through all sub-agents with this function
		}
	architecture.removeAllConnections();
	architecture.removeAllComponents();
	}
public void draw(java.awt.Graphics g) {
	//draw graphics
	grfxObj.drawObject(g);
	//if selected, draw selected graphics
	if(Selected==true){
		grfxObj.drawSelected(g);
	}
}
public void drawDiag(java.awt.Graphics g) {
	//draw the state diagram for this component
	stateDiag.draw(g);
}
/**
 * Insert the method's description here.
 * Creation date: (11/29/00 11:48:08 AM)
 */
public void getAllNames() {
	System.out.println(name);	
	architecture.getAllNames();
}
/**
 * Insert the method's description here.
 * Creation date: (12/7/99 10:17:22 PM)
 * @return system.ATarch
 */
public ATarchitecture getArchitecture() {
	return architecture;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public Vector getAttributes() {
	return attributes;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public boolean getBasic() {
	return basic;
}
public ATagentComponentGRFX getGrfx() {
	return grfxObj;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public boolean gethasSub() {
	return hasSub;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public boolean getisSub() {
	return sub;
}
/**
 * This method was created in VisualAge.
 */
public String getMAML() throws Exception {
	//Opening line for an Component element
	MAML="";
	MAML+="\t\t\t<component";
	MAML+=" version=\"" + Double.toString(version) + "\"";
	MAML+=" name=\"" + name + "\"";
	MAML+=" isstatic=\""+basic+"\"";	
	MAML+=" ischild=\""+sub+"\"";
	MAML+=" isparent=\""+hasSub+"\"";

	// parent is never specified so is always null.  I have removed
	// it's definition from this Object and from the MAML definitions.
	// It will simply be ignored where it appears in old MAML's.  -JAG (2-2-01)
	//MAML+=" parentname=\""+parent+"\">\n";
	
	MAML += " key=\"" + getKey() + "\"\n";
	
	MAML+="\t\t\t\t<description>\n\t\t\t\t";
	MAML+=getDescription();
	MAML+="\n\t\t\t\t</description>\n";
	int[] c = grfxObj.getCoords();
	MAML+="\t\t\t\t<view";
	MAML+=" x=\""+c[0]+"\"";
	MAML+=" y=\""+c[1]+"\"";
	MAML+=" w=\""+c[2]+"\"";
	MAML+=" h=\""+c[3]+"\"/>\n";
	//StateTable
	MAML+=stateDiag.getMAML();
	//Attributes
	Enumeration as  = returnAttributes();
	while(as.hasMoreElements()){
		MAML+=((ATagentComponentAttribute)as.nextElement()).getMAML();
	}	
	//Methods
	Enumeration ms  = returnMethods();
	while(ms.hasMoreElements()){
		MAML+=((ATagentComponentMethod)ms.nextElement()).getMAML();
	}
	//Component architecture
	MAML+=architecture.getMAML();
	//End of encode sequence	
	MAML+="\t\t\t</component>\n";
	
	return MAML;	
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public Vector getMethods() {
	return methods;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public String getName() {
	return name;
}
/**
 * This method was created in VisualAge.
 * @return system.ATstatetable
 */
public ATstatetable getStateTable() {
	return stateDiag;
}
/**
 * Insert the method's description here.
 * Creation date: (11/28/00 12:45:23 PM)
 * @return int
 */
public int getSubComponentCount() {
	return architecture.getComponentCount();
}
/**
 * Insert the method's description here.
 * Creation date: (11/28/00 12:45:23 PM)
 * @return int
 */
public int getSubConnectionCount() {
	return architecture.getConnectionCount();
}
/**
 * Insert the method's description here.
 * Creation date: (10/28/99 10:31:27 AM)
 */
public void integrateKBInfo(String maml, ATsystem sys) {
  String[] tokens = ATtokenOps.parseLine(maml);
  int v=Integer.parseInt(tokens[6]);
	
  //Code for constructing object model from MAML tokens
  if (v==version) {
	constructFromV1(tokens, sys);
  }
  else {
	System.out.println("Loading from KB a version other than 1");	
  }	
}
public void move(int x, int y) {
	grfxObj.move(x,y);
}
/**
 * This method was created in VisualAge.
 */
public void print(java.awt.Graphics g) {
	grfxObj.paint(g);
}
public void removeAttribute(ATagentComponentAttribute a) {
	//remove a from the attributes
	attributes.removeElement(a);
	//resize graphics accordingly
	grfxObj.resetMax();
}
public void removeMethod(ATagentComponentMethod m) {
	//remove method m
	methods.removeElement(m);
	//resize graphics accordingly
	grfxObj.resetMax();
}
/**
 * Insert the method's description here.
 * Creation date: (11/28/00 1:12:31 PM)
 * @return boolean
 * @param n java.lang.String
 */
public boolean repeatName(String n) {
	if(name.equalsIgnoreCase(n)) return true;
	else return architecture.repeatName(n);
}
public void resize(int x, int y) {
	grfxObj.resize(x,y);
}
/**
 * returns an enumeration of the attributes of this component
 */
public java.util.Enumeration returnAttributes() {
	if (attributes == null) {
		return null;
	}
	java.util.Enumeration attributesEnum = attributes.elements();
	return attributesEnum;
}
/**
 * returns an enumeration of the methods of this componet
 */
public java.util.Enumeration returnMethods() {
	if (methods == null) {
		return null;
	}
	java.util.Enumeration MethodsEnum = methods.elements();
	return MethodsEnum;
}
/**
 * Insert the method's description here.
 * Creation date: (12/7/99 12:10:08 PM)
 */
public void setArchitecture(ATarchitecture a) {
  	architecture = a;	
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setAttributes(Vector a) {
	//set attributes to a
	attributes=a;
	//resize graphics accordinly
	grfxObj.resetMax();
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setBasic(boolean b) {
	basic = b;
}
public void setCoordinates(int x, int y, int width, int height) {
	grfxObj.place(x,y,width,height);
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setHasSub(boolean s) {
	if(hasSub && !s) {
		if(architecture.getComponentCount() > 0 || architecture.getConnectionCount() > 0) {
			System.out.println("\nComponent has defined subcomponents!  Cannot remove subArchitecture.");
		} else {
			hasSub = false;
		}
	} else {
		hasSub = s;
	}
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setIsSub(boolean s) {
	sub = s;
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setMethods(Vector m) {
	methods=m;
	grfxObj.resetMax();
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 */
public void setName(String s) {
	name = ATtokenOps.validName(s);
	grfxObj.setName(name);
	grfxObj.resetMax();
}
public void setStateTable(ATstatetable st) {
	stateDiag = st;
	}
}
