package system;

/**
 * ATtokenOps
 * this class provides several static functions that are used
 *   extensively throughout the AT program.  These used to be
 *   part of the Abstract ATobject class.  There was really no reason
 *   to propogate a copy of these functions into each derrived class
 *   since they are not overridden anywhere.  Thus I extracted them into
 *   this seperate utility class as public static methods.
 * Creation date: (10/17/00 2:43:43 PM)
 * @author: Jason Gunsch
 */

import java.util.StringTokenizer;
public class ATtokenOps {
/**
 * Insert the method's description here.
 * Creation date: (10/17/00 2:45:03 PM)
 * @return java.lang.String
 * @param tokens java.lang.String[]
 * @param start int
 * @param end int
 */
public static String extract(String[] tokens, int start, int end) {
	if(start<0) start = 0;
	if(end<0) end = 0;
	
	String extraction = "";
	for(int i= start; i<=end; ++i) {
		extraction += tokens[i];
	}

	return extraction;
}
/**
 * Insert the method's description here.
 * Creation date: (10/17/00 2:49:07 PM)
 * @return int
 * @param x java.lang.String
 * @param tokens java.lang.String[]
 * @param start int
 * @param end int
 */
public static int findlastinstanceof(String x, String[] tokens, int start, int end) {
	if(start < 0) start = 0;
	if(end < 0) end = 0;
	for(int i= end; i>start; --i) {
		if(tokens[i].equals(x)) {
			return i;
		}
	}
	return -1;
}
/**
 * Insert the method's description here.
 * Creation date: (10/17/00 2:50:43 PM)
 * @return int
 * @param x java.lang.String
 * @param tokens java.lang.String[]
 * @param start int
 */
public static int findmatch(String x, String[] tokens, int start) {
	String y = "/"+x;
	int j=0;
	if(start < 0) start = 0;
	for(int i= start+1; i<tokens.length; ++i) {
		if(tokens[i].equals(x)) ++j;
		if((tokens[i].equals(y)) & (j==0)) {
			return i;
		}
		else if (tokens[i].equals(y)) --j;
	}
	return -1;
}
/**
 * Insert the method's description here.
 * Creation date: (10/17/00 2:53:55 PM)
 * @return int
 * @param x java.lang.String
 * @param tokens java.lang.String[]
 * @param start int
 */
public static int findnextinstanceof(String x, String[] tokens, int start) {
	if(start < 0) start = 0;
	for(int i= start; i<tokens.length; ++i) {
		if(tokens[i].equals(x)) {
			return i;
		}
	}
	return -1;
}
/**
 * Insert the method's description here.
 * Creation date: (10/17/00 2:55:24 PM)
 * @return java.lang.String[]
 * @param s java.lang.String
 */
public static String[] parseLine(String s) {
	int len = s.length();
	int ct = 0;
	char[] val = s.toCharArray();
	char buf[] = new char[len];
	
	for(int i=0; i<len; i++) {
		char c = val[i];
		if(c != '\n' && c != '\r'  && c != '\t') {
			buf[ct] = c;
			ct++;
		}
	}
	String r = new String(buf,0,ct);
	
	String delimiters = "=<>\" ";  // tokenizing based on >,<,",=, and space.
	StringTokenizer tokens	= new StringTokenizer(r, delimiters, true);
	int n = tokens.countTokens();

	if(n>0) {
		boolean qflag = false;
		String x[] = new String[n];
		int i = 0;
		String t = null; // current token
		String qb = "";  // buffer for quoted characters

		while(tokens.hasMoreTokens()) {
			t = tokens.nextToken();
			if(t.equals("\"")) {	// encountering quotations
				if(qflag && !qb.equals("")) {
					x[i++] = qb;
					qb = "";
				}
				x[i++] = t;
				qflag = !qflag;
				
			} else if (!qflag) {
				x[i++] = t;			// Not inside quotes
			} else {
				qb += t;			// Inside quotes, buffering
			}
		}
		
		String y[] = new String[i];
		for(int j=0; j<i; j++) {
			y[j] = x[j];
		}
		
		return y;
	} else 
		return new String[0];
	
}
/**
 * Insert the method's description here.
 * Creation date: (10/17/00 2:55:24 PM)
 * @return java.lang.String[]
 * @param s java.lang.String
 */
public static String[] parseLine(String s, Editor.TextWindow display) {
	display.append("        - removing returns and tabs, tokenizing\n");
	
	int len = s.length();
	int ct = 0;
	char[] val = s.toCharArray();
	char buf[] = new char[len];
	
	for(int i=0; i<len; i++) {
		char c = val[i];
		if(c != '\n' && c != '\r' && c != '\t') {
			buf[ct] = c;
			ct++;
		}
	}
	String r = new String(buf,0,ct);
	
	String delimiters = "=<>\" ";  // tokenizing based on >,<,",=, and space.
	StringTokenizer tokens	= new StringTokenizer(r, delimiters, true);
	int n = tokens.countTokens();

	display.append("        - merging quoted content\n");

	if(n>0) {
		boolean qflag = false;
		String x[] = new String[n];
		int i = 0;
		String t = null; // current token
		String qb = "";  // buffer for quoted characters

		while(tokens.hasMoreTokens()) {
			t = tokens.nextToken();
			if(t.equals("\"")) {	// encountering quotations
				if(qflag && !qb.equals("")) {
					x[i++] = qb;
					qb = "";
				}
				x[i++] = t;
				qflag = !qflag;
				
			} else if (!qflag) {
				x[i++] = t;			// Not inside quotes
			} else {
				qb += t;			// Inside quotes, buffering
			}

		}

		String y[] = new String[i];
		for(int j=0; j<i; j++) {
			y[j] = x[j];
		}
		
		return y;
	} else 
		return new String[0];
	
}
/**
 * Insert the method's description here.
 * Creation date: (2/26/01 4:56:29 PM)
 * @return java.lang.String[]
 * @param s java.lang.String[]
 * @param start int
 * @param end int
 */
public static String[] subTokens(String[] tokens, int start, int end) {
	int len = end - start;
	
	if(start > 0 && len > 0) {
		String[] subtokens = new String[len];
		
		for(int i=0; i<len; i++) {
			subtokens[i] = tokens[start+i];
		}

		return subtokens;
	}
	return null;
}
/**
 * Insert the method's description here.
 * Creation date: (2/14/01 3:23:28 PM)
 * @return java.lang.String
 * @param n java.lang.String
 */
public static String validName(String n) {
	if(n == null) return null;
	if (n.equals("\"")) n = "";
	n = n.replace('\"', '\'');
	return n;
}
}
